#!/bin/sh
~iworx/bin/log.pex --message "Installing Let's Encrypt";
# Get OS version
OS_VER=`cat /etc/redhat-release | grep -o -E '[0-9]+' | head -1`

if [ $OS_VER -lt 6 ]; then
  ~iworx/bin/log.pex --message "Unsupported OS, exiting";
  exit 1
fi

# Install necessary repositories and python packages
if [ $OS_VER -eq 6 ]; then
  ~iworx/bin/log.pex --message "Installing EPEL Repo";
  rpm -ivh https://dl.fedoraproject.org/pub/epel/epel-release-latest-6.noarch.rpm > /dev/null 2>&1
  ~iworx/bin/log.pex --message $?
  if [ $? -ne 0 ]; then
    ~iworx/bin/log.pex --message "Required packages could not be installed, exiting";
    exit 2
  else
    ~iworx/bin/log.pex --message $?
  fi
fi

# Install git
~iworx/bin/log.pex --message "Installing Git"
yum -y install git > /dev/null 2>&1
~iworx/bin/log.pex --message $?

# Create clone directory for git
mkdir /home/interworx/lib/letsencrypt > /dev/null 2>&1

# Clone letsencrypt repository
~iworx/bin/log.pex --message "Cloning git repo";
git clone https://github.com/letsencrypt/letsencrypt /home/interworx/lib/letsencrypt > /dev/null 2>&1
~iworx/bin/log.pex --message $?

# checkout v1.9.0
pushd /home/interworx/lib/letsencrypt > /dev/null 2>&1
~iworx/bin/log.pex --message "Checking out version v1.9.0";
git checkout --force v1.9.0 > /dev/null 2>&1
~iworx/bin/log.pex --message $?
popd > /dev/null 2>&1

# Run LetsEncrypt initialization
if [ ! -e /etc/letsencrypt ]; then
  ~iworx/bin/log.pex --message "Running Let's Encrypt installer";
  /home/interworx/lib/letsencrypt/letsencrypt-auto --no-self-upgrade --register-unsafely-without-email --agree-tos > /dev/null 2>&1
  ~iworx/bin/log.pex --message $?
  ~iworx/bin/log.pex --message "Let's Encrypt apparently returns 1 as the norm";
  /home/interworx/lib/letsencrypt/letsencrypt-auto --no-self-upgrade --version > /dev/null 2>&1
fi
