#!/bin/sh

# You can only run this script once.  When it is run it will update this variable to TRUE
ALREADYRUN="FALSE"

# Determine the HTTP Port that InterWorx-CP is Listening on.  Default is 2080.  *Note, this is http, not https.
PORT="`grep '^Listen' /usr/local/interworx/etc/httpd/httpd-custom.conf | awk '{print $2}'`"

# Temporary directory
TMPDIR=/tmp

# httpd configuration directory
CONFDIR=/etc/httpd/conf.d

# localhost IP
IP=127.0.0.1

# check to see if the InterWorx-CP port is up an "listening"
PORTTEST="`nmap -p $PORT localhost | grep open | awk '{print $2}'`"

# If this port is closed or not accessible for some reason, this will have to be resolved before the script can continue.
if [ "$PORTTEST" != "open" ]; then
    echo "ERROR: InterWorx-CP Appears to not be listening on port $PORT.  Cannot continue."
    exit
fi

# check for already installed
SQEXISTS="`grep squirrel /usr/local/interworx/etc/httpd/httpd-custom.conf`"
if [ "$SQEXISTS" != "" ]; then
  exit;
fi

# Bail out if this script has already been run once.
if [ "$ALREADYRUN" == "TRUE" ]; then
    echo "You can only run this script once."
    exit
fi

# Make a copy of the iworx.conf file
cat $CONFDIR/iworx.conf > $TMPDIR/iworx.conf.tmp

# comment out the old /webmail rewrite rule
sed 's|RewriteRule ^/webmail|#RewriteRule ^/webmail|' < $TMPDIR/iworx.conf.tmp > $TMPDIR/iworx.conf.tmp2


# add in the new proxy settings
echo "


ProxyRequests Off

ProxyPass /webmail http://$IP:$PORT/webmail
ProxyPassReverse /webmail http://$IP:$PORT/webmail

ProxyPass /horde http://$IP:$PORT/horde
ProxyPassReverse /horde http://$IP:$PORT/horde

ProxyPass /squirrelmail http://$IP:$PORT/squirrelmail
ProxyPassReverse /squirrelmail http://$IP:$PORT/squirrelmail

" >> $TMPDIR/iworx.conf.tmp2

# replace the iworx.conf with the new one
mv $TMPDIR/iworx.conf.tmp2 $CONFDIR/iworx.conf

# Add the squirrelmail block to httpd-custom.conf
SQMAIL=`grep Squirrelmail /usr/local/interworx/etc/httpd/httpd-custom.conf`
if [ "$SQMAIL" == "" ]; then
    echo "

##
# Squirrelmail
##

Alias /squirrelmail  /usr/local/interworx/lib/squirrelmail/
<Directory /usr/local/interworx/lib/squirrelmail/>
  AllowOverride Authconfig Options
  Order allow,deny
  Allow from all
</Directory>

" >> /usr/local/interworx/etc/httpd/httpd-custom.conf
fi

# clean up tmp file
rm $TMPDIR/iworx.conf.tmp

# set the ALREADYRUN variable in this script to "TRUE"
sed 's|[F]ALSE|TRUE|' < $0 > $0.tmp
mv $0.tmp $0

# make sure the iworx-db link exists
if ! [ -a /usr/local/interworx/bin/iworx-db ]; then
    ln -s /usr/local/interworx/bin/mysql/mysqld /usr/local/interworx/bin/iworx-db
fi

#restart httpd and iworx services
service httpd restart
service iworx restart

echo "
Script Complete.
"
