# InterWorx Config System

The config system is accessible to InterWorx templates and customized
templates. This allows users to store and retrieve meta information
about SiteWorx accounts, individual domains, and resellers, as well
as system-wide config data either per-server or globally (see below).

The Config System is exposed to template authors via the `$Config` object,
available for use in any template. The `$Config` object is contextual, meaning
that if the template being rendered is applicable to a Domain, it will be the
Domain currently being rendered. If the template is only applicable globally, it
would instead be the Global config object. The interface remains the same, so
template authors don't really need to worry about it.

## Config Levels

InterWorx config values may be set at several different levels:

| Level    | Command Argument            | Description                             |
|----------|-----------------------------|-----------------------------------------|
| Global   | `--global`                  | Universal, applies to entire cluster    |
| Server   | `--server`                  | Applies to a single server in a cluster |
| Reseller | `--nodeworx $EMAIL_OR_ID`   | Applies to a single reseller account    |
| SiteWorx | `--siteworx $MASTER_DOMAIN` | Applies to a single SiteWorx account    |
| Domain   | `--domain $VHOST_DOMAIN`    | Applies to a single domain              |

## Config Inheritance

Config values are inherited from the top down, in this order:

    Global -> Server -> NodeWorx -> SiteWorx -> Domain

In a context where the Domain is being asked for a config value, if is not set for
the Domain explicitly, it will look for the config value in the Parent SiteWorx
account, then the Parent Reseller account, then the single Server, then the
whole-cluster configuration, before finally checking for the default value as defined
by InterWorx.

## `$Config->get('CONFIG_NAME')`

Config values are accessed in the template through the `$Config` object, specifically
through the function call `$Config->get()`. The config name must be specified.

Inside a template, a call to retrieve the value set for `SOMEVALUE` would look like:

    This is an example template.

    The value of 'SOMEVALUE' is {$Config->get('SOMEVALUE')}

## `$Config->include('template/name')`

More complex templates, such as the template for a vhost file for Apache, are composed
of multiple smaller templates. This allows customization of the individual pieces
without needing to override the whole thing.

## Viewing Config Values

The `--list-all` function on `~iworx/bin/config.pex` provides a way to see config
values available to a `$Config` object.

    ~iworx/bin/config.pex --list-all --domain example.com | column -t

    DOMAIN_PHP_VERSION        custom   /opt/remi/php72
    EXTERNAL_LB               default  0
    FPM_CATCH_WORKERS_OUTPUT  default  no
    FPM_MAX_CHILDREN          custom   4
    FPM_START_SERVERS         custom   2
    HTTPD_DOCUMENT_ROOT       default  /home/{$UNIX_USER}/{$DOMAIN}/html
    ... (output truncated)

In this example, you can see that the value `EXTERNAL_LB` is _available_ to the Domain
`$Config`, even though it's not particularly relevant. This is due to the inheritance
properties of the Config System.

Additionally, the value `HTTPD_DOCUMENT_ROOT` shows that some config values are
themselves treated as templates.

## Custom Config Names

Custom config name/value pairs may be set at the command line using:

    ~iworx/bin/config.pex --set --name $NAME --value '$VALUE' ...

In the example, `...` should be replaced with the correct argument for the config
level, as explained above.

API users may also set config values using
 - Ctrl_Nodeworx_Siteworx::setConfig
 - Ctrl_Siteworx_DomainsSlave::setConfig

See the [InterWorx API documentation](http://docs.interworx.com/interworx/api/) for
implementation details.

### Config Name Restrictions

Custom config names are subject to a few minor restrictions, to prevent conflicts later.
InterWorx reserves all names that consist of ALLCAPS. Custom config names MUST contain
at least one lowercase letter.

### Config Name Recommendations

InterWorx recommends that customer-created config names use all lowercase lettering
to stand out from InterWorx-created config names. Additionally, we recommend that
authors prefix config names with company initials, etc.  If InterWorx was to create
custom config value for an email address, for example, we would do something like
`iw_email` as the name.

## Notes for Clustering

With the exception of `--server` level configs, all config values are available to
the cluster as a whole. `--server` is the only level that will apply to an individual
server in a cluster node.

## Notes for SiteWorx Backups

A SiteWorx backup with contain all config values set for the SiteWorx account and all
domains owned by the account. These values will be restored on import.
