# Customizing Templates

Templates may be customized at the command line interactively, or created
programmatically through a seperate command.

To create a custom template interactively, use a command like the following:

    ~iworx/bin/config.pex --customize-template $TEMPLATE --for-global

> Note: interactive template customization relies on the `$EDITOR`
environment variable being set correctly.

The InterWorx Template System depends heavily on the InterWorx Config System
for storage and retrieval of metadata. See documentation [here](README-config.md)

## Template Levels

InterWorx templates may be customized at several different levels:

| Level        | Command Argument                    |
| ---          | ---                                 |
| Globally     | `--for-global`                      |
| Per-reseller | `--for-nodeworx nodeworx@email.com` |
| Per-SiteWorx | `--for-siteworx masterdomain.com`   |
| Per-Domain   | `--for-domain vhostdomain.com`      |

Not all templates can be customized at all levels. For example, the
template for the RoundCube webmail config file is only applicable
globally.

Each InterWorx template has a header detailing customization options
for itself. This includes the template name, seen as `$TEMPLATE` in the
example above, as well as which levels apply to the template file.

Directions are also included for generating new output files using
your newly customized template.

## Template Syntax

The templates are processed and rendered using the Smarty Templating system. For
syntax documentation, see the Smarty documentation here: https://www.smarty.net/

Though Smarty is used to render the templates, the InterWorx Template System
DOES NOT use Smarty's inheritance model of defined `{block}` sections.

## Template Inheritance

Templates are inherited from the top down - that is to say, a template
customized globally will apply to that template for a domain, unless
also customized at a more specific level. If the template for vhost files
has been customized globally, it will apply to all domains that do
not have that same template customized `--for-domain`, `--for-siteworx`,
or `--for-nodeworx`.

Put another way - when deciding what template to use for a Domain, InterWorx
will first check if the template is explicitly customized for that Domain. If
not, it will check for explicit customization for the Parent SiteWorx, the
Parent Reseller, and Globally, before deciding that it is not customized,
and instead use the default template provided by InterWorx.

Customized templates are inherited from the top down, in this order:

    Global -> NodeWorx -> SiteWorx -> Domain

## Template Hooks: Pre and Post

Every template and included template will look for the existance of special pre and
post hooks - extra templates to be included before or after rendering the current
template file.

Importantly, this allows for the addition of custom template logic WITHOUT overriding
the InterWorx provided logic. Approached this way, templates can be customized while
still getting the benefit of upstream changes - new features, bug fixes, etc.

To override the `http/vhost/cgi` template for a single domain, for example:

    ~iworx/bin/config.pex --customize-template http/vhost/cgi --for-domain example.com

If you want to instead utilize pre or post hooks, use something like:

    ~iworx/bin/config.pex --customize-template http/vhost/cgi-post --for-domain example.com

Adding `-pre` or `-post` as a suffix to the name of the template.

## Non-interactive Template Customization

For some users, there may be a need to programmatically create custom
templates, perhaps using Puppet or a similar server management tool.

To do so, it is strongly recommended you first create the file using
the `--make-custom-template` command, which will create the file in
the correct location with the correct permissions.

    ~iworx/bin/config.pex --make-custom-template http/vhost/cgi --for-domain example.com

A successful `--make-custom-template` command will output the path to
the created file as its only output. Invoking scripts should capture
this output to use as a target for copying or creating the new template.

## Notes for Clustering

In a clustered environment, only the templates on the Cluster Manager are relevant.
InterWorx syncronizes these files to the nodes, and local changes on the node will be
overwritten.

## Notes for SiteWorx Backups

A SiteWorx backup with contain all custom templates for the SiteWorx account and all
domains owned by the account. These templates will be restored on import.
