/*jslint undef: true, widget: true, browser: true, onevar: true, eqeqeq: true */
/*global YAHOO, iw_js_data, IW_AC_DATA, iw_xhr, iw_form_validate_handler, updateStatusClass, iw_lib, iw_ctrl_handler, iw_form_handler */

var iw_form_autocomplete_data_handler;
var iw_form_generate_password_data_handler;
var iw_form_label_handler;

/**
 * Main iw_form object.  Everything related to interworx forms and javascript goes here.
 */
var iw_form = {

  /**
   * Initialize the iw_form object and any interworx forms present on the page.
   */
    init: function () {
        var YUI = YAHOO.util,
        Dom = YUI.Dom,
        E = YUI.Event,
        shells = Dom.getElementsByClassName('iw-form-shell', 'div');

        if ( iw_js_data.iw_env && iw_js_data.iw_env.is_node === true ) {
            this.initLocalChangesForms(Dom, E);
        }

        E.delegate(document.body, 'click', this.executeActionLink, 'a.iw-link-action');
        E.delegate(document.body, 'click', this.executeActionLink, 'button.iw-link-action');

        this.initUnlimited();
        this.initTooltips(shells);
        this.initSubmits(shells);
        this.initEventHandlers(shells);
        this.initGroupClickHandlers(shells);
        this.initShowSelect();
        this.initRichTextEditors(Dom);
    },

  /**
   * Popups up the "Loading..." popup, before full form load.
   */
    showLoadingPopup: function () {
        iw_lib.popupModal(
            '<div class="center"><img class="iw-i-pending iw-i" width="16" height="16" src="/img/blank.gif"/></div>',
            '##LG_LOADING##'
        );
    },

  /**
   * A "handled" action link is clicked.
   *
   * Take over the click and xhr request the action.
   *
   * @param Event e
   */
    executeActionLink: function ( e ) {
        var ctrl, action, input, defaultCallback, href;
        YAHOO.util.Event.stopEvent(e);

        iw_form.showLoadingPopup();

        if ( this.getAttribute('data-iw-href') ) {
            href = this.getAttribute('data-iw-href');
        } else {
            href = this.href;
        }

        ctrl   = iw_form.getCtrlFromUrl(href);
        action = iw_form.getActionFromUrl(href);
        input  = iw_form.getInputFromUrl(href);

        defaultCallback = function ( r, success, args ) {
            iw_lib.popupModal(r.getReplyData().message, r.getReplyCode() + ': ' + r.getReplyText());
        };

        iw_ctrl_handler.execute(null, defaultCallback, ctrl, action, input);
    },

  /**
   * Given a URL, get the action=blah part from it.
   *
   * @param string url
   * @return string
   */
    getActionFromUrl: function ( url ) {
        var data = this.getQueryStringDataFromUrl(url);

        if ( data.action ) {
            return data.action;
        }
        return '';
    },

  /**
   * Get the query string data from the url (excluding the action).
   *
   * @param string url
   * @return array
   */
    getInputFromUrl: function ( url ) {
        var input = this.getQueryStringDataFromUrl(url);

        if ( input.action ) {
            delete input.action;
        }

        return input;
    },

  /**
   * Get the query string data from url url.
   *
   * @param string url
   * @return array
   */
    getQueryStringDataFromUrl: function ( url ) {
        var parts, pairs, pair, temp, output = {};

        parts = url.split('?');

        if ( parts[1] ) {
            pairs = parts[1].split('&');

            for (pair in pairs) {
                if ( pairs.hasOwnProperty(pair) ) {
                    pair = pairs[ pair ];
                    temp = pair.split('=');
                    output[ temp[0] ] = temp[1];
                }
            }
        }

        return output;
    },

  /**
   * Given a url, determine the interworx controller portion.
   *
   * @param string url
   * @return string
   */
    getCtrlFromUrl: function ( url ) {
        var parts, startPosition, endPosition;
        if ( url.substring(0, 1) === '/' ) {
            parts = url.split('?');
            return parts[0];
        } else if ( url.substring(0, 4).toLowerCase() === 'http' ) {
            startPosition = url.indexOf('worx/') - 5;
            endPosition   = url.indexOf('?') || url.length;
            return url.substring(startPosition, endPosition);
        }
    },

    getSubmitButton: function ( form ) {
        var name = form.attributes.getNamedItem('name').value,
        button = document.getElementById(name + '-submit');
        return button;
    },

  /**
   * Toggle the visibility of... something.
   *
   * @param Event e
   */
    toggleHideFlag: function ( e ) {
        var elements = YAHOO.util.Dom.getElementsByClassName('iw-hide-' + this.id, 'tr'),
        display;

        if ( this.checked === false ) {
            display = '';
        } else {
            display = 'none';
        }

        YAHOO.util.Dom.setStyle(elements, 'display', display);
    },

    setupLabelRefresh: function ( form ) {
        var input = {},
        _refresh;

        _refresh = function () {
            input.form  = form.iw_form_class_name.value;
            input.ctrl  = form.iw_ctrl_name.value;
            iw_xhr.getReply(iw_form_label_handler, input, form);
        };

        iw_xhr.subscribeTo(_refresh, 'addCommit');
        iw_xhr.subscribeTo(_refresh, 'deleteCommit');
    },

  /**
   * Hide a group.
   *
   * @param string group
   */
    _hideGroup: function ( group ) {
        YAHOO.util.Dom.setStyle(this._getGroupElements(group), 'display', 'none');
    },

  /**
   * Show a group.
   *
   * @param string group
   */
    _showGroup: function ( group ) {
        YAHOO.util.Dom.setStyle(this._getGroupElements(group), 'display', '');
    },

  /**
   * Get elements in a group.
   *
   * @param string group
   * @return array
   */
    _getGroupElements: function ( group ) {
        return YAHOO.util.Selector.query('tr[data-iw-shown=' + group + ']');
    },

  /**
   * Toggle a show select.
   *
   * @param Event e
   */
    toggleShowSelect: function ( e ) {
        var option = this.options[ this.selectedIndex ],
        opt_length = this.options.length,
        group, opt_group, i;

        group = option.getAttribute('data-iw-show');

        if ( group ) {
            iw_form._showGroup(group);
        }

        for (i=0; i<opt_length; ++i) {
            opt_group = this.options[i].getAttribute('data-iw-show');

            if ( opt_group && opt_group !== group ) {
                iw_form._hideGroup(opt_group);
            }
        }

        iw_form.restripeTable(YAHOO.util.Dom.getAncestorByTagName(this, 'form'));
    },

  /**
   * Toggle show...?
   *
   * @param Event e
   */
    toggleShow: function ( e ) {
        var group, elements, display, form, others, other_group, others_length, i;

        group = this.getAttribute('data-iw-show');

        if ( group && ( this.checked === true || this.selected === true ) ) {
            iw_form._showGroup(group);
        } else {
            iw_form._hideGroup(group);
        }

        form = YAHOO.util.Dom.getAncestorByTagName(this, 'form');

        if ( this.tagName.toUpperCase() === 'INPUT' && this.type === 'radio' ) {
            others = YAHOO.util.Dom.getElementsByClassName('in-' + this.name, 'input', form);
            others_length = others.length;

            for (i=0; i<others_length; ++i) {
                if ( others[i].id !== this.id ) {
                    other_group = others[i].getAttribute('data-iw-show');

                    if ( other_group ) {
                        iw_form._hideGroup(other_group);
                    }
                }
            }
        }

        iw_form.restripeTable(form);
    },

  /**
   * Toggle show flag.
   *
   * @param Event e
   */
    toggleShowFlag: function ( e ) {
        var elements = YAHOO.util.Dom.getElementsByClassName('iw-show-' + this.id, 'tr'),
        display;

        if ( this.checked === true ) {
            display = '';
        } else {
            display = 'none';
        }

        YAHOO.util.Dom.setStyle(elements, 'display', display);
    },

  /**
   * Init local changes on forms.  ?
   *
   * @param Dom.util?   Dom
   * @param Event.util? Eevnt
   */
    initLocalChangesForms: function ( Dom, Event ) {
        var checkboxes = Dom.getElementsByClassName('in-enable_local_changes', 'input'),
        checkbox_count = checkboxes.length,
        form, input, i, j;

        for (i=0; i<checkbox_count; i++) {
            form = Dom.getAncestorByTagName(checkboxes[ i ], 'form');

            this.disableForm(form, Dom, [ 'enable_local_changes' ]);
        }
    },

  /**
   * On enable local changes
   *
   * @param Event e
   */
    onEnableLocalChanges: function ( e ) {
        var form = YAHOO.util.Dom.getAncestorByTagName(this, 'form');
        iw_form.enableForm(form, YAHOO.util.Dom);
    },

    disableForm: function ( form, Dom, exceptions ) {
        var inputs = Dom.getElementsByClassName('iw-in', null, form),
        input_count = inputs.length,
        i;
        exceptions = exceptions || [];

        for (i=0; i<input_count; ++i) {
            if ( exceptions.indexOf(inputs[ i ].name) === -1 ) {
                inputs[ i ].disabled = 1;
            }
        }

        iw_form.getSubmitButton(form).disabled = 1;
    },

  /**
   * Enable form.
   *
   * @param HTMLElement form
   * @param Dom.util?   Dom
   */
    enableForm: function ( form, Dom ) {
        var inputs = Dom.getElementsByClassName('iw-in', null, form),
        input_count = inputs.length,
        i;

        for (i=0; i<input_count; ++i) {
            inputs[ i ].disabled = 0;
        }

        iw_form.getSubmitButton(form).disabled = 0;
    },

    richTextEditors: {},

    formSubmitStatusCache: {},

    initRichTextEditors: function ( Dom ) {
        var rtes = Dom.getElementsByClassName('iw-rte', 'textarea'), config,
        e,
        rte_count = rtes.length, i;

        if ( !rtes ) {
            return;
        }

        config = {
            height: '300px',
            width: '600px',
            dompath: true,
            focusAtStart: false
        };

        for (i=0; i<rte_count; ++i) {
            e = rtes[i];

            this.richTextEditors[ e.id ] = new YAHOO.widget.SimpleEditor(e.id, config);
            this.richTextEditors[ e.id ]._defaultToolbar.titlebar = false;
            this.richTextEditors[ e.id ].render();
        }
    },

  /**
   * Switch between file upload, and choose a local file.
   *
   * @param Event e
   */
    fileSwitch: function ( e ) {
        var up_lang = '##LG_SWITCH_TO_FILE_UPLOAD##',
        local_lang = '##LG_SWITCH_TO_FILE_LOCAL##',
        len = this.id.length,
        id = this.id.substr(0, len - 9),
        input = document.getElementById(id),
        input_name = input.name,
        alt_input = document.getElementById(id + '_alt'),
        alt_input_name = alt_input.name;
        YAHOO.util.Event.stopEvent(e);

        if ( this.innerHTML === up_lang ) {
            this.innerHTML = local_lang;
        } else {
            this.innerHTML = up_lang;
        }

        input.name  = alt_input.name;
        input.id    = alt_input.id;
        input.value = '';

        alt_input.name = input_name;
        alt_input.id   = id;
        var place_holder = YAHOO.util.Dom.getPreviousSibling(this);
        YAHOO.util.Dom.addClass(place_holder, 'iw-local-file');
        YAHOO.util.Dom.setStyle(input, 'display', 'none');
        YAHOO.util.Dom.setStyle(this, 'display', 'none');
        YAHOO.util.Dom.setStyle(alt_input, 'display', 'inline');
    },

  /**
   * Initialize the auto-complete logic.
   *
   * @param array input
   */
    initAutoComplete: function ( input ) {
        var Dom = YAHOO.util.Dom,
        form = YAHOO.util.Dom.getAncestorByTagName(input, 'form'),
        DS, AC, args;

      /*
       * The awesome bar needs to be instantiated differently, done elsewhere
       */
        if ( input.id === 'iw_awesome_input' ) {
            return;
        }

        if ( input.value === '' ) {
            input.value = '##LG_BEGIN_TYPING_FOR_SUGGESTIONS##';
            Dom.addClass(input, 'iw-dim');
        }

      /*
       * If the autocomplete data is missing, fetch it from the server.
       *
       * The callback that does the fetching will call this function again, which will
       * do the rest of the initialization.  We filled in the "begin typing" to look
       * like it's ready.
       */
        if ( IW_AC_DATA.hasOwnProperty(input.id) === false ) {
            args = iw_form.getHiddenInputValues(form);

            args.input = input.getAttribute('data-ac-name') || input.name;
            args.form  = form.iw_form_class_name.value;

            iw_xhr.getReply(iw_form_autocomplete_data_handler, args, input);
            return;
        }

        DS    = new YAHOO.util.LocalDataSource(IW_AC_DATA[ input.id ]);
        DS.responseSchema = { fields: [ "d", "v" ] };

        AC = new YAHOO.widget.AutoComplete(input, input.id + '_container', DS);
        AC.resultTypeList = false;
        AC.animHoriz      = false;
        AC.animVert       = false;
        if ( YAHOO.env.ua.ie > 0 ) {
            AC.useIFrame = true;
        }
        AC.useShadow = true;
      // We do this ourselves in this.onACSelect
        AC.suppressInputUpdate = true;

        if ( input.tagName === 'TEXTAREA' ) {
          // Set up stuff for multiple suggestions
            AC.delimChar = "\n";
            AC.containerExpandEvent.subscribe(this.onACExpand);
        }

        AC.itemSelectEvent.subscribe(this.onACSelect);

        if ( this.isSetAutoComplete(input) ) {
            AC.forceSelection = true;
            AC.itemSelectEvent.subscribe(this.onACSetSelect);
        }

        this.registeredACs[ input.id ] = AC;
    },

  /**
   * On Auto-complete expand.
   *
   * @param string? sType
   * @param array   args
   */
    onACExpand: function ( sType, args ) {
        var AC = args[0],
        data = args[2],
        input = AC.getInputEl(),
        height = input.clientHeight,
        container = AC.getContainerEl();
        YAHOO.util.Dom.setStyle(container, 'top', height + 'px');

    },

    isSetAutoComplete: function ( input ) {
        var td = YAHOO.util.Dom.getAncestorByTagName(input, 'td'),
        divs = td.getElementsByTagName('div'),
        divs_length = divs.length, i;

        for (i=0; i<divs_length; ++i) {
            if ( YAHOO.util.Dom.hasClass(divs[ i ], 'iw-form-ac-check') ) {
                return true;
            }
        }
        return false;
    },

    registeredACs: {},

    clearAutocompletes: function () {
        var YUI = YAHOO.util,
        Dom = YUI.Dom,
        autocompletes = Dom.getElementsByClassName('iw-form-ac', 'input'),
        textcompletes = Dom.getElementsByClassName('iw-form-ac', 'textarea'),
        autocompletes = autocompletes.concat(textcompletes),
        ac_length = autocompletes.length, i, input;

        for (i = 0; i < ac_length; ++i) {
            input = autocompletes[i];
            if ( input.value === '##LG_BEGIN_TYPING_FOR_SUGGESTIONS##' ) {
                input.value = '';
            }
        }

    },

    onACFocus: function () {
        if ( this.value === '##LG_BEGIN_TYPING_FOR_SUGGESTIONS##' ) {
            this.value = '';
        }
        YAHOO.util.Dom.removeClass(this, 'iw-dim');
    },

    onACSelect: function ( sType, args ) {
        var AC = args[0],
        data = args[2],
        input = AC.getInputEl(),
        old_val = input.value.split("\n"),
        range;
        iw_form.clearStatus(input);

      // Remove the partial match
        old_val.pop();

        old_val.push(data.v);

        input.value = old_val.join("\n") + "\n";

      // Move the caret for IE
        if ( input.createTextRange ) {
            range = input.createTextRange();
            range.collapse(false);
            range.select();
        }
    },

    isInPopup: function ( el ) {
      // Decidedly iffy :)
        if ( el.id.substr(-2) === '_j' ) {
            return true;
        }
      // TODO: should probably also walk back the DOM to see the popup itself.
        return false;
    },

    onACSetSelect: function ( sType, args ) {
        var AC = args[0],
        data = args[2],
        input = AC.getInputEl(),
        check_name = iw_form.getInputNameFromACName(input),
        id_prefix = ( iw_form.isInPopup(input) ) ? check_name + '_j' : check_name ,
        table = document.getElementById(id_prefix + '_table'),
        table_body = table.getElementsByTagName('tbody')[0],
        rows = table.getElementsByTagName('tr'),
        row_count = rows.length,
        id = check_name + '_' + row_count,
        value_input = iw_form.valueAlreadyExists(data.v, table),
        new_row = '', input_string, label_string, oRow, oCellInput, oLabelInput;

        input.value = '';

        if ( value_input ) {
            value_input.checked = 1;
            return;
        }

        input_string = '<input type="checkbox" checked="checked" ' + 'id="' + id +
        '" value="' + data.v + '" name="' + check_name + '[]"/>';

        label_string = '<label for="' + id + '">' + data.d + ' - ##LG_UNSAVED##' + '</label>';

        if ( table_body.insertRow ) {
            oRow        = table_body.insertRow(-1);
            oCellInput  = oRow.insertCell(-1);
            oLabelInput = oRow.insertCell(-1);

            oCellInput.innerHTML  = input_string;
            oLabelInput.innerHTML = label_string;

            YAHOO.util.Dom.addClass(oRow, 'iw-form-unsaved');
        } else {
          // This is for browsers that don't support the first way
            new_row = '<tr class="iw-form-unsaved"><td>' + input_string + '</td>' +
            '<td>' + label_string + '</td></tr>';

            table_body.innerHTML = table_body.innerHTML + new_row;
        }
    },

  /**
   * @return el|false
   */
    valueAlreadyExists: function ( value, table ) {
        var inputs = table.getElementsByTagName('input'),
        inputs_length = inputs.length,
        i;

        if ( !inputs ) {
            return false;
        }

        for (i=0; i<inputs_length; ++i) {
            if ( inputs[i].value === value ) {
                return inputs[i];
            }
        }

        return false;
    },

    getInputNameFromACName: function ( ac_input ) {
      // Whacks off the "_ac"
        return ac_input.name.substring(0, ac_input.name.length - 3);
    },

    focusCustomRadio: function ( e ) {
        var YUI = YAHOO.util,
        div = YUI.Dom.getAncestorByTagName(this, 'div'),
        radio = YUI.Dom.getElementsByClassName('iw-form-radio-custom', 'input', div);

        radio[0].checked = true;
    },

    submitEvents: {},

    getFormActions: function ( form ) {
        var input = form.iw_action_name,
        options,
        option_count,
        output = [],
        i;

        if ( YAHOO.lang.isUndefined(input) ) {
            return [];
        }

        if ( input.tagName.toLowerCase() === 'input' ) {
            return [input.value];
        }

        if ( input.tagName.toLowerCase() === 'select' ) {
            options = input.getElementsByTagName('option');
            option_count = options.length;

            for (i=0; i<option_count; ++i) {
                output.push(options[i].value);
            }
        }

        return output;
    },

    getFormNameFromShell: function ( shell ) {
        return shell.id.substring(0, shell.id.length - 6);
    },

    getFormFromShell: function ( shell ) {
        var form_name = iw_form.getFormNameFromShell(shell);

        return document.forms[ form_name ];
    },

    initSubmits: function ( shells ) {
        var form, form_name, shell, YUI = YAHOO.util;

        for (shell in shells) {
            if ( shells.hasOwnProperty(shell) ) {
                shell     = shells[ shell ];
                form_name = this.getFormNameFromShell(shell);
                form      = this.getFormFromShell(shell);

                this.formSubmitStatusCache[ form_name ] = false;

                this.submitEvents[ form_name ] = new YUI.CustomEvent(form_name + 'Submit', form);

                if ( this.isHandled(form) ) {
                    iw_form_handler(form, null, 'submit');
                }

                if ( YUI.Dom.hasClass(form, 'iw-form-label-refresh') ) {
                    iw_form.setupLabelRefresh(form);
                }
            }//end if
        }//end for
    },

    isHandled: function ( form ) {
        if ( form.no_handler ) {
            return false;
        }
        return true;
    },

    tooltip_instances: [],

  /**
   * Initializes tooltips for error display in validation
   */
    initTooltips: function ( shells ) {
        var statusImgs = YAHOO.util.Dom.getElementsByClassName('iw-form-img', 'img'),
        tooltip,
        name;

        name = 'iw-form-tt' + iw_form.tooltip_instances.length;
        tooltip = new YAHOO.widget.Tooltip(name, { context: statusImgs, zIndex: 10000 });
        tooltip.bringToTop();
        iw_form.tooltip_instances.push(tooltip);

        tooltip.contextMouseOverEvent.subscribe(
            function ( type, args ) {
                var context = args[0];
                if ( context && this._tempTitle ) {
                    return true;
                }
                return false;
            }
        );
    },

  /**
   * Initializes unlimited checkboxes
   */
    initUnlimited: function () {
        var unlimChecks = YAHOO.util.Dom.getElementsByClassName('iw-unlim-check', 'input'),
        unlimChecks_length = unlimChecks.length,
        i, div, input;

        for (i = 0; i < unlimChecks_length; ++i) {
            div   = YAHOO.util.Dom.getAncestorByTagName(unlimChecks[i], 'div');
            input = div.getElementsByTagName('input')[0];
            if ( input.value === '999999999' ) {
                unlimChecks[i].checked = 1;
                iw_form.doUnlimitedCheck(unlimChecks[i]);
            } else {
                unlimChecks[i].checked = 0;
                iw_form.doUnlimitedCheck(unlimChecks[i]);
            }
        }
    },

    /**
     * Init show-on selects.
     */
    initShowSelect: function () {
        var selects = YAHOO.util.Selector.query('select.iw-show'),
            select,
            option,
            opt_length,
            group,
            opt_group,
            i;

        for (select in selects) {
            if ( selects.hasOwnProperty(select) ) {
                select = selects[select];
                option = select.options[ select.selectedIndex ];
                group = option.getAttribute('data-iw-show');
                if (group) {
                    iw_form._showGroup(group);
                }
                opt_length = select.options.length;
                for (i=0; i < opt_length; ++i) {
                    opt_group = select.options[i].getAttribute('data-iw-show');
                    if ( opt_group && opt_group !== group ) {
                        iw_form._hideGroup(opt_group);
                    }
                }
                iw_form.restripeTable(YAHOO.util.Dom.getAncestorByTagName(select, 'form'));
            }
        }
    },

    restripeTable: function ( parent_el ) {
        var el = iw_lib.getEl(parent_el),
        rows = YAHOO.util.Selector.query('tr.iw-tr', el),
        row, className = 'row2', oppositeClass;

        for (row in rows) {
            if ( rows.hasOwnProperty(row) ) {
                row = rows[ row ];

                if ( YAHOO.util.Dom.getStyle(row, 'display') === 'none' ) {
                    continue;
                }

                className     = ( className === 'row1' ) ? 'row2' : 'row1';
                oppositeClass = ( className === 'row1' ) ? 'row2' : 'row1';

                if ( YAHOO.util.Dom.hasClass(row, className) ) {
                    continue;
                }

                YAHOO.util.Dom.removeClass(row, oppositeClass);
                YAHOO.util.Dom.addClass(row, className);
            }
        }
    },

  /**
   * Sets up form-related event handlers that can be delegated
   *
   * @param array   shells
   * @param boolean skipCancel
   */
    initEventHandlers: function ( shells, skipCancel ) {
        var forms, Event = YAHOO.util.Event, shell;

        skipCancel = ( skipCancel === true );

      // Submit handlers
        for (shell in shells) {
            if ( shells.hasOwnProperty(shell) ) {
                shell = shells[ shell ];
                forms = YAHOO.util.Dom.getElementsByClassName('iw-form', 'form', shell);
                this.initFormEvents(forms);
            }
        }

        Event.delegate(shells, 'click', this.unlimitedCheck, 'input.iw-unlim-check');

        Event.delegate(shells, 'click', this.toggleShow, 'input[data-iw-show]');

        Event.delegate(shells, 'click', this.generatePassword, 'input.iw-password-gen');

        Event.delegate(shells, 'click', this.checkAll, 'a.iw-check-all');
        Event.delegate(shells, 'click', this.checkNone, 'a.iw-check-none');

      // Grr...webkit browsers don't currently fire focus and blur events on checkboxes
      // Attaching a click listener to clear the status, because form submission is
      // prevented otherwise when there's an error on the form already.
        Event.delegate(shells, 'click', this._clearStatus, 'input.iw-in[type=checkbox]');

        Event.delegate(shells, 'click', this.fileSwitch, '.iw-file-switch');

        if ( skipCancel === false ) {
            Event.delegate(shells, 'click', this.onCancel, 'input.iw-form-cancel');
        }

        Event.delegate(shells, 'click', this.onEnableLocalChanges, 'input.in-enable_local_changes');
    },

  /**
   * Sets up the form group hide/show click handling.
   *
   * @param array shells
   */
    initGroupClickHandlers: function ( shells ) {
        var E = YAHOO.util.Event;
        E.delegate(shells, 'dblclick', this.toggleGroupDisplay, 'tr.iw-form-group-hd');
        E.delegate(shells, 'click', this.toggleGroupDisplay, 'tr.iw-form-group-hd img.iw-collapse');
    },

  /**
   * Initialize group initial status.
   *
   * @param array shells
   */
    initGroupInitialStatus: function () {
        var D = YAHOO.util.Dom,
        S = YAHOO.util.Selector,
        parent,
        group_num,
        children,
        child,
        groups,
        group_row;

        groups = S.query('.iw-form-shell .iw-collapse');
        for (group in groups) {
            if ( groups.hasOwnProperty(group) ) {
                group = groups[group];
                if (D.hasClass(group, 'iw-i-plus')) {
                    parent = D.getAncestorByTagName(group, 'table');
                    group_row = D.getAncestorByTagName(group, 'tr');
                    group_num = D.getAttribute(group_row,'data-iw-group-num');
                    children = S.query('tr.group-num-' + group_num, parent);
                    for (child in children) {
                        if ( children.hasOwnProperty(child) ) {
                            child = children[ child ];
                            if ( !D.hasClass(child, 'iw-hide') ) {
                                D.addClass(child, 'iw-hide');
                            }
                        }//end if has own property
                    }//end for
                }
            }
        }
    },

  /**
   * Connects the events for the form.  These are the events that cannot be delegated,
   * due to browser incompatibilities or other inconveniences.
   *
   * @param string|HTMLElement|array form_ish
   */
    initFormEvents: function ( form_ish ) {
        var forms, form, validated_inputs, autocompletes, autocomplete, set_custom_inputs,
        D = YAHOO.util.Dom,
        E = YAHOO.util.Event,
        S = YAHOO.util.Selector,
        text_collapse, textarea, textdiv,
        show_toggles, show_selects, action;

        if ( YAHOO.lang.isString(form_ish) ) {
            form_ish = document.getElementById(form_ish);
        }

        if ( YAHOO.lang.isArray(form_ish) === false ) {
            forms = [ form_ish ];
        } else {
            forms = form_ish;
        }

        for (form in forms) {
            if ( forms.hasOwnProperty(form) ) {
                form = forms[ form ];
                validated_inputs  = D.getElementsByClassName('iw-in', null, form);
                autocompletes     = D.getElementsByClassName('iw-form-ac', null, form);
                set_custom_inputs = D.getElementsByClassName('iw-radio-custom-input', null, form);
                text_collapse     = D.getElementsByClassName('iw-form-text-collapse', 'textarea', form);
                show_toggles      = S.query('input[data-iw-show]');
                show_selects      = S.query('select.iw-show');

                E.on(validated_inputs, 'focus', this._clearStatus);
                E.on(validated_inputs, 'blur', this.validate);

                E.on(autocompletes, 'focus', this.onACFocus);

                for (autocomplete in autocompletes) {
                    if ( autocompletes.hasOwnProperty(autocomplete) ) {
                        autocomplete = autocompletes[ autocomplete ];

                        this.initAutoComplete(autocomplete);
                    }
                }

                for (textarea in text_collapse) {
                    if ( text_collapse.hasOwnProperty(textarea) ) {
                        textarea = text_collapse[ textarea ];

                        textdiv = textarea.parentNode;
                        if ( D.hasClass(textdiv, 'iw-mult') === false ) {
                            this.initTextarea(textarea);
                        }
                    }
                }

                E.on(set_custom_inputs, 'focus', this.focusCustomRadio);

                E.on(text_collapse, 'focus', this.listenForNewlines);
                E.on(text_collapse, 'blur', this.removeListenForNewlines);
                E.on(text_collapse, 'keyup', this.updateCharCount);
                E.on(text_collapse, 'mouseup', this.resizeCharCount);

                E.on(D.getElementsByClassName('in-password-with-generator', null, form), 'keyup', this.keydownPasswordStrength);
                iw_form_validate_handler.xhrSuccessEvent.subscribe(this.updateStrengthMessage);
              //E.on( D.getElementsByClassName( 'in-password', null, form ), 'blur', this.validatePasswordStrength );

              //E.on( show_toggles, 'change', this.toggleShow );
                E.on(show_selects, 'change', this.toggleShowSelect);

                action = this.getAction(form);

              // Anything that appears to be an add*Commit action gets the reset enabled
                if ( action.match(/^add[a-zA-Z]*Commit$/) ) {
                    iw_xhr.subscribeTo(this.enableResetOption, action, this.getCtrl(form));
                }
            }//end if
        }//end for

        E.on(forms, 'submit', this.onSubmit);
    },

    deleteOptionByValue: function ( select, value ) {
        var length = select.length, i;
        for (i = 0; i < length; i++) {
            if ( select[i].value === value ) {
                select.remove(i);
                length = select.length;
            }
        }
    },

    toggleGroupDisplay: function ( e ) {
        var E = YAHOO.util.Event,
        D = YAHOO.util.Dom,
        S = YAHOO.util.Selector,
        target = D.getAncestorByTagName(E.getTarget(e), 'tr'),
        parent,
        group_num,
        children,
        child,
        img;

        parent = D.getAncestorByTagName(target, 'table');
        group_num = D.getAttribute(target, 'data-iw-group-num');
        children = S.query('tr.group-num-' + group_num, parent);
        img = S.query('img.iw-collapse', target)[0];

        if ( D.hasClass(img, 'iw-i-minus') ) {
            iw_lib.updateIWIcon(img, 'iw-i-plus');
        } else {
            iw_lib.updateIWIcon(img, 'iw-i-minus');
        }

        for (child in children) {
            if ( children.hasOwnProperty(child) ) {
                child = children[ child ];
                if ( D.hasClass(child, 'iw-hide') ) {
                    D.removeClass(child, 'iw-hide');
                } else {
                    D.addClass(child, 'iw-hide');
                }
            }//end if has own property
        }//end for
    },

    newlineKeyListener: false,
    keyListenerTarget: false,

    adjustCollapse: function () {
        var el = iw_form.keyListenerTarget,
        value = el.value,
        length = value.split("\n").length,
        currentRows = el.rows;

        if ( length >= currentRows && currentRows < el.getAttribute('data-maxrows') ) {
            el.rows = currentRows + 1;
        }

    },

    listenForNewlines: function ( e ) {
        iw_form.keyListenerTarget = this;
        iw_form.newlineKeyListener = new YAHOO.util.KeyListener(
            this,
            { keys: [10,13] },
            { fn: function () {
                iw_form.adjustCollapse(this); } }
        );
        iw_form.newlineKeyListener.enable();
    },

    removeListenForNewlines: function ( e ) {
        if ( iw_form.newlineKeyListener ) {
            iw_form.newlineKeyListener.disable();
        }
    },

    initTextarea: function ( input ) {
        var limit = input.getAttribute('data-maxlength'),
        div = document.createElement('div'),
        counter = document.createElement('div'),
        end = YAHOO.util.Dom.getLastChild(input.parentNode),
        width = YAHOO.util.Dom.getStyle(input, 'width');

        YAHOO.util.Dom.setStyle(div, 'width', width);

        div.className = 'charCounter';
        YAHOO.util.Dom.insertAfter(div, end);
        div.appendChild(counter);
        counter.innerHTML = input.value.length + '/' + limit;
    },

    resizeCharCount: function ( e ) {
        var input = YAHOO.util.Event.getTarget(e),
        div = YAHOO.util.Dom.getElementsByClassName('charCounter'),
        width = YAHOO.util.Dom.getStyle(input, 'width');

        YAHOO.util.Dom.setStyle(div, 'width', width);
    },

    updateCharCount: function ( e ) {
        var limit = this.getAttribute('data-maxlength'),
        input = YAHOO.util.Event.getTarget(e),
        div, counter;

        div = YAHOO.util.Dom.getNextSibling(input);
        while ( div.className !== 'charCounter' ) {
            div = YAHOO.util.Dom.getNextSibling(div);
        }

        counter = YAHOO.util.Dom.getFirstChild(div);
        counter.innerHTML = input.value.length + '/' + limit;

        if ( input.value.length > limit ) {
            iw_form.markInvalid(input, '##LG_INPUT_VALUE_OUT_OF_RANGE##');
        } else {
            iw_form.clearStatus(input);
        }
    },

  /**
   * Clears the input status
   */
    _clearStatus: function ( e ) {
        iw_form.clearStatus(this);
    },

  /**
   * Determines if an input is required or not
   *
   * @return boolean
   */
    isRequired: function ( input ) {
        var Dom = YAHOO.util.Dom;

        return Dom.hasClass(input, 'iw-form-req');
    },

  /**
   * Gets the input status image from a particular form input
   *
   * @param HTMLElement el The input next to the status image to get
   */
    getStatusImage: function ( el ) {
        var img = document.getElementById(el.id + '_status_span'),
        Dom = YAHOO.util.Dom,
        div,
        stats;

        if ( img ) {
            return img;
        }

        div   = Dom.getAncestorByClassName(el, 'iw-form-input', 'div');
        stats = Dom.getElementsByClassName('iw-i', 'img', div);

        if ( YAHOO.lang.isUndefined(stats[0]) === true ) {
            return false;
        }

        return stats[0];
    },

  /**
   * Gets the row that a form input is contained in
   *
   * @param HTMLElement el The input in the row to get
   */
    getTableRow: function ( el ) {
        var Dom = YAHOO.util.Dom,
        tr = Dom.getAncestorByClassName(el, 'iw-tr', 'tr');

        if ( YAHOO.lang.isUndefined(tr) === true || YAHOO.lang.isNull(tr) ) {
            return false;
        }

        return tr;
    },

    getTableCells: function ( el ) {
        var tr = iw_form.getTableRow(el),
        cells;

        if ( tr === false ) {
            return false;
        }

        return tr.getElementsByTagName('td');
    },

  /**
   * Formats an array or string into a string suitable for display in the errors
   *
   * @param string|array
   * @return string
   */
    formatError: function ( arrayOrString ) {
        if ( YAHOO.lang.isArray(arrayOrString) ) {
            return arrayOrString.join('<br>');
        }
        return arrayOrString;
    },

  /**
   * Marks a given input as invalid (red X)
   *
   * @param HTMLElement input
   * @param string      msg   error message for tooltip
   */
    markInvalid: function ( input, msg ) {
        this._updateStatus(input, 'iw-i-negative', this.formatError(msg));
    },

  /**
   * Marks a given input as valid (green check)
   *
   * @param HTMLElement input
   */
    markValid: function ( input ) {
        this._updateStatus(input, 'iw-i-positive');
    },

  /**
   * Marks a given input as pending (spinner)
   *
   * @param HTMLElement Input
   */
    markPending: function ( input ) {
        this._updateStatus(input, 'iw-i-pending');
    },

    markMessages: function ( input, msg ) {
        this._updateStatus(input, 'iw-i-help', this.formatError(msg));
    },

  /**
   * Removes all status from the given input
   *
   * @param HTMLElement input
   */
    clearStatus: function ( input ) {
        var error = document.getElementById('iw-error-' + input.id);
        this._updateStatus(input, 'iw-i-clear');
        if ( error ) {
            YAHOO.util.Dom.removeClass(error, 'iw-error-populated');
        }
    },

    _updateStatus: function ( input, css_class, msg ) {
        msg      = msg || '';
        var stat = iw_form.getStatusImage(input),
        turn_red = ( css_class === 'iw-i-negative' ) ? true : false,
        td, error = document.getElementById('iw-error-' + input.id);

        if ( css_class === 'iw-i-negative' && msg !== '' ) {
            css_class = 'iw-i-alert';
        }

        if ( stat !== false ) {
            iw_lib.updateIWIcon(stat, css_class);
            stat.title = msg;

            if ( turn_red === true ) {
                td = iw_form.getTableCells(input);
                if ( td !== false ) {
                    YAHOO.util.Dom.addClass(td, 'iw-error-row');
                }
            }

            if ( css_class === 'iw-i-clear' || css_class === 'iw-i-positive' ) {
                td = iw_form.getTableCells(input);
                if ( td !== false ) {
                    YAHOO.util.Dom.removeClass(td, 'iw-error-row');
                }
            }

            if ( error ) {
                error.innerHTML = msg;
                YAHOO.util.Dom.addClass(error, 'iw-error-populated');
            }
        }//end if
    },

  /**
   * Checks if an input has an error
   *
   * @param HTMLElement input
   */
    hasError: function ( input ) {
        var stat = iw_form.getStatusImage(input);
        if ( stat === false ) {
            return false;
        }
        return YAHOO.util.Dom.hasClass(stat, 'iw-i-negative') ||
        YAHOO.util.Dom.hasClass(stat, 'iw-i-alert');
    },

    enableInputs: function ( form ) {
        var i, el_count = form.elements.length;
        for (i=0; i<el_count; ++i) {
            if ( form.elements[i].type && form.elements[i].type === 'checkbox' ) {
                continue;
            }
            form.elements[i].disabled = false;
        }
    },

    onSubmit: function ( e ) {
        var form = YAHOO.util.Event.getTarget(e),
        form_name = form.attributes.getNamedItem('name').value;

        iw_form.clearAutocompletes();
        iw_form.saveRichTextEditors();
        if ( iw_form.validateFormForErrors(e) === true ) {
          // First, enable all the form fields, in case they were disabled
            iw_form.enableInputs(form);
            iw_form.submitEvents[ form_name ].fire(e);
        } else {
            YAHOO.util.Event.stopEvent(e);
        }
    },

    saveRichTextEditors: function () {
        var editor;

        for (editor in iw_form.richTextEditors) {
            if ( iw_form.richTextEditors.hasOwnProperty(editor) ) {
                iw_form.richTextEditors[ editor ].saveHTML();
            }
        }
    },

  /**
   * Validates that the form has no errors
   *
   * @param Event e
   */
    validateFormForErrors: function ( e ) {
        var is_valid = true,
        form = YAHOO.util.Event.getTarget(e),
        i, len, current_input, limit, file;

        iw_form.clearStatus(iw_form.getSubmitButton(form));

        for (i = 0, len = form.elements.length; i < len; ++i) {
            current_input = form.elements[i];

            if ( current_input.type === 'file' ) {
                if ( current_input.files &&
                current_input.files[0] &&
                iw_js_data &&
                iw_js_data.post_max_size ) {
                    file  = current_input.files[0];
                    limit = iw_js_data.post_max_size;
                    if ( file.size > limit ) {
                        YAHOO.util.Event.stopEvent(e);
                        iw_form.markInvalid(iw_form.getSubmitButton(form), '##LG_UPLOADMAXSIZELIMIT_ALT##');
                        return false;
                    }
                }
                continue;
            }

            iw_form.checkValidateRequired(current_input);
            if ( current_input.name === 'submit' ||
            current_input.name === 'cancel' ) {
                continue;
            }
            if ( iw_form.hasError(current_input) === true ) {
              // Flag that the submit should be cancelled
                is_valid = false;
            }
        }//end for

        if ( is_valid === false ) {
            YAHOO.util.Event.stopEvent(e);
            iw_form.markInvalid(iw_form.getSubmitButton(form), '##LG_INPUT_FORM_DID_NOT_VALIDATE##');
            return false;
        }

        return true;
    },

    onCancel: function ( e ) {
        YAHOO.util.Event.stopEvent(e);
        history.back();
    },

  /**
   * Event wrapper to validate a given input
   *
   * @param Event  e
   * @param object form
   */
    validate: function ( e ) {
        var form = YAHOO.util.Dom.getAncestorByTagName(YAHOO.util.Event.getTarget(e), 'form');
        YAHOO.util.Event.stopEvent(e);
        if ( iw_form.formSubmitStatusCache[ form.attributes.getNamedItem('name').value ] === false ) {
            iw_form.doValidate(YAHOO.util.Event.getTarget(e));
        }
    },

  /**
   * Checks if further validation is required.
   *
   * Returns true or false. If true, it means that it did nothing, if false it
   * means that it did the validation, so the next step should not continue
   *
   * @param HTMLElement el
   * @return boolean READ DESCRIPTION ABOVE
   */
    checkValidateRequired: function ( el ) {
        var value,
        style_check1,
        style_check2,
        empty_array = [],
        Dom = YAHOO.util.Dom,
        tr = Dom.getAncestorByTagName(el, 'tr');

      // Only check if the value is blank
        if ( el.value === '' ) {
            style_check1 = Dom.getStyle(el, 'display');
            style_check2 = Dom.getStyle(tr, 'display');

            if ( style_check1 !== false &&
            style_check1 !== 'none' &&
            style_check2 !== false &&
            style_check2 !== false ) {
                if ( Dom.getStyle(el, 'display') !== 'none' &&
                Dom.getStyle(tr, 'display') !== 'none' ) {
                    if ( iw_form.isRequired(el) === true ) {
                        iw_form.markInvalid(el, '##LG_INPUT_IS_REQUIRED##');
                        return false;
                    } else {
                        iw_form.markValid(el);
                        return false;
                    }
                }
            }
        } else if ( el.type === 'checkbox' &&
               Dom.hasClass(el, 'iw-unlim-check') === false &&
               Dom.hasClass(el, 'iw-check-all') === false ) {
            if ( el.checked ) {
                iw_form.markValid(el);
                return false;
            }
            if ( iw_form.isRequired(el) === false ) {
                iw_form.markValid(el);
                return false;
            }

            value = iw_form.getValue(el);

            if ( value.join(',') !== '' ) {
                iw_form.markValid(el);
                return false;
            } else {
                iw_form.markInvalid(el, '##LG_INPUT_IS_REQUIRED##');
                return false;
            }
        }//end if
        return true;
    },

  /**
   * Validates an input
   *
   * @param HTMLElement el
   */
    doValidate: function ( el ) {
        if ( iw_form.registeredACs[ el.id ] ) {
            if ( iw_form.registeredACs[ el.id ].isContainerOpen() ) {
                iw_form.clearStatus(el);
                return;
            }
        }

        var Dom = YAHOO.util.Dom,
        div = Dom.getAncestorByTagName(el, 'div'),
        tr = Dom.getAncestorByTagName(div, 'tr'),
        form = Dom.getAncestorByTagName(el, 'form'),
        stat = iw_form.getStatusImage(el),
        temp,
        temp_length,
        args,
        isRequired = iw_form.isRequired(el),
        i,
        check,
        element_name,
        radios,
        linked, linkedInput, id;

        if ( Dom.getStyle(el, 'display') === 'none' ) {
          // Not displayed, so the user can't do anything about it
            return;
        }

        if ( el.type && el.type === 'file' ) {
            return;
        }

        if ( YAHOO.util.Dom.hasClass(el, 'iw-form-radio-custom') ) {
            return;
        }

        if ( el.name === 'confirm_password' ) {
          // Stupid. The confirm password needs the input of the actual password
            if ( el.value !== form.password.value ) {
                iw_form.markInvalid(el, '##LG_PASSWORDSDONOTMATCH##');
                return;
            } else if ( el.value !== '' ) {
                iw_form.markValid(el);
                return;
            }
        }

        element_name = el.name;

        radios = div.getElementsByTagName('input');

        if ( radios.length ) {
            if ( radios[0].type === 'radio' ) {
                element_name = radios[0].name;
                if ( el.value === '' ) {
                    iw_form.clearStatus(el);
                    return;
                }
            }
        }

      // First check the required status
        if ( iw_form.checkValidateRequired(el) === false ) {
            return;
        }

      // Now check the status of unlimited inputs
        if ( el.value === '999999999' ) {
            check = Dom.getElementsByClassName('iw-unlim-check', 'input', div);
            if ( check[0] ) {
                iw_form.markValid(el);
                return;
            }
        }
      // If it's a select, it's always valid unless someone is being a real turd
      // It'll be validated on the server anyway, so return
        if ( el.tagName === 'SELECT' ) {
            iw_form.markValid(el);
            return;
        }

        if ( iw_js_data.xhr_validate !== '1' ) {
            return;
        }

      // Clear the status image
        iw_form.markPending(el);

        args = iw_form.getHiddenInputValues(form);

        args.form  = form.iw_form_class_name.value;
        args.ctrl  = form.iw_ctrl_name.value;
        args.input = element_name;
        args.value = el.value;

        linked = Dom.getAttribute(el, 'data-iw-linked');

        if ( linked ) {
            linked = linked.split(',');

            for (id in linked) {
                if ( linked.hasOwnProperty(id) ) {
                    id = linked[ id ];
                    linkedInput = document.getElementById(id);

                    if ( linkedInput ) {
                        args[ linkedInput.attributes.getNamedItem('name').value ] = iw_form.getValue(linkedInput);
                    }
                }
            }
        }

        iw_xhr.getReply(iw_form_validate_handler, args, stat);
    },

    getHiddenInputValues: function ( form, include_controller_info ) {
        var temp, temp_length, i, output = {},
        controller_info = { 'iw_action_name' : true,
            'action' : true,
            'iw_ctrl_name' : true ,
            'iw_form_class_name' : true,
            'security_token' : true };

        include_controller_info = include_controller_info || false;

      // Get all inputs
        temp        = form.getElementsByTagName('input');
        temp_length = temp.length;
      // Loop through and add the values of any hidden inputs to the form
        for (i = 0; i < temp_length; ++i) {
            if ( temp[i].type === 'hidden' ) {
                if ( include_controller_info === false && controller_info[ temp[i].name ] ) {
                    continue;
                }

                output[ temp[i].name ] = temp[i].value;
            }
        }

        return output;
    },

  /**
   * Resets a form, but keeps the current values of show/hide flags
   *
   * @param string|form id_or_form
   */
    reset: function ( id_or_form ) {
        form = iw_lib.getEl(id_or_form);
        var show_selects = YAHOO.util.Selector.query('select.iw-show'),
        show_selects_length = show_selects.length,
        toggle_shows = YAHOO.util.Selector.query('input[data-iw-show]' , form),
        toggle_shows_length = toggle_shows.length,
        current_values = {},
        j;
        for (j = 0; j < toggle_shows_length; j++) {
            current_values[ toggle_shows[ j ].id ] = ( toggle_shows[ j ].checked );
        }
        for (j = 0; j < show_selects_length; j++) {
            current_values[ show_selects[ j ].id ] = iw_form.getValue(show_selects[ j ]);
        }
        form.reset();
        for (j = 0; j < toggle_shows_length; j++) {
            toggle_shows[ j ].checked = current_values[ toggle_shows[ j ].id ];
        }
        for (j = 0; j < show_selects_length; j++) {
            iw_form.setValue(show_selects[ j ], current_values[ show_selects[ j ].id ]);
        }
    },


  /**
   * Checks all the checkboxes related to the input
   *
   * "related to the input" means "in the same form cell", which is how the Form
   * class and template will display it automatically.
   *
   * @param event e
   */
    checkAll: function ( e ) {
        YAHOO.util.Event.stopEvent(e);
        var checks = iw_form._getCheckboxesForCheckAll(this),
        checks_length = checks.length,
        i;

        for (i = 0; i < checks_length; ++i) {
            checks[i].checked = 'checked';
        }
    },

  /**
   * Gets the checkboxes relevant to a checkall/none event.
   *
   * @param  {element} el Check all or none that was clicked.
   *
   * @return {array}
   */
    _getCheckboxesForCheckAll: function ( el ) {
        var div    = YAHOO.util.Dom.getAncestorByClassName(el, 'iw-form-input', 'div'),
        checks = YAHOO.util.Selector.query('input[type=checkbox]', div);

        return checks;
    },

  /**
   * Unchecks all the checkboxes related to the input
   *
   * "related to the input" means "in the same div", which is how the Form class
   * and template will display it automatically.
   *
   * @param event e
   */
    checkNone: function ( e ) {
        YAHOO.util.Event.stopEvent(e);
        var checks = iw_form._getCheckboxesForCheckAll(this),
        checks_length = checks.length,
        i;

        for (i = 0; i < checks_length; ++i) {
            checks[i].checked = '';
        }
    },

  /**
   * Cache for holding properties of animated elements
   */
    inputPropertyCache: {},

  /**
   * Generates strong passwords.
   */
    generatePassword: function ( e ) {
        var input = document.getElementById(this.getAttribute('data-iw-input')),
        maxlength = input.getAttribute('maxlength');
        iw_xhr.getReply(iw_form_generate_password_data_handler, maxlength, this);
        iw_form.markPending(input);
    },

  /**
   * Method for scoring password quality.
   */
    passwordStrength: function ( pwd ) {
        var score = 0,
        has_num = false,
        has_char = false,
        result;

        if ( pwd.length < 1 ) {
            return "";
        }
        if ( pwd.length < 4 ) {
            return "##LG_VERY_WEAK_PASSWORD##";
        }
        if ( pwd.length >= 7 ) {
            score++;
        }
        if ( pwd.length >= 12 ) {
            score++;
        }
        if ( pwd.match(/[a-z]/) && pwd.match(/[A-Z]/) ) {
            score++;
        }
        if ( pwd.match(/[0-9]/) ) {
            has_num = true;
        }
        if ( pwd.match(/.[!,@,#,$,%,\^,&,*,?,_,~,-,£,(,)]/) ) {
            has_char = true;
        }

        if ( score === 0 ) {
            result = '##LG_VERY_WEAK_PASSWORD##';
        } else if ( score === 1 ) {
            result = '##LG_WEAK_PASSWORD##';
        } else if ( score === 2 && has_num ) {
            result = '##LG_STRONG_PASSWORD##';
        } else if ( score === 3 && ( has_num || has_char ) ) {
            result = '##LG_VERY_STRONG_PASSWORD##';
        } else {
            result = '##LG_MEDIUM_PASSWORD##';
        }

        return result;
    },

    updateStrengthMessage: function ( eventName, args ) {
        var reply = args[0],
        el = args[1],
        id = ( el.id ).replace(/_status_span/g, ""),
        strength_id = id + '_strength',
        input = document.getElementById(id);
        if ( reply.getReplyCode() !== 204 ) {
            return;
        }

        if ( YAHOO.util.Dom.hasClass(input, 'in-password') === false ) {
            return;
        }
        if ( reply.getReplyData()[0] === '##LG_COMMON_PASSWORD##' ) {
            document.getElementById(strength_id).innerHTML = reply.getReplyData()[0];
            document.getElementById(strength_id).className = 'iw-common';
        }
    },

  /**
   * Provides keydown support for as-you-type password rating.
   */
    keydownPasswordStrength: function ( e ) {
        var el = YAHOO.util.Event.getTarget(e),
        pwd = el.value;
        iw_form.processPasswordStrength(el, pwd);
    },

  /**
   * Inserts/updates password strength element.
   */
    processPasswordStrength: function ( el, pwd ) {
        var strength = iw_form.passwordStrength(pwd),
        stat = iw_form.getStatusImage(el),
        strength_id = el.id + '_strength',
        el_idtag = el.id.substring(el.id.indexOf('_')),
        name_id,
        name,
        email_id,
        email,
        classes = {
            "##LG_VERY_WEAK_PASSWORD##":"iw-veryweak",
            "##LG_WEAK_PASSWORD##":"iw-weak",
            "##LG_MEDIUM_PASSWORD##":"iw-medium",
            "##LG_STRONG_PASSWORD##":"iw-strong",
            "##LG_VERY_STRONG_PASSWORD##":"iw-verystrong",
            "##LG_COMMON_PASSWORD##":"iw-common"
        };

        name_id = 'nickname' + el_idtag;
        email_id = 'email' + el_idtag;

        if ( document.getElementById(name_id) && document.getElementById(email_id) ) {
            name = document.getElementById(name_id).value;
            email = document.getElementById(email_id).value;

            if ( name === pwd ||
            email === pwd ||
            email.substring(0, email.indexOf('@')) === pwd ) {
                strength = "##LG_VERY_WEAK_PASSWORD##";
            }
        }

        if ( document.getElementById(strength_id) ) {
            document.getElementById(strength_id).innerHTML = strength;
            document.getElementById(strength_id).className = classes[strength];
            document.getElementById(strength_id).innerHTML = strength;
            document.getElementById(strength_id).className = classes[strength];
        }
    },

  /**
   * When unlimited is checked, fills in the correct value for the input
   *
   * @param event e
   */
    unlimitedCheck: function ( e ) {
        iw_form.doUnlimitedCheck(this);
    },

  /**
   * Actually does the unlimited check
   *
   * This needed to be split out because it needs to be called directly by the
   * code that initializes the unlimited behavior if the input is actually
   * unlimited already
   */
    doUnlimitedCheck: function ( el ) {
        var div = YAHOO.util.Dom.getAncestorByTagName(el, 'div'),
        td = YAHOO.util.Dom.getAncestorByTagName(div, 'td'),
        inputs = div.getElementsByTagName('input'),
        inputs_length = inputs.length,
        input = false,
        hider,
        setUnlimitedValue,
        setUnlimitedStyle,
        i;

        for (i=0; i<inputs_length; ++i) {
            if ( input === false ) {
                if ( inputs[i].type === 'text' ) {
                    input = inputs[i];
                }
            }
        }

        if ( YAHOO.lang.isUndefined(iw_form.inputPropertyCache[input.name]) ) {
            iw_form.inputPropertyCache[input.name] = { width: input.offsetWidth,
                paddingLeft: YAHOO.util.Dom.getStyle(input, 'padding-left'),
                paddingRight: YAHOO.util.Dom.getStyle(input, 'padding-right')
            };
        }

        if ( el.checked ) {
            iw_form.inputPropertyCache[input.name].value = input.value;

            setUnlimitedValue = function () {
                input.value = '999999999';
                iw_form.doValidate(input);
            };

            setUnlimitedStyle = function () {
                YAHOO.util.Dom.setStyle(input, 'padding-left', 0);
                YAHOO.util.Dom.setStyle(input, 'padding-right', 0);
            };

            hider = new YAHOO.util.Anim(
                input,
                { opacity: { to: 0 },
                    width: { to: 0 },
                    paddingLeft: { to: 0 },
                    paddingRight: { to: 0 } },
                0.25,
                YAHOO.util.Easing.easeOut
            );

            hider.onComplete.subscribe(setUnlimitedValue);
            hider.onComplete.subscribe(setUnlimitedStyle);
        } else {
            if ( YAHOO.lang.isUndefined(iw_form.inputPropertyCache[input.name].value) === false ) {
                input.value = iw_form.inputPropertyCache[input.name].value;
            }

            hider = new YAHOO.util.Anim(
                input,
                { opacity: { from: 0, to: 100 },
                    width: { to: iw_form.inputPropertyCache[input.name].width }
                                     },
                0.25,
                YAHOO.util.Easing.easeIn
            );

            setUnlimitedStyle = function () {
                YAHOO.util.Dom.setStyle(input, 'padding-left', iw_form.inputPropertyCache[input.name].paddingLeft);
                YAHOO.util.Dom.setStyle(input, 'padding-right', iw_form.inputPropertyCache[input.name].paddingRight);
            };

            hider.onStart.subscribe(setUnlimitedStyle);
        }//end if
        hider.animate();
    },

    getAction: function ( form ) {
        return this.getValue(form.iw_action_name);
    },

    getCtrl: function ( form ) {
        return this.getValue(form.iw_ctrl_name);
    },

  /**
   * This should ultimately be expanded to handle multi-selects, checkboxes, etc
   */
    getValue: function ( input ) {
        var D = YAHOO.util.Dom,
        S = YAHOO.util.Selector,
        checks, iw_id, esc_iw_id, output = [], i, check_length,
        namePrefix, form, inputs, inputs_length;

        input = iw_lib.getEl(input);

        if ( input.tagName.toLowerCase() === 'input' ) {
            if ( input.type === 'checkbox' ) {
                iw_id = D.getAttribute(input, 'data-iw-id');
              // escape the iw_id for use in the selectory query below
                esc_iw_id = iw_id.replace('[', '\\[').replace(']', '\\]');
                checks = S.query('input[data-iw-id=' + esc_iw_id + ']');
                check_length = checks.length;

                for (i=0; i<check_length; ++i) {
                    if ( checks[i].checked ) {
                        output.push(checks[i].value);
                    }
                }
                return output;
            } else if ( input.type === 'hidden' ) {
                if ( input.name.indexOf('[') === -1 ) {
                    return input.value;
                } else {
                    namePrefix = input.name.substring(0, input.name.indexOf('['));
                    namePrefix = namePrefix + '\\[';
                    form = D.getAncestorByTagName(input, 'form');
                    inputs = S.filter(form.elements, 'input[name^=' + namePrefix + ']');
                    inputs_length = inputs.length;

                    for (i=0; i<inputs_length; ++i) {
                        output.push(inputs[i].value);
                    }
                    return output;
                }
            }//end if
            return input.value;
        }//end if

        if ( input.tagName.toLowerCase() === 'select' ) {
            return input.options[ input.selectedIndex ].value;
        }
    },

    setValue: function ( input, value ) {
        var check;
        if ( input.hasOwnProperty(0) && input[0].type && input[0].type === 'checkbox' ) {
          // Then it's a checkbox.
            if ( YAHOO.lang.isArray(value) === false ) {
                value = [ value ];
            }

            for (check in input) {
                if ( input.hasOwnProperty(check) ) {
                    check = input[ check ];

                    if ( value.indexOf(check.value) !== -1 ) {
                        check.checked = 1;
                    } else {
                        check.checked = 0;
                    }
                }
            }
            return;
        }

        input = iw_lib.getEl(input);

        if ( input.tagName.toLowerCase() === 'input' ) {
            input.value = value;
            return;
        }

        if ( input.tagName.toLowerCase() === 'select' ) {
            input.value = value;
            return;
        }
    },

    setFocus: function ( form, name ) {
        form = iw_lib.getEl(form);
        form.elements[name].focus();
    },

  /**
   * Enables form reset when subscribed to success events.
   *
   * @param string eventType YUI Custom event name
   * @param array  args      args[0] = the reply object, args[1] = options array
   */
    enableResetOption: function ( eventType, args ) {
        var options = args[1];

        options.prevent_reset = false;
    },

  /**
   * Prevents form reset when subscribed to success events.
   *
   * This is the default.
   *
   * @param string eventType YUI Custom event name
   * @param array  args      args[0] = the reply object, args[1] = options array
   */
    disableResetOption: function ( eventType, args ) {
        var options = args[1];

        options.prevent_reset = true;
    },

    enableSubmit: function ( form ) {
      /**
     * Only enable the button and hide spinner if the button is really there
     */
        if ( form ) {
            if ( YAHOO.lang.isNull(iw_form.getSubmitButton(form)) === false &&
            YAHOO.lang.isUndefined(iw_form.getSubmitButton(form).disabled) === false ) {
                iw_form.getSubmitButton(form).disabled = 0;
              // Clear the spinner
                if ( YAHOO.util.Dom.hasClass(iw_form.getStatusImage(iw_form.getSubmitButton(form)), 'iw-i-pending') ) {
                    iw_form.clearStatus(iw_form.getSubmitButton(form));
                }
            }
        }
    }
};

/**
 * Handles requests back to the server for line-by-line validation
 */
iw_form_validate_handler = {
    responder: 'Form_InputValidator',

    xhrSuccessEvent: new YAHOO.util.CustomEvent('validateCompleteEvent'),

  /**
   * Ok result
   */
    handle200: function ( reply, el ) {
        iw_form.markValid(el);
        iw_form_validate_handler.xhrSuccessEvent.fire(reply, el);
    },

    handle201: function ( reply, el ) {
        iw_form.markMessages(el, reply.getReplyData());
        iw_form_validate_handler.xhrSuccessEvent.fire(reply, el);
    },

  /**
   * Not Ok result
   */
    handle204: function ( reply, el ) {
        iw_form.markInvalid(el, reply.getReplyData().join('<br>'));
        iw_form_validate_handler.xhrSuccessEvent.fire(reply, el);
    },

    handle403: function ( reply, el ) {
        iw_form.markInvalid(el, reply.getReplyText());
    }
};

iw_form_autocomplete_data_handler = {
    responder: 'Form_AutoCompleteData',

    handle201: function ( reply, el ) {
        IW_AC_DATA[ el.id ] = reply.getReplyData();
        iw_form.initAutoComplete(el);
    }
};

iw_form_label_handler = {
    responder: 'Form_LabelRefresh',

    handle201: function ( reply, form ) {
        var new_label = reply.getReplyData(),
        old_label = YAHOO.util.Dom.getElementsByClassName('iw-form-label', 'span', form);

        old_label[0].innerHTML = new_label;
    }
};

iw_form_generate_password_data_handler = {
    responder: 'GeneratePassword',

    handle201: function ( reply, el ) {
        var id = el.getAttribute('data-iw-input'),
        input = document.getElementById(id),
        confirm = document.getElementById('confirm_' + id),
        container;

        iw_form.processPasswordStrength(input, reply.getReplyData());

        iw_form.clearStatus(input);

        iw_form.setValue(input, reply.getReplyData());
        confirm.value = reply.getReplyData();

        if ( document.getElementById('pass_gen' + id) ) {
            document.getElementById('pass_gen' + id).innerHTML = reply.getReplyData();
        } else {
            container = document.createElement('div');
            container.setAttribute('id', 'pass_gen' + id);
            container.innerHTML = reply.getReplyData();
            input.parentNode.appendChild(container);
        }

        // trigger event - necessary for sync password option in sw user edit form.
        let $passwordInput = $('#' + id);
        $passwordInput && $passwordInput.trigger('input');
    }
};

/**
 * Special function that adds a listener to a given eventname on the specified
 * element, and on that action submit/routes that form xhr style through the
 * json controller mechanism
 *
 * @param HTMLElement form      The form that will ultimately be submitted
 * @param HTMLElement element   The element that the specified event is listen on
 *                              If null, form is assumed as the element
 * @param string      eventname The event that is listened for: 'change' 'submit' etc
 */
var iw_form_handler = function ( form, element, eventname ) {
    var formName;
    element = element || form;
    eventname = eventname || 'submit';

  /**
   * The handler callback, this function shows a banner if the reply contains data
   * and pops up an error if success was false.  Also routes to callback, if it was
   * specified when the iw_form_handler was made.  Finally, it  re-enables the
   * submit button if applicable
   *
   * @param iw_xhr.reply r
   * @param boolean      success
   */
    function complete( r, success )
    {
        var redirect,
        i,
        error,
        input,
        d = r.getReplyData(),
        formName = form.attributes.getNamedItem('name').value,
      // Need to re-get the form, because sometimes it's been innerHTMLed and is gone
        newForm = document.forms[ formName ],
        ctrl = iw_form.getCtrl(newForm),
        action = iw_form.getAction(newForm),
        options = { display_banner: true,
            display_errors: true,
            prevent_redirect: false,
            prevent_reset: true
        };

        if ( success ) {
            if ( iw_xhr.events.success[ ctrl ] && iw_xhr.events.success[ ctrl ][ action ] ) {
                iw_xhr.events.success[ ctrl ][ action ].fire(r, options);
            }
          // show success banner
            if ( YAHOO.lang.isNull(d) === false ) {
                if ( YAHOO.lang.isUndefined(d.redirect) === false ) {
                    if ( YAHOO.lang.isUndefined(d.force_redirect) === false ||
                    (window.location.pathname + window.location.search) !== d.redirect &&
                    options.prevent_redirect !== true ) {
                        d.message += ' -  ##LG_PLEASEWAIT##';
                    }
                }
                if ( options.display_banner ) {
                    iw_lib.iw_message_banner(d.message, 'iw-positive');
                }
                if ( options.prevent_reset === false ) {
                    iw_form.reset(newForm);
                }
            }
        } else {
            iw_form.formSubmitStatusCache[ formName ] = false;

            if ( iw_xhr.events.failure[ ctrl ] && iw_xhr.events.failure[ ctrl ][ action ] ) {
                iw_xhr.events.failure[ ctrl ][ action ].fire(r, options);
            }

          // show error banner
            if ( YAHOO.lang.isNull(d) === false && options.display_banner ) {
                iw_lib.iw_message_banner(d.message, 'iw-negative');
            }

          /**
       * If we got input_errors back, then we should try to mark the inputs
       * in the form as invalid
       */
            if ( options.display_errors &&
            YAHOO.lang.isUndefined(d.input_errors) === false ) {
                for (i in d.input_errors) {
                    if ( d.input_errors.hasOwnProperty(i) ) {
                        error = d.input_errors[ i ];
                        input = form[ error.name ];
                      // workaround for input_sets.
                        if ( YAHOO.lang.isUndefined(input) === true ) {
                            input = form[ error.name + '_0' ];
                        }
                      // Last resort
                        if ( YAHOO.lang.isUndefined(input) === true ) {
                            input = iw_form.getSubmitButton(form);
                            error.err = error.name + ': ' + error.err;
                        }

                      // Hidden inputs don't have the necessary structure around them.
                        if ( input.type !== 'hidden' ) {
                            iw_form.markInvalid(input, error.err);
                        }
                    }
                }
            }//end if
        }//end if

        iw_form.enableSubmit(form);

      /**
     * If redirect data was part of the response, redirect to the new location
     * if not already there
     */
        if ( options.prevent_redirect === false && YAHOO.lang.isNull(d) === false ) {
            if ( YAHOO.lang.isUndefined(d.redirect) === false ) {
                redirect = d.redirect;

                if ( YAHOO.lang.isUndefined(form.iw_force_redirect) === false ) {
                    redirect = form.iw_force_redirect.value;
                    d.force_redirect = true;
                }

                if ( YAHOO.lang.isUndefined(d.force_redirect) === false ||
                (window.location.pathname + window.location.search) !== redirect ) {
                    window.setTimeout(function () {
                        window.location = redirect; }, iw_lib.redirect_timeout);
                }
            }
        }
    }

  /**
   * This is the function that actually executes the iw_ctrl_handler / submits the form
   * It spins up the spinner and disables the submit button if applicable
   *
   * @param string type The type of event fired
   * @param array args
   */
    function route( type, args )
    {
      // If this is called by a custom event, it's different
        var e = ( args ) ? args[0] || type : type,
        form = null, form_name;

        if ( e.type === 'submit' ) {
          // if it's a submit event, then the target IS the form
            form = form || YAHOO.util.Event.getTarget(e);
        } else {
          // if it's not a submit event, then the target is a form
          // element, and target.form is the form.
            form = YAHOO.util.Event.getTarget(e).form;
        }

        form_name = form.attributes.getNamedItem('name').value;

        if ( e.type === 'submit' ) {
            iw_form.formSubmitStatusCache[ form_name ] = true;
        }

      // stop the event, we're taking over
        YAHOO.util.Event.stopEvent(e);

      // in IE form.submit is defined even if there is no
      // "submit" button in the form, so checking both submit and
      // submit.disabled here
        if ( !YAHOO.lang.isNull(iw_form.getSubmitButton(form)) &&
        !YAHOO.lang.isUndefined(iw_form.getSubmitButton(form).disabled) ) {
          // disable the submit button
            iw_form.getSubmitButton(form).disabled = 1;
          // Show the spinner on the submit button
            iw_form.markPending(iw_form.getSubmitButton(form));
        } else {
          // Show the spinner on the input element that triggered
          // the event
            iw_form.markPending(YAHOO.util.Event.getTarget(e));
        }
      // fire off to the xhr request
        iw_ctrl_handler.execute(form, complete, iw_form.getCtrl(form), iw_form.getAction(form));
    }//end function route

  // Only subscribe the event if the form is not null
    if ( YAHOO.lang.isUndefined(element) === false ) {
      // Interrupt the normal subscription and subscribe to the custom event instead
        if ( eventname === 'submit' ) {
            formName = form.attributes.getNamedItem('name').value;
            iw_form.submitEvents[ formName ].subscribe(route, element);
        } else {
            YAHOO.util.Event.on(element, eventname, route);
        }
    }
};

iw_form.init();
