/*jslint undef: true, widget: true, browser: true, onevar: true, eqeqeq: true, laxbreak: true */
/*global YAHOO iw_js_data */

var IW = {
    /**
     * Determines if we're in nodeworx or siteworx
     * @return string
     */
    getIWEnv: function () {
        return iw_js_data.iw_env.env;
    },

    getSuccessBannerTimeout: function () {
        return iw_js_data.iw_env.success_banner_timeout;
    },

    debug: { io: [] },
    registry: {},
    is_unloading: false
};

var iw_lib = {
    redirect_timeout: 2000,
    /**
     * Updates the class of an iw_message element
     *
     * @param HTMLElement el
     * @param string      new_class
     */
    updateStatusClass: function ( el, new_class ) {
        var Dom = YAHOO.util.Dom;
        el = this.getEl(el);
        if ( YAHOO.lang.isNull(new_class) === false ) {
            Dom.removeClass(el, 'iw-neutral');
            Dom.removeClass(el, 'iw-negative');
            Dom.removeClass(el, 'iw-positive');
            Dom.removeClass(el, 'iw-disabled');
            Dom.addClass(el, new_class);
        }
    },

    getEl: function ( id_or_el ) {
        var el;
        if ( YAHOO.lang.isString(id_or_el) ) {
            el = document.getElementById(id_or_el);
        } else {
            el = id_or_el;
        }

        return el;
    },

    _modalMasks: {},

    markRefreshInProgress: function ( el ) {
        var m, r, Dom = YAHOO.util.Dom;

        el = this.getEl(el);
        el = document.getElementById(el.id);

        m = this._modalMasks[ el.id ];

        if ( YAHOO.lang.isUndefined(m) ) {
            m = document.createElement('div');
            this._modalMasks[ el.id ] = m;

            Dom.addClass(m, 'iw-modal-mask');

            m.id = el.id + '-mask';

            m.innerHTML = '<div></div><img src="/img/blank.gif"/>';

            document.body.appendChild(m);
        }

        r = Dom.getRegion(el);

        Dom.setStyle(m, 'display', 'block');
        Dom.setStyle(m, 'top', r.y + 'px');
        Dom.setStyle(m, 'left', r.x + 'px');
        Dom.setStyle(m, 'width', r.width + 'px');
        Dom.setStyle(m, 'height', r.height + 'px');
    },

    clearRefreshInProgress: function ( el ) {
        el = this.getEl(el);
        if ( this._modalMasks[ el.id ] ) {
            YAHOO.util.Dom.setStyle(this._modalMasks[ el.id ], 'display', 'none');
        }
    },

    /**
     * Toggles the iworx spinner.
     *
     * @param HTMLElement el
     */
    toggleSpinner: function ( el ) {
        var Dom = YAHOO.util.Dom;

        if ( Dom.hasClass(el, 'iw-i-pending') ) {
            Dom.removeClass(el, 'iw-i-pending');
        } else {
            Dom.addClass(el, 'iw-i-pending');
        }
    },

    /**
     * Updates an iw_message originally generated by smarty
     *
     * input.el        => JS Element reference to what you're updating
     * input.new_class => string (optional) - new classname to use for the element
     * input.message   => string (optional) - new message
     *
     * @param object input
     */
    updateIWMessage: function ( input ) {
        var el = input.el,
        new_class = input.new_class || null,
        message = input.message || null;

        if ( YAHOO.lang.isNull(el) ) {
            // To stop us from making stupid javascript mistakes because
            alert("The element which was passed in is null, so JS will now " +
             "silently fail.  Just warning you.");
        }

        iw_lib.updateStatusClass(el, new_class);

        if ( YAHOO.lang.isNull(message) === false ) {
            if ( YAHOO.util.Dom.hasClass(el, 'iw-status') ) {
                el.innerHTML = message;
            } else {
                el.getElementsByTagName('div')[2].innerHTML = message;
            }
        }
    },

    updateIWIcon: function ( el, new_class ) {
        if ( !el.className ) {
            return;
        }

        var existing_classes = el.className.split(' '),
        class_length = existing_classes.length,
        i;

        for (i=0; i < class_length; ++i) {
            if ( existing_classes[i].substr(0, 5) === 'iw-i-' ) {
                YAHOO.util.Dom.replaceClass(el, existing_classes[i], new_class);
                return;
            }
        }
    },

    /**
     * Cached instance of a YUI Panel
     */
    iw_message_panel: null,

    /**
     * Puts a message on screen in a Panel
     * @param string body
     * @param string header (optional)
     */
    iw_message: function ( body, header ) {
        header = header || '##LG_JSONSYSTEMERRORMESSAGE##';

        if ( YAHOO.lang.isNull(iw_lib.iw_message_panel) ) {
            iw_lib.iw_message_panel = new YAHOO.widget.Panel("iw-msg", {
                fixedcenter: true,
                constraintoviewport: true,
                underlay: 'none',
                draggable: false,
                modal: true,
                close: true});

            // Subscribe an anonymous function to clear the body when hiding
            iw_lib.iw_message_panel.hideEvent.subscribe(function () {
                this.setBody(''); });
        }
        iw_lib.iw_message_panel.setHeader(header);

        var pre = document.createElement('pre');

        pre.innerHTML = body.replace(/\n/g, '<br>');

        iw_lib.iw_message_panel.appendToBody(pre);
        iw_lib.iw_message_panel.render(document.body);
        iw_lib.iw_message_panel.show();
    },

    /**
     * Cached instance of a YUI draggable panel
     */
    iw_popup_panel: null,
    iw_modal_popup: null,

    init_modal_popup: function () {
        if ( YAHOO.lang.isNull(iw_lib.iw_modal_popup) ) {
            iw_lib.iw_modal_popup = new YAHOO.widget.Panel("iw-modal-popup", {
                constraintoviewport: true,
                visible: false,
                close: true,
                draggable: true,
                modal: true });

            var kl = new YAHOO.util.KeyListener(
                document,
                { keys:27 },
                { fn:iw_lib.iw_modal_popup.hide,
                    scope:iw_lib.iw_modal_popup,
                    correctScope:true }
            );

            iw_lib.iw_modal_popup.cfg.queueProperty('keylisteners', kl);

            YAHOO.util.Dom.addClass(iw_lib.iw_modal_popup.element, 'iw-popup');

            // Subscribe an anonymous function to clear the body when hiding
            iw_lib.iw_modal_popup.hideEvent.subscribe(
                function () {
                    this.setBody('');
                }
            );
        } else {
            iw_lib.resetModalPopupWidth();
        }//end if
    },

    resetModalPopupWidth: function ( center ) {
        center = ( center === false ) ? false : true;
        iw_lib.iw_modal_popup.cfg.setProperty('width', 'auto');
        if ( center ) {
            iw_lib.iw_modal_popup.center();
        }
    },

    popupModal: function ( content, header, replaceNewlines, show, minWidth, theWidth ) {
        show            = ( show === false ) ? false : true;
        replaceNewlines = ( replaceNewlines === false ) ? false : true;

        this.init_modal_popup();

        this.iw_modal_popup.setHeader(this._getPopupHeader(header));

        this.iw_modal_popup.cfg.setProperty('zIndex', '9999');

        if ( replaceNewlines ) {
            this.iw_modal_popup.setBody(content.replace(/\n/g, '<br>'));
        } else {
            this.iw_modal_popup.setBody(content);
        }
        this.iw_modal_popup.render(document.body);

        // Hide the quickhelp popup for good measure
        this.iw_popup_panel.hide();
        if (theWidth) {
            this.iw_modal_popup.cfg.setProperty('width', theWidth + 'px');
        }
        if ( show ) {
            this.showModalPopup(minWidth);
        } else if ( minWidth ) {
            this._correctModalPopupWidth(minWidth);
        }
    },

    _correctModalPopupWidth: function ( minWidth ) {
        var p = this.iw_modal_popup,
        w = p.element.clientWidth;

        minWidth = minWidth || 125;

        if ( w > ( YAHOO.util.Dom.getViewportWidth() * 0.85 ) ) {
            // This is for older IE's that just maximize the width
            w = 650;
        }

        w = Math.max(w, minWidth);

        p.cfg.setProperty('width', w + 'px');
    },

    showModalPopup: function ( minWidth ) {
        this.iw_modal_popup.show();
        //Forcing the z-Index back where it should be, YUI was being wrong, bug #1855
        this.iw_modal_popup.cfg.setProperty('zIndex', '9999');
        this._correctModalPopupWidth(minWidth);
        this.iw_modal_popup.center();
    },

    _getPopupHeader: function ( header ) {
        return '<table><tr>' +
        '<td class="l">' +
        // span hack to make IE 6 pay attention to white-space: nowrap
        '<span style="white-space: nowrap">' + header + '</span>' +
        '</td>' +
        '<td class="r">&nbsp;</td>' +
        '</tr></table>';
    },

    /**
     * Displays a draggable YUI panel
     *
     * So, this feels a bit shitty but here's what's going on.  When you try to
     * append a YUI panel to a node it will inheret all of its CSS properties,
     * which makes perfect sense.  BUT, this means that how the panels look is
     * totally unpredictable so it's a problem.  What we need to do is get the
     * XY position of the element and create the panel at that location.
     *
     * @param string      content
     * @param string      header
     * @param HTMLElement el        (see above) Element you want to the panel to
     *                              showup near, give or take.
     * @param boolean     dont_show (optional).  Just build the panel, don't show it
     * @param boolean     center    (optional).  Center the panel on screen, or put relative
     *                              to el.
     */
    iw_popup: function ( content, header, el, dont_show, center ) {
        var xy_config;

        if ( !center ) {
            center = false; // default centering
        }

        dont_show = Boolean(dont_show);  // make sure we're a boolean.

        if ( YAHOO.lang.isNull(iw_lib.iw_popup_panel) ) {
            iw_lib.iw_popup_panel = new YAHOO.widget.Panel("iw-popup", {
                constraintoviewport: true,
                underlay: "shadow",
                visible: false,
                close: true,
                draggable: true,
                fixedcenter: center,
                width: 350 + "px"});
            YAHOO.util.Dom.addClass(iw_lib.iw_popup_panel.element, 'iw-popup');

            // Subscribe an anonymous function to clear the body when hiding
            iw_lib.iw_popup_panel.hideEvent.subscribe(
                function () {
                    this.setBody('');
                    this.setHeader('');
                }
            );
        }
        this.iw_popup_panel.setHeader(this._getPopupHeader(header));

        if ( center ) {
            iw_lib.iw_popup_panel.center();
        } else {
            xy_config = [ YAHOO.util.Dom.getY(el) + 10,
                        YAHOO.util.Dom.getY(el) + 10 ];
            iw_lib.iw_popup_panel.cfg.setProperty('xy', xy_config);
        }

        iw_lib.iw_popup_panel.cfg.setProperty('zIndex', '10000');
        iw_lib.iw_popup_panel.setBody(content.replace(/\n/g, '<br>'));
        iw_lib.iw_popup_panel.render(document.body);

        // sometimes we may want to build the panel, not now show it right away.
        if ( dont_show !== true ) {
            iw_lib.iw_popup_panel.show();
        }
    },

    resetPopupWidth: function ( center ) {
        iw_lib.iw_popup_panel.cfg.setProperty('width', 'auto');
        if ( center ) {
            iw_lib.iw_popup_panel.center();
        }
    },

    banner_panel: null,

    /**
     */
    iw_message_banner: function ( text, textclass ) {
        var YUI = YAHOO.util,
        Dom = YUI.Dom,
        E = YUI.Event,
        message = document.getElementById('iw-mb-content'),
        el = document.getElementById('iw-mb'),
        iframe = document.getElementById('iw-mb-iframe'),
        UA = YAHOO.env.ua,
        bIEQuirks = (UA.ie === 6 || (UA.ie === 7 && document.compatMode === "BackCompat")),
        region = Dom.getClientRegion(),
        Panel,
        x,
        Message;

        if ( !text ) {
            return;
        }

        text = text.replace(/\n/g, '<br>');

        iw_lib.updateIWMessage({ el: message, new_class: textclass, message: text });

        // Half the width of the viewport, minus half the width of the panel
        x = Dom.getViewportWidth() / 2 - 300;

        if ( this.banner_panel ) {
            Panel = this.banner_panel;
        } else {
            Panel = new YAHOO.widget.Panel('iw-mb', {
                close: true,
                underlay: 'none',
                draggable: false,
                width: '600px',
                effect:{ effect:YAHOO.widget.ContainerEffect.FADE,duration:0.35 },
                visible: false
            });
            this.banner_panel = Panel;
        }

        function ieScroll()
        {
            region = Dom.getClientRegion();

            Dom.setStyle(iframe, 'top', region.top + 'px');
            Dom.setStyle(el, 'top', region.top + 'px');
        }

        Panel.render(document.body);

        if ( bIEQuirks === true ) {
            //Dom.setStyle( iframe, 'display', 'block' );
            Dom.setStyle(iframe, 'top', region.top + 'px');
            Dom.setStyle(iframe, 'left', x + 'px');
            Dom.setStyle(iframe, 'width', el.clientWidth);
            Dom.setStyle(iframe, 'height', el.clientHeight);
            Dom.setStyle(iframe, 'position', 'absolute');
            Dom.setStyle(el, 'top', region.top + 'px');
            Dom.setStyle(el, 'position', 'absolute');
            Dom.setStyle(el, 'left', x + 'px');
            E.on(window, 'scroll', ieScroll);

            Panel.showEvent.subscribe(function () {
                Dom.setStyle(iframe, 'display', 'block');
            });

            Panel.beforeHideEvent.subscribe(function () {
                Dom.setStyle(iframe, 'display', 'none');
                E.removeListener(window, 'scroll', ieScroll);
            });
        } else {
            Dom.setStyle(el, 'left', x + 'px');
        }//end if

        Panel.show();
        Panel.bringToTop();

        if ( textclass === 'iw-positive' ) {
            setTimeout(function () {
                Panel.hide();
            }
            , IW.getSuccessBannerTimeout());
        }
    },

    showSpinner: function () {
        this.iw_message_banner(
            '<div class="center"><img src="/img/spinner.gif"/></div>',
            'iw-neutral'
        );
    }
};

// This makes it possible to easily "extend" existing objects
if ( typeof Object.beget !== 'function' ) {
    Object.beget = function ( o ) {
        var F       = function () {};
        F.prototype = o;
        return new F();
    };
}

// Add the array indexOf function for ie6
if ( !Array.indexOf ) {
    Array.prototype.indexOf = function ( obj ) {
        var i, length = this.length;
        for (i=0; i<length; i++) {
            if ( this[i] === obj ) {
                return i;
            }
        }
        return -1;
    };
}

/**
 * OLD FUNCTIONS START HERE - NEWER FUNCTIONS BELONG ABOVE THIS
 */

/**
 * simple browser detection.
 */

var iw_body = document.getElementsByTagName('body')[0];
var isOpera = YAHOO.util.Dom.hasClass(iw_body, 'iw_opera');
var isIE    = YAHOO.util.Dom.hasClass(iw_body, 'iw_ie');
var isMoz   = YAHOO.util.Dom.hasClass(iw_body, 'iw_ff') ||
              YAHOO.util.Dom.hasClass(iw_body, 'iw_moz');

/**
 * function used ton confirm multi-checkbox actions
 * via a select onchange call
 * @param selectobj: the select object that triggers the submit
 * @param checkboxesVarname: the string that is the name of the checkboxes
 *        involved
 * @param boxesText: text that describes what the list of checkboxes actually is
 *                   (used in confirm dialog)
 * @param confirmText: almost always ##LG_CONFIRM_ACTION##
 * @param emptySetText: text to alert user with if no boxes are checked

 */

function confirmSubmitCheckbox(
    selectobj,
    checkboxesVarname,
    boxesText,
    confirmText,
    emptySetText,
    xhrrequest
) {
    var boxes, boxes_length, formobj, onecheck, answer, action, i, params;

    if ( YAHOO.lang.isUndefined(boxesText) ) {
        // it's "the new way".  I know this is ghetto.  Will change at some point.
        params            = checkboxesVarname;
        selectobj         = params.selectobj;
        checkboxesVarname = params.checkname;
        boxesText         = params.boxtext;
        confirmText       = params.confirmtext;
        emptySetText      = params.emptytext;
        xhrrequest        = params.xhrrequest;
    }
    // check the params
    if ( !selectobj || typeof selectobj !== 'object' ) {
        alert("invalid 'selectobj' param.  should be a SELECT object");
        return false;
    }

    formobj = selectobj.form;

    if ( !formobj || typeof formobj  !== 'object' ) {
        alert("invalid 'formobj' param.  should be a FORM object");
        return false;
    }

    if ( !checkboxesVarname || typeof checkboxesVarname !== 'string' ) {
        alert("invalid 'checkboxesVarname' param.  Should be a string, the name of the checkboxes var.");
        return false;
    }
    if ( !boxesText || typeof boxesText !== 'string' ) {
        alert("invalid 'boxesText' param.  Should be a string, language entry decribing the list of checkboxes.");
        return false;
    }
    if ( !confirmText || typeof confirmText !== 'string' ) {
        alert("invalid 'confirmText' param.  Should be a string.");
        return false;
    }
    if ( !emptySetText || typeof emptySetText !== 'string' ) {
        alert("invalid 'emptySetText' param.  Should be a string.");
        return false;
    }

    // first make sure something is checked.
    boxes        = document.getElementsByName(checkboxesVarname);
    boxes_length = boxes.length;
    onecheck     = false;

    for (i = 0; i < boxes_length; i++) {
        if ( boxes[i].checked ) {
            onecheck = true;
            break;
        }
    }

    if ( !onecheck ) {
        alert(emptySetText);
        selectobj.selectedIndex = 0;
        return( false );
    }

    answer = confirm(confirmText + " : " +
             selectobj.options[selectobj.selectedIndex].text.toUpperCase() +
               " " + boxesText + "?");

    if ( answer ) {
        action = selectobj.options[selectobj.selectedIndex].value;
        if ( xhrrequest && xhrrequest.getAction() === action ) {
            selectobj.disabled = 1;
            xhrrequest.route();
        } else {
            formobj.submit();
            selectobj.disabled = 1;
        }
    } else {
        selectobj.selectedIndex = 0;
        return false;
    }

}

/**
 * popup to confirm a deletion.
 *
 */

function confirmDelete( form, conditions, message )
{

}

/**
 * popUp a window given the URL
 *
 * @param URL
 * @param window name
 * @param width of window
 * @param height"
 */

function popUp(webpage, windowName, width, height)
{
    window.open(
        webpage,
        windowName,
        "scrollbars=yes,width="+width+", height="+height
    );
}

/**
 * highlight the bad forms elements on the page.
 *
 * @param list of elements.
 */
function highlightBadFormElem( /* variable no. of arguments */ )
{
    var elem, i, arg_length;
    if ( arguments.length < 1 ) {
        return;
    }
    arg_length = arguments.length;
    for (i = 0; i < arg_length; i++) {
        // Figure out if we were passed the object itself, or the object
        // name

        if ( typeof( arguments[i] ) === 'string' ) {
            elem = document.getElementById(arguments[i]);

            if ( elem !== null ) {
                elem.className = 'bad_form_element';
            }
        }
    }

    // Focus on the first element
    if ( typeof( arguments[0] ) === 'string' ) {
        document.getElementById(arguments[0]).focus();
    } else {
        arguments[0].focus();
    }
}

/**
 * remove the highlight from bad forms elements on the page.
 *
 * @param list of elements.
 */
function removeHighlightBadFormElem( /* variable no. of arguments */ )
{
    var elem, i, arg_length;
    if ( arguments.length < 1 ) {
        return;
    }

    arg_length = arguments.length;
    for (i = 0; i < arg_length; i++) {
        // Figure out if we were passed the object itself, or the object
        // name

        if ( typeof( arguments[i] ) === 'string' ) {
            elem = document.getElementById(arguments[i]);

            if ( elem !== null ) {
                elem.className = 'textbox';
            }
        }
    }

    // Focus on the first element
    if ( typeof( arguments[0] ) === 'string' ) {
        document.getElementById(arguments[0]).focus();
    } else {
        arguments[0].focus();
    }
}

/**
 * set checkboxes in the given form
 * to checked or unchecked.
 *
 * @param form
 * @param checkbox gropu name
 * @param true = checked.
 */

function setCheckboxes(formname, boxname, checkit)
{

    var boxes = document.forms[ formname ].elements[ boxname ],
      boxes_length = boxes.length,
      i;

    if ( !YAHOO.lang.isArray(boxes) ) {
        if ( !boxes.disabled ) {
            boxes.checked = checkit;
        }
    } else {
        for (i = 0; i < boxes_length; i++) {
            if ( !boxes[i].disabled ) {
                boxes[i].checked = checkit;
            }
        }
    }

    return( false );
}


function textboxSelect(oTextbox, iStart, iEnd)
{
    switch (arguments.length) {
        case 1:
            oTextbox.select();
      break;

        case 2:
            iEnd = oTextbox.value.length;
      break;

        case 3:
            iEnd = oTextbox.value.length;
            if ( isIE ) {
                var oRange = oTextbox.createTextRange();
                oRange.moveStart("character", iStart);
                oRange.moveEnd("character", -oTextbox.value.length + iEnd);
                oRange.select();
            } else if ( isMoz ) {
                oTextbox.setSelectionRange(iStart, iEnd);
            }
    }

    oTextbox.focus();
}

function textboxReplaceSelect(oTextbox, sText)
{
    var oRange, iStart;
    if ( isIE ) {
        oRange      = document.selection.createRange();
        oRange.text = sText;
        oRange.collapse(true);
        oRange.select();
    } else if ( isMoz ) {
        iStart         = oTextbox.selectionStart;
        oTextbox.value = oTextbox.value.substring(0, iStart) + sText + oTextbox.value.substring(oTextbox.selectionEnd, oTextbox.value.length);
        oTextbox.setSelectionRange(iStart + sText.length, iStart + sText.length);
    }

    oTextbox.focus();
}

function autocompleteMatch(sText, arrValues)
{
    var arr_length = arrValues.length,
        i;

    for (i = 0; i < arr_length; i++) {
        if ( arrValues[i].indexOf(sText) === 0) {
            return arrValues[i];
        }
    }

    return null;
}

function autocomplete(oTextbox, oEvent, arrValues)
{
    var iLen, sMatch;

    switch (oEvent.keyCode) {
        case 38: //up arrow
        case 40: //down arrow
        case 37: //left arrow
        case 39: //right arrow
        case 33: //page up
        case 34: //page down
        case 36: //home
        case 35: //end
        case 13: //enter
        case 9:  //tab
        case 27: //esc
        case 16: //shift
        case 17: //ctrl
        case 18: //alt
        case 20: //caps lock
        case 8:  //backspace
        case 46: //delete
      return true;

        default:
            textboxReplaceSelect(oTextbox, String.fromCharCode(isIE ? oEvent.keyCode : oEvent.charCode));
            iLen = oTextbox.value.length;

            sMatch = autocompleteMatch(oTextbox.value, arrValues);

            if ( sMatch !== null ) {
                oTextbox.value = sMatch;
                textboxSelect(oTextbox, iLen, oTextbox.value.length);
            }

      return false;
    }//end switch
}

/**
 * disable this component and submit
 */
function disableMe( comp, bval )
{
    comp.disabled = true;
    comp.value    = bval;
}
