/*jslint undef: true, widget: true, browser: true, onevar: true, eqeqeq: true */
/*global YAHOO iw_message iw_lib iw_form IW IW_PAYLOAD_DATA iw_payload_widget iw_js_data */
/**
 * Singleton object (sorta) set up to handle communications. Specifically
 * designed to communicate with xhr.php and receive JSON_Replys
 */

var iw_xhr = function () {
    var _caching = true,
      _url = '/xhr.php',
      _registered_transactions,
      _h,
      _e = {success: {}, failure: {}},
      YUI = YAHOO.util;

  /**
   * Object for keeping track of currently running  transactions
   *
   * format is basically a hash table, with trans_{id_number} as the index
   */
    _registered_transactions = {};

  /**
   * Returns an immutable object containing the reply
   */
    function _reply( c, t, d, ctrl, action )
    {
        d = d || null;
        return {
            getReplyCode: function () {
                return c;
            },
            getReplyText: function () {
                return t;
            },
            getReplyData: function () {
                return d;
            },
            getReplySeries: function () {
                return Math.floor(c / 100) * 100;
            },
            getCtrl: function () {
                return ctrl;
            },
            getAction: function () {
                return action;
            }
        };
    }

  /**
   * Default handler.  Put functions here if you need to handle a specific
   * response globally, such as not being logged in, or permissions issues.
   */
    _h = {

      /**
     * Generic 200 series reply, should not happen.
     */
        handle200series: function ( r ) {
            iw_lib.iw_message(
                'A 200 Series reply was recieved, but the handler did not ' +
                  'do anything with it - perhaps a handle2xx function was forgotten?',
                'Unexpected 200 series reply'
            );
        },

      /**
     * You must be logged in to perform this action.
     */
        handle401: function ( r ) {
            iw_lib.iw_message('##LG_JSONREPLY401DETAIL##', '##LG_JSONREPLY401##');
        },

      /**
     * Your permissions do not allow this action.  Your session may have expired.
     */
        handle403: function ( r ) {
            iw_lib.iw_message('##LG_JSONREPLY403DETAIL##', '##LG_JSONREPLY403##');
        },

        handle507: function ( r ) {
            iw_lib.popupModal('##LG_IN_MAINTENANCE_MODE##', '##LG_MAINTENANCE_MODE##', null, null, 0, 360);
        }
    };

  /**
   * Hands off the reply to the handler, if possible
   * @param object h Handler object
   * @param object r Output of _reply()
   * @return boolean
   */
    function _dispatchHandler( h, r, args )
    {
      // STEP 1:  look for a function specific to the replyCode
      // EXAMPLE: handle201, handle603, etc.
        if ( YAHOO.lang.isFunction(h['handle' + r.getReplyCode()]) ) {
            h['handle' + r.getReplyCode()]( r, args );
            return true;
        // STEP 2:  look for a function that handles an entire series
        // EXAMPLE: handle200series, handle600series
        } else if ( YAHOO.lang.isFunction(h['handle' +
                                      r.getReplySeries() +
                                      'series'])) {
            h['handle' + r.getReplySeries() + 'series']( r, args );
            return true;
        // STEP 3: nothing more specific was defined, so use the default handler
        } else {
            return false;
        }
    }

  /**
   * Handles successful response from the server (whether that response was an
   * error or not is irrelevant)
   *
   * This is also used for file uploads, in which case the o object is a little
   * different.  As of 2/15/2012, we don't use any of the information that's not
   * in both responses, so it's ok for now.
   *
   * @param object o Response object from YUI
   */
    function _handleSuccess( o )
    {
        var temp, r, h, args;

        if ( iw_js_data.iw_env ) {
            if ( iw_js_data.iw_env.is_dev_mode_js ) {
                IW.debug.io[ o.tId ].response = o;
            }
        }

        try {
            iw_xhr.unregisterTransaction(o.tId);
            temp   = YAHOO.lang.JSON.parse(o.responseText);
            temp.d = temp.d || null;

            if ( iw_js_data.iw_env.is_dev_mode_js ) {
                IW.debug.io[ o.tId ].responseJsonParsed = temp;
            }

            r    = _reply(temp.c, temp.t, temp.d, temp.ctrl, temp.action);
            h    = o.argument.h;
            args = o.argument.args;

            if ( _dispatchHandler(h, r, args) ) {
                return;
            } else if ( _dispatchHandler(_h, r, args) ) {
                return;
            } else {
                iw_xhr.handleError(r);
            }
        } catch ( e ) {
            if ( YAHOO.lang.isFunction(o.argument) ) {
                iw_xhr.handleError(_reply(501, '##LG_JSONUNABLETOPARSE## JSONReply'));
            } else {
                //throw(e);
                iw_xhr.handleError(_reply(600, '<br/>Uncaught Javascript Exception' +
                                         '<br/>' +
                                         'File: ' + e.fileName +
                                         '<br/>' +
                                         'Line: ' + e.lineNumber +
                                         '<br/>' +
                                         e.name + ': ' + e.message));
            }
        }//end try
    }

  /**
   * Triggered if the xhr.php script fails to respond completely
   * @param object Response object from YUI
   */
    function _handleFailure( o )
    {
        var r, h, args;

        if ( iw_js_data.iw_env.is_dev_mode_js ) {
            IW.debug.io[ o.tId ].response = o;
        }

        if ( o.status === -1 ) {
            alert(o.tId);
            return;
        }

        r    = _reply(504, '##LG_JSONREPLY504##');
        h    = o.argument.h;
        args = o.argument.args;

        if ( IW.is_unloading === false ) {
            if ( _dispatchHandler(h, r, args) ) {
                return;
            } else if ( _dispatchHandler(_h, r, args) ) {
                return;
            } else {
                iw_xhr.handleError(r);
            }
        }
    }

  /**
   * Dynamically provides the callback handler for YUI.
   *
   * @param object h Handler object
   * @return object
   */
    function _callback( h, args )
    {
        return  { success: _handleSuccess,
            upload: _handleSuccess,
            failure: _handleFailure,
            cache: _caching,
            argument: { 'h': h, 'args': args }
        };
    }

  // Return an object containing the public methods and properties
    return {
      /**
     * Gets the response from the server and handles it using the handler
     *
     * @param object h     Handler object with functions for dealing with results
     * @param mixed  input Input for the JSON_Responder
     * @param mixed  args  Arguments to pass back to handler function
     */
        getReply: function ( h, input, args, form ) {
            var i, query_string, trans, has_file, io, form_len, elem, j;
            input = input || {};
            args  = args || null;
            form  = form || null;
            if ( YAHOO.lang.isUndefined(h.responder) ) {
                throw "h.responder was not a defined";
            }

            input.where_was_i = window.location.pathname + window.location.search;

            i = { r: h.responder,
                i: input };
            i = YAHOO.lang.JSON.stringify(i);
            i = escape(i);
            i = i.replace(/\+/g, '%2B');

            if ( !YAHOO.lang.isNull(form) ) {
                has_file = false;

                elem = form.elements;
                form_len = elem.length;
                for (j = 0; j < form_len; j++) {
                    if ( elem[j].type && elem[j].type === 'file' ) {
                        has_file = true;
                        break;
                    }
                }

                YAHOO.util.Connect.setForm(form, has_file, has_file);
            }
            trans = YAHOO.util.Connect.asyncRequest('POST', _url, _callback(h, args), 'i=' + i);
            if ( iw_js_data.iw_env ) {
                if ( iw_js_data.iw_env.is_dev_mode_js ) {
                    io = { responder: h.responder,
                        input: input,
                        args: args,
                        form: form,
                        response: null };

                    IW.debug.io[ trans.tId ] = io;
                }
            }

            iw_xhr.registerTransaction(trans);
        },

        subscribeTo: function ( callback, action, ctrl, type ) {
            var _event;

            ctrl = ctrl || location.pathname;
            type = type || 'success';

          // Instantiate the category if missing
            if ( YAHOO.lang.isUndefined(_e[ type ][ ctrl ]) ) {
                _e[ type ][ ctrl ] = {};
            }

          // Instantiate the Event if missing
            if ( YAHOO.lang.isUndefined(_e[ type ][ ctrl ][ action ]) ) {
                _e[ type ][ ctrl ][ action ] = new YUI.CustomEvent(type + '_' + ctrl + '_' + action);
            }

            _event = _e[ type ][ ctrl ][ action ];

            _event.subscribe(callback);
        },

        events: _e,

      /**
     *
     */
        abortTransactions: function ( e ) {
            var trans;
            for (trans in _registered_transactions) {
                if ( _registered_transactions.hasOwnProperty(trans) ) {
                    YAHOO.util.Connect.abort(_registered_transactions[ trans ]);
                }
            }
        },

        beforeUnload: function () {
            IW.is_unloading = true;
            iw_xhr.abortTransactions();
        },

      /**
     * Central error handler for default responses
     * @param object r Output of _reply()
     */
        handleError: function ( r ) {
            var dump = 'replyCode: ' + r.getReplyCode() + "\n" +
                 'replyText: ' + r.getReplyText() + "\n" +
                 'replyData: ' + "\n" + r.getReplyData();

            iw_lib.iw_message(dump);
        },

      /**
     *
     */
        registerTransaction: function ( obj ) {
         // alert( 'registered ' + obj.tId );
            _registered_transactions['trans_' + obj.tId ] = obj;
        },

      /**
     *
     */
        unregisterTransaction: function ( tId ) {
          //alert( 'unregistering ' + tId );
            delete _registered_transactions['trans_' + tId ];
        },

      /**
     * Sets whether or not the service will cache results.
     *
     * @param boolean bool
     */
        setCaching: function ( bool ) {
            _caching = bool;
        }
    };
}();

/**
 * A special handler that is used to handle responses from a Controller/Action
 * combination xhr request.
 */
var iw_ctrl_handler = {
    responder: 'Controller',

    isMultistepOpen: function () {
        if ( document.getElementById('iw-multistep-process') ) {
            return true;
        }
        return false;
    },

  /**
   * A 200 is a successful response
   */
    handle200: function ( r, callback ) {
        if ( YAHOO.lang.isFunction(callback.callback) ) {
            callback.callback(r, true, callback.args);
        }
    },

  /**
   * A 201 is a successful response
   */
    handle201: function ( r, callback ) {
        iw_ctrl_handler.handle200(r, callback);
    },

  /**
   * A 204 means somethign went wrong
   */
    handle204: function ( r, callback ) {
        if ( YAHOO.lang.isFunction(callback.callback) ) {
            callback.callback(r, false, callback.args);
        }
    },

    handle208Event: new YAHOO.util.CustomEvent('handle208'),
    handle210Event: new YAHOO.util.CustomEvent('handle210'),
    handle211Event: new YAHOO.util.CustomEvent('handle211'),

  /**
   * A 208 means the reply data is Form HTML.
   */
    handle208: function ( r, callback ) {
        var shell,
        hide,
        form,
        data = r.getReplyData(),
        actions, action,
        first_input,
        popup;

      // very important this is reset see bug #1971
        iw_form.inputPropertyCache = {};

        iw_form.enableSubmit(callback.form);

        if ( iw_ctrl_handler.isMultistepOpen() ) {
            iw_ctrl_handler.setMultistepFrameContent(data.html);
            popup = document.getElementById('iw-multistep-frame');
        } else {
            iw_lib.popupModal(data.html, r.getReplyText(), false, false, data.min_w);
            popup = iw_lib.iw_modal_popup.body;
        }

        shell = YAHOO.util.Dom.getElementsByClassName('iw-form-shell', 'div', popup);

        iw_form.initSubmits(shell);
        iw_form.initEventHandlers(shell, true);
        iw_form.initGroupClickHandlers(shell);
        iw_form.initUnlimited();
        iw_form.initShowSelect();
        iw_form.initTooltips(shell);
        YAHOO.util.Event.delegate(shell[0], 'click', function () {
            iw_lib.iw_modal_popup.hide(); }, 'input.iw-form-cancel');

        if ( data.js ) {
            YAHOO.util.Get.script(data.js);
        }

        iw_lib.showModalPopup();
        iw_form.initGroupInitialStatus();
        form     = iw_form.getFormFromShell(shell[0]);
        actions  = iw_form.getFormActions(form);

        first_input = YAHOO.util.Dom.getElementsByClassName('iw-first-visible', null, form)[0];
        if ( first_input ) {
            first_input.focus();
        } else {
            iw_form.getSubmitButton(form).focus();
        }

        hide = function () {
            iw_lib.iw_modal_popup.hide();
        };

        for (action in actions) {
            if ( actions.hasOwnProperty(action) ) {
                action = actions[ action ];
                iw_xhr.subscribeTo(hide, action, iw_form.getCtrl(form));
            }
        }
      //
        this.handle208Event.fire({ name:form.name,
            form:form,
            shell:shell,
            reply: r });
      //
    },

    setMultistepFrameContent: function ( content ) {
        document.getElementById('iw-multistep-frame').innerHTML = content;
        iw_lib.resetModalPopupWidth();
    },

  /**
   * A 209 means data is a "listing" or "printout" of misc data.
   */
    handle209: function ( r, callback ) {
        var message, data = r.getReplyData();

        message = '<div id="iw-printout">';

        if ( data.js ) {
            YAHOO.util.Get.script(data.js);
        }

        if ( data.js_data ) {
            if ( !iw_js_data.printout ) {
                iw_js_data.printout = {};
            }
            iw_js_data.printout.data = data.js_data;
        }

        if ( data.explanation ) {
            message = message + '<p>' + data.explanation + '</p>';
        }

        if ( data.is_textarea ) {
            message = message + '<textarea cols="50" rows="20">' + data.printout + '</textarea>';
        } else {
            message = message + '<pre>' + data.printout + '</pre>';
        }

        message = message + '</div>';

        iw_lib.popupModal(message, r.getReplyText());
    },

    handle210: function ( r, callback ) {
        var data = r.getReplyData(), payload, html;

        if ( !document.getElementById(data.id) ) {
          // Dumb, but needed - the element of the payload itself doesn't exist yet, so
          // this needs to happen here, even though the payload callback will do something
          // very similar.
            html = '<h1>' + data.title + '</h1>' + data.payload;

            if ( iw_ctrl_handler.isMultistepOpen() ) {
                iw_ctrl_handler.setMultistepFrameContent(html);
            } else {
                iw_lib.popupModal(html, data.title, false);
            }
        }

        IW_PAYLOAD_DATA[ data.id ] = data.input;

        if ( data.js ) {
            YAHOO.util.Get.script(data.js);
        }

        payload = iw_payload_widget(data.id);

        payload.callback(r, true);

        iw_form.initSubmits([payload.getElement()]);

        this.handle210Event.fire({ name:data.id,
            payload: payload,
            reply: r });
    },

    handle211: function ( r, callback ) {
        var data = r.getReplyData(), html, step, current_class, id, style,
        step_length, ctrl;

        html  = '<div id="iw-multistep-process">';
      //html += '<h1>' + data.label + '</h1>';
        html += '<table id="iw-multistep-steps"><tr>';
        current_class = ' class="iw-multistep-current"';

        step_length = data.steps.length;

        for (step in data.steps) {
            if ( data.steps.hasOwnProperty(step) ) {
                step = data.steps[ step ];

                id = 'iw-step-' + step.controller + '-' + step.action;
                style = ' style="z-index:' + step_length + '"';
                step_length--;

                html += '<td id="' + id + '"' + current_class + ' data-iw-action="' + step.action +
                '" data-iw-ctrl="' + step.controller + '">' +
                step.label + '</td>';
                current_class = '';
            }
        }

        html += '</tr></table>';
        html += '<div id="iw-multistep-frame"><div class="iw-loading"></div></div>';
        html += '</div>';

        iw_lib.popupModal(html, data.label, false);

        ctrl = data.steps[0].controller;

        if ( !ctrl ) {
            ctrl = iw_form.getCtrlFromUrl(window.location.pathname);
        }

        this.handle211Event.fire({ data: data });

      // TODO: This possibly needs input for some actions.
        iw_ctrl_handler.execute(null, function (){}, ctrl, data.steps[0].action);

    },

  /**
   * A 500 series means...
   */
    handle500series: function ( r, callback ) {
        iw_ctrl_handler.handle700series(r, callback);
    },

    handle507: function ( r, callback ) {
        iw_lib.popupModal('##LG_IN_MAINTENANCE_MODE##', '##LG_MAINTENANCE_MODE##', null, null, 0, 360);
    },

  /**
   * A 700 series means...
   */
    handle700series: function ( r, callback ) {
        if ( callback.form ) {
            iw_form.clearStatus(iw_form.getSubmitButton(callback.form));
            iw_form.getSubmitButton(callback.form).disabled = 0;
        }
        iw_xhr.handleError(r);
    },

  /**
   * Fire off the request to iw_xhr
   *
   * Callback can have two forms - the first form is to just be a function itself.
   * In this form, the callback function is called with two arguments; the reply,
   * and a boolean value indicating true or false.
   *
   * The second form is an object, like this
   *
   * {
   *   callback: function(),
   *   args: mixed
   * }
   *
   * In this second form, the args parameter is simply passed along to the
   * callback function as the third parameter.  Doesn't matter what it is.
   *
   * @param element         form     HTML form element
   * @param function|object callback see above
   * @param string|null     ctrl     controller path
   * @param string|null     action   action function name
   * @param string|null     input    additional input to the controller action
   */
    execute: function ( form, callback, ctrl, action, input ) {
        var new_callback = {};

        if ( YAHOO.lang.isNull(ctrl) || YAHOO.lang.isUndefined(ctrl) ) {
            ctrl   = form.iw_ctrl_name.value;
            action = form.iw_action_name.value;
        }

        if ( YAHOO.lang.isFunction(callback) ) {
            new_callback.callback = callback;
            new_callback.args     = null;
            new_callback.form     = form;
        } else {
            new_callback.callback = callback.callback;
            new_callback.args     = callback.args;
            new_callback.form     = form;
        }

        new_callback.ctrl   = ctrl;
        new_callback.action = action;

        input   = input || {};
        input.c = ctrl;
        input.a = action;
        input.iw_sess_hint = IW.getIWEnv();

        if ( !input.iw_payload_output ) {
            input.iw_payload_output = 'html';
        }

        if ( form && form.iw_payload_output ) {
            input.iw_payload_output = form.iw_payload_output.value;
        }

        iw_xhr.getReply(iw_ctrl_handler, input, new_callback, form);
    }
};

var iw_multistep = function ( eventType, args ) {
    var data = args[0].data, _handle208, _handle210, steps = data.steps,
      final_step = data.final_step,
      list = document.getElementById('iw-multistep-steps'),
      previous_step_el, current_step_el, _markForm, _markCurrentAction, id,
      _tearDown, _setUp;

    _handle208 = function ( eventType208, args208 ) {
        var form = args208[0].form, reply = args208[0].reply;

        _markForm(form);
        _markCurrentAction(reply);
    };

    _handle210 = function ( eventType210, args210 ) {
        var payload = args210[0].payload,
        reply = args210[0].reply;

        _markForm(payload.getForm());
        _markCurrentAction(reply);
    };

    _markForm = function ( form ) {
        if ( iw_form.getAction(form) !== final_step.action ) {
            iw_form.getSubmitButton(form).value = '##LG_NEXT##';
        }
    };

    _markCurrentAction = function ( reply ) {
        var ctrl = reply.getCtrl(), action = reply.getAction();

        if ( previous_step_el ) {
            YAHOO.util.Dom.removeClass(previous_step_el, 'iw-multistep-previous');
        }

        if ( current_step_el ) {
            YAHOO.util.Dom.removeClass(current_step_el, 'iw-multistep-current');
        }

        id = 'iw-step-' + ctrl + '-' + action;

        current_step_el  = document.getElementById(id);
        previous_step_el = YAHOO.util.Dom.getPreviousSibling(current_step_el);

        if ( previous_step_el ) {
            YAHOO.util.Dom.addClass(previous_step_el, 'iw-multistep-previous');
        }

        YAHOO.util.Dom.addClass(current_step_el, 'iw-multistep-current');
    };

  /**
   * Teardown function to clean up after ourselves.
   */
    _tearDown = function () {
        iw_ctrl_handler.handle208Event.unsubscribe(_handle208);
        iw_ctrl_handler.handle210Event.unsubscribe(_handle210);
        iw_lib.iw_modal_popup.hideEvent.unsubscribe(_tearDown);
    };

  /**
   * Sets up the event handlers for dealing with popups.
   */
    _setUp = function () {
        iw_ctrl_handler.handle208Event.subscribe(_handle208);
        iw_ctrl_handler.handle210Event.subscribe(_handle210);
        iw_lib.iw_modal_popup.hideEvent.subscribe(_tearDown);
    }();// <-- Auto-called.
};

iw_ctrl_handler.handle211Event.subscribe(iw_multistep);

//var links = document.getElementsByTagName('a');
YAHOO.util.Event.on(window, 'beforeunload', iw_xhr.beforeUnload);
