/*jslint undef: true, widget: true, browser: true, onevar: true, eqeqeq: true */
/*global YAHOO,
         iw_xhr,
         iw_lib,
         iw_js_data,
         iw_transferlist_dropdown_handler,
         iw_reseller_info_handler,
         update_ipv6_pool_groups,
         update_ssl_groups,
         confirmSubmitCheckbox,
 */
var iw_reseller_info,
    ipv6_pool_groups,
    ipv4_groups,
    ipv6_groups,
    ipv6_pool_option,
    ssl_groups,
    ipv4_state = [],
    ipv6_state = [],

iw_reseller_info_handler = {
    responder: 'Transferlist',

    /**
     * Handle XHR request success.
     */
    handle201: function ( r ) {
        var result = r.getReplyData();

        /** The Global **/
        iw_reseller_info = result;
        this.populate();

        /** DIRTY HACK FOR SYNC **/
        iw_transferlist_dropdown_handler.init();
    },

    /**
     * Initialize iw_reseller_info object
     */
    init: function () {
        iw_xhr.getReply(this);
    },

    /**
     * Populate import transfer rows based on the reseller info which was grabbed
     * via XHR and placed into an object level variable.
     */
    populate: function () {
        var nickname, reseller_id, reseller, reseller_info;

        for (reseller in iw_reseller_info) {
            if ( iw_reseller_info.hasOwnProperty(reseller) ) {
                reseller_id = reseller;
                break;
            }
        }//end for

        reseller_info = this.getResellerInfo(reseller_id);

        this.populateRowsByClass('iw-control-rows', reseller_info);
        this.populateRowsByClass('iw-data-rows', reseller_info);
    },

    /**
     * Populates the table rows with the reseller information based on the row's
     * DOM class
     */
    populateRowsByClass: function ( tr_class, reseller_info ) {
        var trs = YAHOO.util.Dom.getElementsByClassName(tr_class, 'tr');
        this.populateRows(trs, reseller_info);
    },

    /**
     * Populates all rows passed in with the reseller information.
     */
    populateRows: function ( trs, reseller_info ) {
        var tr_length = trs.length,
        i;

        for (i = 0; i < tr_length; i++) {
            this.populateRow(trs[i], reseller_info);
        }
    },

    /**
     * Populates given row with reseller information
     */
    populateRow: function ( tr, reseller_info ) {
        var selects = tr.getElementsByTagName('select'),
        checkbox = tr.getElementsByTagName('input')[ 0 ],
        noips = [],
        count = 0,
        k;

        // resellers
        if ( selects[0] ) {
            selects[0].disabled = false;
            this.populateSelect(selects[0], reseller_info.accounts);
        }

        // db servers
        if ( selects[1] ) {
            selects[1].disabled = false;
            this.populateSelect(selects[1], reseller_info.db_servers);
        }

        if ( selects[2] ) {
            selects[2].disabled = false;
            this.populateSelect(selects[2], reseller_info.pools);
        }

        // ip addresses
        if ( selects[3] ) {
            selects[3].disabled = false;

            // we need to see if there is more than 0 ips, and this is the only
            // way to do it since it's an object and not an actual array :(
            for (k in reseller_info.ips) {
                if ( reseller_info.ips.hasOwnProperty(k) ) {
                    count++;
                    break;
                }
            }

            // some ips, populate away
            if ( count > 0 ) {
                this.populateSelect(selects[3], reseller_info.ips);
                if ( checkbox ) {
                    checkbox.disabled = false;
                }
            } else {
                // no ips, say so and disable the row
                selects[3].disabled = true;
                noips.push('No Shared IPs Available');
                this.populateSelect(selects[3], noips);
                if ( checkbox ) {
                    checkbox.disabled = true;
                }
            }
        }//end if selects[3] (ip addresses)

        if ( selects[4] ) {
            selects[4].disabled = false;
            this.populateSelect(selects[4], reseller_info.ipv6s);
        }

        // import type
        if ( selects[5] ) {
            selects[5].disabled = false;
        }
    },

    /**
     * When given a reseller_id, a reseller_info array is built which contains
     * all reseller options, all servers the given reseller has access to, and
     * all IP addresses they have access to.
     */
    getResellerInfo: function ( reseller_id ) {
        var reseller,
        nickname,
        reseller_info_array = [],
        /*
         * Private function, grabs all IP addresses the given reseller ID has
         * access to.
         */
        getIps = function ( reseller_id ) {
            var ipaddr,
              reseller_info_ips = [];

            for (ipaddr in iw_reseller_info[reseller_id].ips) {
                if ( iw_reseller_info[reseller_id].ips.hasOwnProperty(ipaddr) ) {
                    reseller_info_ips[ipaddr] = iw_reseller_info[reseller_id].ips[ipaddr];
                }
            }
            return reseller_info_ips;
        },
        getIPv6 = function ( reseller_id ) {
            var ipv6,
              reseller_info_ipv6s = [];

            for (ipv6 in iw_reseller_info[reseller_id].ipv6s) {
                if ( iw_reseller_info[reseller_id].ipv6s.hasOwnProperty(ipv6) ) {
                    reseller_info_ipv6s[ipv6] = iw_reseller_info[reseller_id].ipv6s[ipv6];
                }
            }
            return reseller_info_ipv6s;
        },
        getPools = function ( reseller_id ) {
            var pool, reseller_info_pools = [];

            for (pool in iw_reseller_info[reseller_id].pools) {
                if ( iw_reseller_info[reseller_id].pools.hasOwnProperty(pool) ) {
                    pool = iw_reseller_info[reseller_id].pools[pool];
                    reseller_info_pools[ pool.value ] = pool.display;
                }
            }

            reseller_info_pools[ 0 ] = '##LG_IPV6_NO_POOL##';

            return reseller_info_pools;
        },
        /*
         * Private function, grabs all database servers the given reseller has
         * access to.
         */
        getDbServers = function ( reseller_id ) {
            var dbserver,
              reseller_info_servers = [];

            for (dbserver in iw_reseller_info[reseller_id].dbservers) {
                if ( iw_reseller_info[reseller_id].dbservers.hasOwnProperty(dbserver) ) {
                    reseller_info_servers[dbserver] = iw_reseller_info[reseller_id].dbservers[dbserver];
                }
            }
            return reseller_info_servers;
        };

        // Because the reseller data always has to be populated with all resellers.
        reseller_info_array.reseller_id = reseller_id;
        reseller_info_array.accounts    = [];
        reseller_info_array.ips         = [];
        reseller_info_array.ipv6s       = [];
        reseller_info_array.db_servers  = [];
        reseller_info_array.pools       = [];

        for (reseller in iw_reseller_info) {
            if ( iw_reseller_info.hasOwnProperty(reseller) ) {
                nickname = iw_reseller_info[reseller].nickname;

                reseller_info_array.accounts[reseller] = nickname;
            }
        }

        reseller_info_array.ips        = getIps(reseller_id);
        reseller_info_array.ipv6s      = getIPv6(reseller_id);
        reseller_info_array.db_servers = getDbServers(reseller_id);
        reseller_info_array.pools      = getPools(reseller_id);

        return reseller_info_array;
    },

    /**
     * Populates the select box given the assoicative array of options.
     */
    populateSelect: function ( select, options ) {
        var i = 0,
        found_current_value = false,
        current_value,
        is_selected,
        option,
        extra_option,
        /**
         * Lamda function to find the create option if needed.
         *
         * I think it's a hack that this function is here, but I can't
         * think of a better way to do this. -jmd
         */
        getImportIfExists = function ( select ) {
            var options = select.options,
              option,
              opt_cnt = options.length,
              i = 0;

            if ( ( select.id.match(/reseller_group\[(.+)\]/) !== null ) ||
              ( select.id.match(/reseller\[(.+)\]/) !== null ) ) {
                /*
                for( option in options ) {
                if( options[option] && options[option].value === "create" ) {
                return options[option];
                }
                }
                 */

                for (i = 0; i < opt_cnt; i++) {
                    if ( options[ i ] && options[ i ].value === "create" ) {
                        return options[ i ];
                    }
                }//end for
            }//end if

            return null;
        };

        extra_option  = getImportIfExists(select);
        current_value = select.value;

        select.options.length = 0;
        select.options.length = 0; // webkit developer tools bug workaround -pko */

        if ( extra_option !== null ) {
            if ( extra_option.value === current_value ) {
                found_current_value = true;
            }
            select.options.add(new Option(extra_option.text, extra_option.value), i);
            i++;
        }//end if
        for (option in options) {
            if ( options.hasOwnProperty(option) ) {
                if ( option === current_value ) {
                    found_current_value = true;
                }

                select.options.add(new Option(options[option], option), i);
                i++;
            }//end if
        }

        if ( found_current_value === true && current_value ) {
            select.value = current_value;
        } else {
            select.value = select.options[0].value;
        }
    }//end function populateSelect
},

/**
 * iw_reseller_perms_updater object
 */
iw_reseller_perms_updater = {
    /**
     * Given the value of the select (the reseller id), populate the rest of the
     * row with the approprate data.
     */
    updateInfoForReseller: function ( select ) {
        var reseller_id = select.value,
        reseller_info,
        first_reseller_id,
        tr = YAHOO.util.Dom.getAncestorByTagName(select, 'tr'),
        validateResellerId = function ( reseller_id ) {
            var reseller;
            /**
             * If the reseller_id is "create" ( Importing the reseller from the other
             * server ), then we give this "imported" reseller the permissions of the
             * first reseller on the list, because they will be the one whith the
             * highest access level.
             */
            if ( reseller_id === "create" ) {
                for (reseller in iw_reseller_info) {
                    if ( iw_reseller_info.hasOwnProperty(reseller) ) {
                        if ( iw_reseller_info[reseller].reseller_id ) {
                            reseller_id = iw_reseller_info[reseller].reseller_id;
                            break;
                        }
                    }
                }//end for
            }//end if( reseller_id === "create" )
            return reseller_id;
        },
        valid_reseller_id = validateResellerId(reseller_id);

        reseller_info = iw_reseller_info_handler.getResellerInfo(valid_reseller_id);
        iw_reseller_info_handler.populateRow(tr, reseller_info);
    }
},

/**
 * iw_transferlist_dropdown_handler object
 */
iw_transferlist_dropdown_handler = {
    /**
     * When the master checkbox is checked, and then a row under it has been
     * unchecked, we are going to want to uncheck the master row, as it is no
     * longer in a valid state.
     */
    toggleMasterCheckbox: function ( e ) {
        var getMasterCheck = function ( element ) {
            var yay_me = YAHOO.util.Event.getTarget(e),
            my_group = yay_me.attributes.checkbox_group.value,
            checkbox_group_master;

            if ( !YAHOO.lang.isUndefined(element.attributes.checkbox_group_master) ) {
                checkbox_group_master = element.attributes.checkbox_group_master.value;
                if ( checkbox_group_master === my_group ) {
                    return true;
                }
            }
            return false;
        },
        master_check = YAHOO.util.Dom.getElementBy(getMasterCheck, 'input');

        master_check.checked = false;
    },

    /**
     * When the master checkbox is checked, we want to check all checkboxes in
     * it's group.
     */
    toggleAllCheckboxes : function ( e ) {
        var getChecks = function ( element ) {
            var master_check, cgm_value, checkbox_group;

            master_check = YAHOO.util.Event.getTarget(e);
            if ( ( element.type === 'checkbox' ) && ( element.disabled === false ) ) {
                cgm_value = master_check.attributes.checkbox_group_master.value;

                if ( element.attributes.checkbox_group ) {
                    checkbox_group = element.attributes.checkbox_group.value;
                }

                return cgm_value === checkbox_group;
            }//end if
            return false;
        },//end function
        checks = YAHOO.util.Dom.getElementsBy(getChecks, 'input'),
        checks_length = checks.length,
        i;

        for (i = 0; i < checks_length; i++) {
            if ( checks[i] ) {
                checks[i].checked = this.checked;
            }//end if
        }//end for
    },//end function

    /**
     * Toggles all Select boxes based on the select box in the "advanced options"
     * row.
     */
    toggleAllSelects : function ( e ) {
        iw_transferlist_dropdown_handler.manualToggleSelects(this);
    },

    /**
     * Toggles all Select boxes based on the select box in the "advanced options"
     * row.
     */
    manualToggleSelects: function ( select ) {
        var tr = YAHOO.util.Dom.getAncestorByTagName(select, 'tr'),
        cell_index = -1,
        i,
        index,
        selects,
        selects_length,
        checkSelectForOption = function ( select, value ) {
            var options_length = select.options.length,
              i;

            for (i = 0; i < options_length; i++) {
                if ( select.options[i] && select.options[i].value === value ) {
                    return true;
                }
            }//end for
        }; //end function

        selects        = tr.getElementsByTagName('select');
        selects_length = selects.length;
        for (i = 0; i < selects_length; i++) {
            if ( selects[i].id.match(/reseller_group\[(.+)\]/)  !== null ) {
                iw_reseller_perms_updater.updateInfoForReseller(selects[i]);
            }
            if ( selects[i].id === select.id ) {
                cell_index = i;
            }
        }//end for

        if ( cell_index === -1 ) {
            return;
        }

        tr = YAHOO.util.Dom.getNextSibling(tr);
        i  = ipv6_pool_groups.indexOf(select);
        if ( i === -1 ) {
            i = ipv6_groups.indexOf(select);
        }
        ssl_groups = YAHOO.util.Selector.query('[data-iw-ssl]');

        while ( YAHOO.lang.isNull(tr) === false ) {
            if ( ipv6_pool_option ) {
                if ( select.value === '0' && checkSelectForOption(ipv6_groups[i], 'pool') === true ) {
                    ipv6_groups[i].remove(1);
                }

                if ( select.value !== '0' && ipv6_pool_groups.indexOf(ipv6_pool_groups[i]) !== -1 ) {
                    if ( checkSelectForOption(ipv6_groups[i], 'pool') !== true ) {
                        ipv6_groups[i].options.add(new Option(
                            ipv6_pool_option.text,
                            ipv6_pool_option.value
                        ), 1);
                    }
                }
            }

            /*
             * Kind of a hack, but if we've hit a seperator row then we know we've
             * changed all the dropdowns for that particular reseller.
             */
            if ( YAHOO.util.Dom.hasClass(tr, 'iw-seperator-rows') === true ) {
                break;
            }

            selects = tr.getElementsByTagName('select');

            index = ipv6_groups.indexOf(selects[cell_index]);
            if ( index !== -1 && ssl_groups[index].value === '1' ) {
                tr = YAHOO.util.Dom.getNextSibling(tr);
                i++;
                continue;
            }

            if ( YAHOO.lang.isUndefined(selects[cell_index]) === false ) {
                if ( checkSelectForOption(selects[cell_index], select.value) === true ) {
                    selects[cell_index].value = select.value;
                }

                /*
                 * If this select box is the reseller select box then we want to fire
                 * the execute event on the row handler to update all the related
                 * select boxes.
                 */
                if ( selects[cell_index].id.match(/reseller\[(.+)\]/) !== null ) {
                    iw_reseller_perms_updater.updateInfoForReseller(selects[cell_index]);
                }//end if
            }//end if

            tr = YAHOO.util.Dom.getNextSibling(tr);
            i++;
        }//end while
    },

    /**
     * Inits the object, but initialization is done when iw_reseller_info_handler
     * has finished it's XHR request.
     */
    init: function () {
        var tr = YAHOO.util.Dom.getElementsByClassName('iw-control-rows', 'tr'),
        tr_length = tr.length,
        ipv4_length,
        ipv6_length,
        ssl_length,
        target_length,
        target_option,
        index,
        selects,
        inputs,
        mapped_id,
        i, j, k,
        updateResellerInfo = function () {
            iw_reseller_perms_updater.updateInfoForReseller(this);
        };

        for (i = 0; i < tr_length; i++) {
            selects = tr[i].getElementsByTagName('select');
            inputs  = tr[i].getElementsByTagName('input');

            // Puts the event handler on the first checkbox
            if ( inputs[0] ) {
                if ( inputs[0].id.match(/checkbox_group\[(.+)\]/) !== null ) {
                    YAHOO.util.Event.on(inputs[0], 'click', this.toggleAllCheckboxes);
                }
                /*
                 * This is what sets the approprate reseller item in the dropdown
                 */
                if ( selects[0].id.match(/reseller_group\[(.+)\]/) !== null ) {
                    mapped_id        = selects[0].attributes.mapped_id.value;
                    selects[0].value = mapped_id;

                    // removes the "bad" attribute, as we no longer need it.
                    selects[0].removeAttribute('mapped_id');
                    iw_reseller_perms_updater.updateInfoForReseller(selects[0]);
                    this.manualToggleSelects(selects[0]);
                }
            }

            if ( selects[4][1] ) {
                ipv6_pool_option = selects[4][1];
            }
            YAHOO.util.Event.on(selects, 'change', this.toggleAllSelects);
        }//end for

        tr        = YAHOO.util.Dom.getElementsByClassName('iw-data-rows', 'tr');
        tr_length = tr.length;
        for (i = 0; i < tr_length; i++) {
            selects = tr[i].getElementsByTagName('select');
            inputs  = tr[i].getElementsByTagName('input');

            if ( inputs[0] ) {
                YAHOO.util.Event.on(inputs[0], 'click', this.toggleMasterCheckbox);
            }

            if ( selects[0] ) {
                if ( selects[0].id.match(/reseller\[(.+)\]/) !== null ) {
                    YAHOO.util.Event.on(selects[0], 'change', updateResellerInfo);
                }
            }
        }//end for

        //
        tr_length = ipv6_groups.length;
        for (i = 0; i < tr_length; i++) {
            if ( ipv6_pool_groups[i][1] ) {
                ipv6_pool_groups[i].selectedIndex = 1;
                ipv6_groups[i].selectedIndex      = 1;
            } else {
                ipv6_pool_groups[i].selectedIndex = 0;
                ipv6_groups[i].remove(1);
            }

            ipv4_length = ipv4_groups[i].length;
            ipv6_length = ipv6_groups[i].length;
            if ( !iw_js_data.iw_can_multissl ) {
                if ( ssl_groups[i].value === '1' ) {
                    for (j = 0; j < ipv4_length; j++) {
                        if ( ipv4_groups[i][j].label.indexOf('##LG_SHARED##') !== -1 ) {
                            ipv4_groups[i].remove(j);
                            ipv4_length = ipv4_groups[i].length;
                            j--;
                        }
                    }
                    for (k = 0; k < ipv6_length; k++) {
                        if ( ipv6_groups[i][k].label.indexOf('##LG_SHARED##') !== -1 ) {
                            ipv6_groups[i].remove(k);
                            ipv6_length = ipv6_groups[i].length;
                            k--;
                        }
                    }
                }
            }
        }//end for

        if ( !iw_js_data.iw_can_multissl ) {
            ssl_length = ssl_groups.length;
            for (i = 0; i < ssl_length; i++) {
                for (j = 0; j < ssl_length; j++) {
                    if ( i !== j && ssl_groups[i].value === '1' && ssl_groups[j].value === '1' ) {
                        target_option = ipv4_groups[i][ ipv4_groups[i].selectedIndex ];
                        target_length = ipv4_groups[j].length;
                        for (k = 0; k < target_length; k++) {
                            if ( target_option !== undefined && ipv4_groups[j][k].value === target_option.value ) {
                                ipv4_groups[j].remove(k);
                                target_length--;
                            }
                        }
                    }
                }
                if ( ssl_groups[i].value === '1' ) {
                    ipv4_groups[i].options.add(new Option('No IPv4 Selected', '-1'), 0);
  
                    if ( ipv4_groups[i].length > 1 ) {
                        ipv4_groups[i].selectedIndex = 1;
                    }
                }
            }//end for
        }

        for (i = 0; i < tr_length; i++) {
            if ( ipv6_groups[i][1] ) {
                ipv6_groups[i].selectedIndex = 1;
            }
            ipv4_state[i] = ipv4_groups[i].selectedIndex;
            ipv6_state[i] = ipv6_groups[i].selectedIndex;
        }

        //

    }//end function init
};

ipv4_groups      = YAHOO.util.Selector.query('select[data-iw-ipv4-group]');
ipv6_groups      = YAHOO.util.Selector.query('select[data-iw-ipv6-group]');
ipv6_pool_groups = YAHOO.util.Selector.query('select[data-iw-ipv6pool-group]');
ssl_groups       = YAHOO.util.Selector.query('[data-iw-ssl]');

iw_reseller_info_handler.init();

YAHOO.util.Event.on('import', 'change',  function ( e ) {
    var select, current_ip, i,
      Dom = YAHOO.util.Dom,
      ips_in_use = {},
      ip_selects = Dom.getElementsByClassName('ipaddr', 'select'),
      boxes = document.getElementsByName('input[users][]');

    for (i in boxes) {
        if ( boxes.hasOwnProperty(i) ) {
            if ( boxes[i].checked === true ) {
                // TIMDO - This needs to be un-broken and updated to work with new stuff
                // select     = document.getElementById( 'ipaddr[' + boxes[i].id + ']' );
                // current_ip = select.value;

                // if( current_ip in ips_in_use ) {
                //   e.target.selectedIndex = 0;
                //   alert( '##LG_CANNOT_SPECIFY_DEDICATED_IP_TWICE##' );
                //   return false;
                // }
                // if( iw_js_data.ips[current_ip].type === 'static' ) {
                //   ips_in_use[current_ip] = current_ip;
                // }
            }
        }
    }

    confirmSubmitCheckbox(
        e.target,
        "input[users][]",
        "##LG_SELECTED_ACCOUNTS##",
        "##LG_CONFIRM_ACTION##",
        "##LG_NOTHING_SELECTED##"
    );
});

//RULES
// a dedicated IP may be used for exactly one siteworx account
// a dedicated IP may be used for exactly one domain using SSL and unlimited other domains
//  in the same siteworx account that do not use SSL
// if 'no pool' is selected, 'pull from pool' may not be an option for ipv6
// when changing from 'no pool' to any pool, all ipv6 within that siteworx account should
//  change to 'pull from pool' if the current value is 'no ipv6'
// secondary domain options need to be updated when reseller changes

update_ssl_groups = function ( e ) {
    var target_select, target_index, ip_group, ip_state, state_length,
      old_option, new_option, select_length, length, i, j, k, add_index;

    target_select = YAHOO.util.Event.getTarget(e);
    if ( ipv4_groups.indexOf(target_select) !== -1 ) {
        ip_group = ipv4_groups;
        ip_state = ipv4_state;
    } else {
        ip_group = ipv6_groups;
        ip_state = ipv6_state;
    }

    target_index = ip_group.indexOf(target_select);
    old_option   = ip_group[ target_index ][ ip_state[ target_index ] ];
    new_option   = target_select[ target_select.selectedIndex ];

    if ( ssl_groups[ target_index ].value !== '1' ) {
        return;
    }

    length = ip_group.length;
    for (i = 0; i < length; i++) {
        if ( i === target_index ) {
            continue;
        }

        if ( ssl_groups[i].value === '1' ) {
            if ( new_option.value !== '-1' && new_option.value !== 'pool' && new_option.value !== '0' ) {
                select_length = ip_group[i].length;
                for (j = 0; j < select_length; j++) {
                    if ( ip_group[i][j].value === new_option.value ) {
                        ip_group[i].remove(j);
                        select_length--;
                    }
                }
            }

            if ( old_option.value !== '-1' && old_option.value !== 'pool' && old_option.value !== '0' ) {
                add_index = ip_group[i].length;
                state_length = ip_group[0].length;
                for (k = 0; k < state_length; k++) {
                    if ( ip_group[0][k].value === old_option.value ) {
                        add_index = k;
                        break;
                    }
                }

                ip_group[i].options.add(new Option(
                    old_option.text,
                    old_option.value
                ), add_index);
            }
        }//end if
    }//end for

    for (i = 0; i < length; i++) {
        ipv4_state[i] = ipv4_groups[i].selectedIndex;
        ipv6_state[i] = ipv6_groups[i].selectedIndex;
    }
};

update_ipv6_pool_groups = function ( e ) {
    var target_select, index;

    if ( ipv6_pool_option ) {
          target_select = YAHOO.util.Event.getTarget(e);
          index = ipv6_pool_groups.indexOf(target_select);

        if ( target_select.value === '0' ) {
            if ( ipv6_groups[index][1].value === 'pool' ) {
                ipv6_groups[index].remove(1);
                index++;
                while ( ipv6_groups[index] && ipv6_groups[index].hasAttribute('data-iw-ipv6-group-secondary') ) {
                    ipv6_groups[index].remove(1);
                    index++;
                }
            }
        } else if ( ipv6_groups[index][1].value !== 'pool' ) {
            ipv6_groups[index].options.add(new Option(
                ipv6_pool_option.text,
                ipv6_pool_option.value
            ), 1);
            if ( ipv6_groups[index].value === '0' ) {
                ipv6_groups[index].selectedIndex = 1;
            }
            index++;

            while ( ipv6_groups[index] && ipv6_groups[index].hasAttribute('data-iw-ipv6-group-secondary') ) {
                ipv6_groups[index].options.add(new Option(
                    ipv6_pool_option.text,
                    ipv6_pool_option.value
                ), 1);
                if ( ipv6_groups[index].value === '0' ) {
                    ipv6_groups[index].selectedIndex = 1;
                }
                index++;
            }
        }//end if
    }//end if
};

YAHOO.util.Event.on(ipv4_groups, 'change',  update_ssl_groups);
YAHOO.util.Event.on(ipv6_groups, 'change',  update_ssl_groups);
YAHOO.util.Event.on(ipv6_pool_groups, 'change',  update_ipv6_pool_groups);
