<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage API
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Nov 30, 2008
 */

/**
 * API Controller class.
 *
 * @package    InterWorx
 * @subpackage API
 */
class IWorx_API_Controller
{

    /**
     * Routes the request to corresponding controller.
     *
     * @param string|array|struct $apikey
     * @param string              $ctrl_name
     * @param string              $action
     * @param string|array|struct $input
     * @return array
     */
    public function route($apikey, $ctrl_name, $action, $input)
    {
        try {
            if (is_array($apikey)) {
                $apikey['_ctrl_name_'] = $ctrl_name; // sessionid auth needs this detail
                                             // to know what type of session we
                                             // want
            }

            if ($input === null) {
                $input = [];
            }

            foreach ($input as &$in) {
                if ($in === true) {
                    $in = '1';
                }
                if ($in === false) {
                    $in = '0';
                }
                if (is_array($in)) {
                    foreach ($in as &$i) {
                        if ($i === true) {
                            $i = '1';
                        }
                        if ($i === false) {
                            $i = '0';
                        }
                    }
                }
            }

            foreach ($input as $key => $value) {
                IW::Env()->addInput($key, $value);
            }

            API::auth($apikey, API::AUTH_RETURN_SESSION);

            $ctrl_class = Ctrl_Util::convertPathToClassName($ctrl_name);
            $result     = IW::FC()->route($ctrl_class, $action);
        } catch (IWorx_Exception_NotLoggedIn $e) {
            $result = API::formatResponse(API::AUTHENTICATION_FAILURE, $e->getMessage(), Reply::PERM_NOT_LOGGED_IN);
        } catch (IWorx_Exception_Api $e) {
            $result = API::formatResponse($e->getCode(), $e->getMessage(), $e->getReplyCode());
        } catch (Exception $e) {
            $result = API::formatResponse(API::GENERAL_FAILURE, $e->getMessage());
        }//end try

        // enable for debugging
        // if( $result['status'] != API::SUCCESS ) {
        // IW::getLog()->error('API FAIL: ' );
        // IW::getLog()->info(var_export( $result['payload'], 1 ));
        // IW::getLog()->info('API INPUT: ');
        // IW::getLog()->info(var_export( $input, 1 ));
        // IW::getLog()->info(var_export( $apikey, 1 ));
        // } else {
        // IW::getLog()->info('API SUCCESS');
        // IW::getLog()->info("Controller: $ctrl_name, Action: $action");
        // IW::getLog()->info(var_export( $result['payload'], 1 ));
        // IW::getLog()->info('API INPUT: ');
        // IW::getLog()->info(var_export( $input, 1 ));
        // }

        return $result;
    }
}
