<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage API
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Nov 20, 2008
 */

/**
 * Siteworx account related API funcs.
 *
 * @package    InterWorx
 * @subpackage API
 */
class IWorx_API_SiteworxAccount
{

    /**
     * Edit a siteworx account features.
     *
     * @param array|struct $params
     * @return mixed
     */
    public function editSiteworx($params)
    {
        $NodeWorx = API::auth($params, API::AUTH_RETURN_INTERWORX);
        /*
        $input  = $params['account'];
        $domain = $params['domain'];

        $SiteWorx = $NodeWorx->getSiteWorx( $domain );

        $current_input = $SiteWorx->generateAccountInput( $bool_style = 'string' );

        $input = array_merge( $current_input, $input ); // overwriting any matches

        IW::Env()->addInput( 'input', $input );
        IW::Env()->addInput( 'domain', $domain );
         */
        foreach ($params as $key => $value) {
            IW::Env()->addInput($key, $value);
        }

        $controller_name = Ctrl_Util::convertPathToClassName('nodeworx/siteworx');
        return IW::FC()->route($controller_name, 'editCommit');
    }

    /**
     * Suspend a siteworx account.
     *
     * @param array|struct $params
     * @return mixed
     */
    public function suspendaccount($params)
    {
        $params['status'] = '0';
        $result = $this->editSiteworx($params);

        if ($result['status'] === API::SUCCESS) {
            if (isset($params['_client_version']) &&
            $params['_client_version'] >= 2.0 ) {
                return API::SUCCESS;
            } else {
                return 'Account Suspended';
            }
        } else {
            return $result['payload'];
        }
    }

    /**
     * Unsuspend a siteworx account.
     *
     * @param array|struct $params
     * @return array|struct
     */
    public function unsuspendaccount($params)
    {
        $params['status'] = '1';
        $result = $this->editSiteworx($params);

        if ($result['status'] === API::SUCCESS) {
            if (isset($params['_client_version']) &&
            $params['_client_version'] >= 2.0 ) {
                return API::SUCCESS;
            } else {
                return 'Account Unsuspended';
            }
        } else {
            return $result['payload'];
        }
    }

    /**
     * Delete a siteworx account.
     *
     * @param array|struct $params
     * @return array|struct
     */
    public function deleteaccount($params)
    {
        API::auth($params);

        IW::Env()->addInput('domain', $params['domain']);

        $controller_name = Ctrl_Util::convertPathToClassName('nodeworx/siteworx');
        $result = IW::FC()->route($controller_name, 'deleteCommit');

        if ($result['status'] === API::SUCCESS) {
            if (isset($params['_client_version']) &&
            $params['_client_version'] >= 2.0 ) {
                return API::SUCCESS;
            } else {
                return 'Account Deleted';
            }
        } else {
            return $result['payload'];
        }
    }

    /**
     * Initializes the siteworx add Form.
     *
     * @param array $input
     */
    private function _convertOldAccountData(array $input)
    {
        $input_map = array( 'reseller'         => 'parent_id',
                        'packagetemplate'  => 'package_name',
                        'master_domain'       => 'domain',
                        'master_domain_ipv4'        => 'ipaddr',
                        'confirm_password' => 'password' );

        foreach ($input as $key => $value) {
            if (is_array($value) === true) {
                if ($key === 'options') {
                    foreach ($value as $opt_name => $opt_value) {
                        if (substr($opt_name, 0, 3) === 'LG_') {
                            $newname = substr($opt_name, 3);
                        } else {
                            $newname = $opt_name;
                        }
                        if ($opt_value === 'true') {
                            $opt_value = '1';
                        }
                        if ($opt_value === 'false') {
                            $opt_value = '0';
                        }
                        IW::Env()->addInput($newname, $opt_value);
                    }
                } else {
                    continue;
                }
            }
            IW::Env()->addInput($key, $value);
            if (($found = array_search($key, $input_map)) !== false) {
                IW::Env()->addInput($found, $value);
            }
        }//end foreach
    }

    /**
     * Create a new siteworx account.
     *
     * @param array|struct $params
     * @return array|struct
     */
    public function createaccount($params)
    {
        API::auth($params);

        if (isset($params['account'])) {
            $input = $params['account'];
            $megaold = false;
        } else {
            $megaold = true;
            $input = [];
            // set the basic info
            if (isset($params['account_info'])) {
                if (isset($params['account_info'][0])) {
                    $input['nickname'] = $params['account_info'][0];
                } else {
                    $input['nickname'] = '';
                }

                if (isset($params['account_info'][1])) {
                    $input['email'] = $params['account_info'][1];
                } else {
                    $input['email'] = '';
                }

                if (isset($params['account_info'][2])) {
                    $input['password'] = $params['account_info'][2];
                } else {
                    $input['password'] = '';
                }

                if (isset($params['account_info'][3])) {
                    $input['domain'] = $params['account_info'][3];
                } else {
                    $input['domain'] = '';
                }

                if (isset($params['account_info'][4])) {
                    $input['ipaddr'] = $params['account_info'][4];
                } else {
                    $input['ipaddr'] = '';
                }

                $default_lang = Ini::get(Ini::IWORX_LANGUAGE, 'siteworx');

                if (isset($params['account_info'][5])) {
                    $input['language'] = $params['account_info'][5];
                } else {
                    $input['language'] = $default_lang;
                }
            }//end if

            if (isset($params['package_name'])) {
                $input['package_name'] = $params['package_name'];
            }
        }//end if

        $unixuser = IWorxUtil::generate_uniqname($input['domain']);
        $input['uniqname'] = $unixuser;

        if (!isset($input['theme'])) {
            $input['theme'] = 'interworx';
        }

        if (IW::Env()->isNode()) {
            $input['theme'] = 'interworx'; // force interworx theme here
                                     // so there is no error if the
                                     // the theme in not installed
                                     // on the node
        }

        $this->_convertOldAccountData($input);

        $controller_name = Ctrl_Util::convertPathToClassName('nodeworx/siteworx');
        $res = IW::FC()->route($controller_name, 'addCommit');

        if ($res === true) {
            if (!$megaold) {
                return( API::SUCCESS );
            } else {
                return( 'Account Created' );
            }
        } else {
            IW::getLog()->error($res);
            return( $res );
        }
    }

    /**
     * Get the number of siteworx accounts on this box.
     *
     * @param array|struct $params
     * @return integer|array|struct
     */
    public function getaccountcount($params)
    {
        $id = API::auth($params, API::AUTH_RETURN_INTERWORX_ID);

        if ($id == 1) {
            return count(SiteWorx::getIdList());
        } else {
            return API::formatResponse(API::ERROR_PERMISSION_DENIED);
        }
    }

    /**
     * Get an array of the siteworx account master domains on this box.
     *
     * @param array|struct $params
     * @return array|struct
     */
    public function getaccountlist($params)
    {
        $id = API::auth($params, API::AUTH_RETURN_INTERWORX_ID);

        if ($id == 1) {
            $domains = SiteWorx::getSiteWorxDomainList();
            $domains = array_keys($domains);

            // return a comma seperated list since we
            // can't return the array itself.
            $list = implode(',', $domains);

            return $list;
        } else {
            return API::formatResponse(API::ERROR_PERMISSION_DENIED);
        }
    }

    /**
     * Get an array of the domains that have virutalhosts on this box.
     *
     * @param array|struct $params
     * @return array|struct
     */
    public function getvhostlist($params)
    {
        API::auth($params);

        $vhost_reader = ApacheVhostReader::getInstance();
        $domains      = $vhost_reader->getDomainList();

        // return a comma seperated list since we
        // can't return the array itself.
        $list = implode(',', $domains);

        return $list;
    }

    /**
     * Add a slave domain to a clustered node.
     *
     * @param array|struct $params
     * @return array|struct
     */
    public function addslavedomain($params)
    {
        $NodeWorx = API::auth($params, API::AUTH_RETURN_INTERWORX);

        $input = $params;

        $domain = $input['slave'];

        IW::getLog()->debug("adding new SLAVE domain: $domain");

        if ($NodeWorx->getId() <= 0) {
            IW::getLog()->debug('no such NodeWorx user for new SLAVE domain addition');
            return API::formatResponse(API::GENERAL_FAILURE);
        }

        // get the siteworx account in question
        $siteworx = $NodeWorx->getSiteWorx($input['master']);

        if ($siteworx === false || $siteworx->getId() <= 0) {
            IW::getLog()->debug("no such SiteWorx: {$input['master']} " .
                     'for new SLAVE domain');
            return API::formatResponse(API::GENERAL_FAILURE);
        }

        if (($ok = $siteworx->addSlaveDomain($domain)) === true) {
            return API::formatResponse(API::SUCCESS);
        }

        // something bad happened if we get here
        IW::getLog()->error("ERROR on adding SLAVE domain: $domain");

        $errors = IWorxError::extractErrorMessages($ok);
        IW::getLog()->error('SITEWORX ERROR:', IWorxLog::DEBUG, $errors);

        // uh oh!!
        $codes = IWorxError::extractErrorCodes($ok);
        IW::getLog()->error('API ERROR:', IWorxLog::DEBUG, $codes);

        // just return the 1st code for now
        return API::formatResponse($codes[0], $errors);
    }

    /**
     * Delete a slave domain from a clustered node.
     *
     * @param array|struct $params
     * @return array|struct
     */
    public function deleteslavedomain($params)
    {
        $NodeWorx = API::auth($params, API::AUTH_RETURN_INTERWORX);

        $input  = $params;
        $domain = $input['slave'];

        IW::getLog()->debug("deleting SLAVE domain: $domain");

        if ($NodeWorx->getId() <= 0) {
            IW::getLog()->debug('no such NodeWorx user for new SLAVE domain deletion');
            return API::formatResponse(API::ERROR_INVALID_NODEWORX_ACCOUNT);
        }

        // get the siteworx account in question
        $siteworx = $NodeWorx->getSiteWorx($input['master']);

        if ($siteworx === false || $siteworx->getId() <= 0) {
            IW::getLog()->debug("no such SiteWorx: {$input['master']} for SLAVE deletion");
            return API::formatResponse(API::ERROR_INVALID_SITEWORX_ACCOUNT);
        }

        // if it doesn't exist, it's ok, we're deleting anyway
        if (!$siteworx->slaveDomainExists($domain)) {
            return API::formatResponse(API::SUCCESS);
        }

        // delete the slave
        if ($siteworx->deleteSlaveDomain($domain)) {
            return API::formatResponse(API::SUCCESS);
        }

        // something bad happened if we get here
        IW::getLog()->debug("ERROR on deleting SLAVE domain: {$input['domain']}");

        $errors = IWorxError::extractErrorMessages($ok);
        IW::getLog()->error('SITEWORX ERROR:', IWorxLog::DEBUG, $errors);

        // uh oh!!
        $codes = IWorxError::extractErrorCodes($ok);
        IW::getLog()->error('API ERROR:', IWorxLog::DEBUG, $codes);

        return API::formatResponse($codes[0]);
    }

    /**
     * Sync the apache server alias configs with the cluster manager on this account.
     *
     * @param array|struct $params
     * @return array|struct
     */
    public function syncserveralias($params)
    {
        $NodeWorx = API::auth($params, API::AUTH_RETURN_INTERWORX);

        $input = $params;

        if (!isset($input['domain'])) {
            return API::formatResponse(API::ERROR_INVALID_SITEWORX_ACCOUNT);
        }

        $domain = $input['domain'];

        // get the siteworx account in question
        $siteworx = $NodeWorx->getSiteWorx($domain);

        if ($siteworx === false || $siteworx->getId() <= 0) {
            IW::getLog()->debug("no such SiteWorx: $domain ServerAlias Sync");
            return API::formatResponse(API::ERROR_INVALID_SITEWORX_ACCOUNT);
        }

        if (($ok = $siteworx->syncServerAlias()) !== true) {
            $codes = IWorxError::extractErrorCodes($ok);

            if (isset($codes[0])) {
                return API::formatResponse($codes[0]);
            }

            IW::getLog()->error("unknown error whiling syncing server alias : $domain");

            return API::formatResponse(API::GENERAL_FAILURE);
        }

        // just assuming this worked for now.  Maybe this is bad.
        return API::formatResponse(API::SUCCESS);
    }
}
