<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage API
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Nov 24, 2008
 */

/**
 * API functions related to Siteworx Packages.
 *
 * @package    InterWorx
 * @subpackage API
 */
class IWorx_API_SiteworxPackage
{

    /**
     * Check whether a given package name exists.
     *
     * @param array|struct $params
     * @return string
     */
    public function packageexists($params)
    {
        $NodeWorx = API::auth($params, API::AUTH_RETURN_INTERWORX);

        if (isset($params['package_name'])) {
            $name = $params['package_name'];
        }

        $exists = $NodeWorx->packageTemplateExists($name);

        return ($exists) ? '1' : '0';
    }

    /**
     * Add a new SiteWorx package template.
     *
     * @param array|struct $params
     * @return array|struct
     */
    public function createpackage($params)
    {
        API::auth($params, API::AUTH_RETURN_SESSION);

        $package_name  = $params['package_name'];
        $features_list = $params['features_list'];

        IW::Env()->addInput('package_name', $package_name);
        foreach ($features_list as $name => $value) {
            IW::Env()->addInput($name, $value);
        }

        $controller_name = Ctrl_Util::convertPathToClassName('nodeworx/packages');
        return IW::FC()->route($controller_name, 'addCommit');
    }

    /**
     * Delete a SiteWorx package template.
     *
     * @param array|struct $params
     * @return array|struct
     */
    public function deletepackage($params)
    {
        API::auth($params, API::AUTH_RETURN_SESSION);

        $confirm = array( 'passthru' => $params['package_name'] );
        $confirm = IWorxHTTP::squeezeArray($confirm);

        IW::Env()->addInput('id', $confirm);

        $controller_name = Ctrl_Util::convertPathToClassName('nodeworx/packages');
        return IW::FC()->route($controller_name, 'delCommit');
    }
}
