#!/bin/sh

LOGDIR=/var/log/send
TAI64N2TAI=/usr/bin/tai64n2tai
AWK=/usr/bin/awk

if [[ "$1" == "" ]]; then
  echo "Usage: $0 timestamp"
  echo
  echo "Finds qmail send logs containing date requested, and newer data"
  exit 1
fi

last=$1
oldest_stamp=0
oldest_file=""
files=""
for file in "$LOGDIR"/{"@",cur}*; do
  stamp=`stat -c %Z $file`
  if [[ $oldest_stamp == 0 ]]; then
    oldest_stamp=$stamp
    oldest_file=$file
  fi
  if [[ $stamp < $oldest_stamp ]]; then
    oldest_stamp=$stamp
    oldest_file=$file
  fi
  if [[ $stamp > $last ]]; then
    files="$files $file"
  fi
done

if [[ $last < $oldest_stamp ]]; then
  first_stamp=`head -n1 $oldest_file | awk '{print $1}'`
  first_secs=`echo $first_stamp | $TAI64N2TAI | $AWK -F. '{print $1}'`
  if [[ $last < $first_secs ]]; then
    echo -n "Warning: we only have logs dating back to " 1>&2
    echo $first_stamp | tai64nlocal 1>&2
    echo 1>&2
    sleep 1
  fi
fi

echo $files