<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Sep 26, 2009
 */

/**
 * Plugin to auto-enable ssh shell users on account creation.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_AutoEnableShellAccount extends Plugin
{
    /**
     * Init my edit form.
     *
     * Add plugin settings that specify which resellers this feature is enabled
     * for.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function initializeMyEditForm(Form_NW_Plugins_Edit $Form)
    {

        $Group = $Form->addGroup('extra_options', 'Extra Plugin Settings');

        $all_resellers = $this->getDatastoreVar('all_resellers');
        $Group->addInput(new Input_Flag('all_resellers', $all_resellers));

        $resellers = $this->getDatastoreVar('resellers');
        if ($resellers === '') {
            $resellers = [];
        }
        $Group->addInput(new Input_Select('resellers', $resellers))
        ->setDataSource(new DataSource_NW_ResellerIds())
        ->setRequired(true)
        ->setMultipleAllowed(true);
    }

    /**
     * Process my edit form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function processMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        if ($Form->getValue('all_resellers')) {
            $resellers = [];
        } else {
            $resellers = $Form->getValue('resellers');
        }

        $this->setDatastoreVar('all_resellers', $Form->getValue('all_resellers'));
        $this->setDatastoreVar('resellers', $resellers);
    }

    /**
     * Get priority.
     *
     * @return integer
     */
    public function getPriority()
    {
        return 40;
    }

    /**
     * Handle the controller:action we care about.
     *
     * @param string        $ctrl_act
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function postAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        if ($ctrl_act !== 'Ctrl_Nodeworx_Siteworx:addCommit') {
            return;
        }
        if (!$Ctrl->isViewSuccess()) {
            return;
        }
        if ($this->_userHasNoAccess()) {
            return;
        }

        assert($params instanceof Form);
        /**
         * @var Form.
         */
        $Form     = $params;
        $uniqname = $Form->getValue('uniqname');
        $password = $Form->getValue('password');

        $ctrl = 'Ctrl_Nodeworx_Shell';
        if (IW::NW()->isReseller()) {
            $NW = new NodeWorx(NodeWorx::MASTER_ID);
        } else {
            $NW = IW::NW();
        }

        $action = 'changeshell';
        $input = array( 'shell' => Ini::get(Ini::SHELL, 'default'),
                    'users' => array( $uniqname ) );
        $Reply = IW::FC()->routeFromPHP($ctrl, $action, $input, $NW);

        $action = 'enable';
        $input = array( 'users' => array( $uniqname ) );
        $Reply = IW::FC()->routeFromPHP($ctrl, $action, $input, $NW);

        $action = 'password';
        $input = array( 'user'             => $uniqname,
                    'password'         => $password,
                    'confirm_password' => $password );
        $Reply = IW::FC()->routeFromPHP($ctrl, $action, $input, $NW);

        $sshConfig = IW::DM()->getSSHD()->readConfig();

        if (count($sshConfig['allowusers']) > 0) {
            $SSH       = new SSHD();
            $allowed   = $sshConfig['allowusers'];
            $allowed[] = $uniqname;
            $SSH->setAllowUsers($allowed);
            $SSH->writeConfig();
        }

        if ($Reply->wasSuccessful() === true) {
            $msg = 'Shell account user enabled';
        } else {
            $msg = "Tried to enable shell user {$uniqname}, but failed.";
        }
        $Ctrl->getView()->addMessage($msg);
    }

    /**
     * True if the current reseller doesn't have access to use this plugin.
     *
     * @return boolean
     */
    private function _userHasNoAccess()
    {
        if ($this->getDatastoreVar('all_resellers')) {
            return false;
        }
        if (true === in_array(
            IW::NW()->getId(),
            $this->getDatastoreVar('resellers')
        ) ) {
            return false;
        }

        return true;
    }
}
