<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Matt Parmelee <mparmelee@interworx.com>
 * @date    Jul 21, 2011
 */

/**
 * Autoset Spamprefs plugin.
 *
 * Automatically sets SpamAssassin preferences on creation of a Siteworx account.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_AutosetSpamprefs extends Plugin
{
    const INPUT_DROPSCORE = 'dropscore_custom';
    const INPUT_SPAMSCORE = 'spamscore_custom';
    const INPUT_REWRITE_SUBJECT = 'rewrite_subject_custom';

    const ENABLE_SPAM = 'enable_spam';
    const DROPSCORE = 'dropscore';
    const SPAMSCORE = 'spamscore';
    const REWRITE_SUBJECT = 'rewrite_subject';
    const REPORT_SAFE = 'report_safe';
    /**
     * Post action.
     *
     * @param string        $ctrl_act Controller_Name:action
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function postAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        if ($ctrl_act === 'Ctrl_Nodeworx_Siteworx:addCommit') {
            if ($Ctrl->isViewSuccess()) {
                $domain = $params->getValue('domainname');
                $input = array(
                    self::ENABLE_SPAM => $this->getDatastoreVar(self::ENABLE_SPAM),
                    self::DROPSCORE => $this->getDatastoreVar(self::DROPSCORE),
                    self::SPAMSCORE => $this->getDatastoreVar(self::SPAMSCORE),
                    self::REWRITE_SUBJECT => $this->getDatastoreVar(self::REWRITE_SUBJECT),
                    self::REPORT_SAFE => $this->getDatastoreVar(self::REPORT_SAFE)
                );

                $Reply = IW::FC()->routeFromPHP(
                    'Ctrl_Siteworx_EmailSpamprefs',
                    'editCommit',
                    $input,
                    IW::NW()->getSiteWorx($domain)
                );
            }
        }
        return;
    }

    /**
     * Initialize this plugin's form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function initializeMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        $Group = $Form->addGroup('extra_options', '##LG_EXTRA_PLUGIN_SETTINGS##');

        //enable spam
        $DS = DataSource_Boolean::factory(DataSource_Boolean::ON_OFF);

        $enable_spam = $this->getDatastoreVar(self::ENABLE_SPAM);
        $Group->addInput(new Input_Select(self::ENABLE_SPAM, $enable_spam))
            ->setLabel('##LG_SPAMASSASSIN_STATUS##')
            ->setDataSource($DS);

        //drop score
        $InputDropscore = $this->_getDropscoreCustomInput();
        $Group->addInput($InputDropscore)->setLabel('##LG_SPAMDELIVERY##');

        //spam score
        $InputSpamscore = $this->_getSpamscoreCustomInput();
        $Group->addInput($InputSpamscore)->setLabel('##LG_SPAMSCORETHRESHOLD##');

        //rewrite subject
        $InputRewriteSubject = $this->_getRewriteSubjectCustomInput();
        $Group->addInput($InputRewriteSubject)->setLabel('##LG_REWRITESUBJECT##');

        //report safe
        $values         = array(0, 1, 2);
        $display_values = array(
            '##LG_REPORT_SAFE_0##',
            '##LG_REPORT_SAFE_1##',
            '##LG_REPORT_SAFE_2##'
        );

        $DS = new DataSource_Array($values, $display_values);

        $report_safe = $this->getDatastoreVar(self::REPORT_SAFE);
        $Group->addInput(new Input_Set(self::REPORT_SAFE, $report_safe))
            ->setDataSource($DS)
            ->setLabel('##LG_MESSAGEATTACHMENTPOLICY##');
    }

    /**
     * @return Input_SetCustom
     */
    private function _getDropscoreCustomInput()
    {
        $DS = new DataSource_Array(
            array('no'),
            array('##LG_DELIVER_TO_INBOX_OR_SPAM_FOLDER##')
        );

        $CustomInput = new Input_Number(self::INPUT_DROPSCORE);
        $CustomInput->setRange(0, 999);
        $CustomInput->setLengthRange(1, 6);
        $CustomInput->setLabel('##LG_DELETE_SPAM_SCORED_HIGHER_THAN_THIS_VALUE##');

        $dropscore = $this->getDatastoreVar(self::DROPSCORE);

        return new Input_SetCustom(
            self::INPUT_DROPSCORE,
            $DS,
            $CustomInput,
            $dropscore
        );
    }

    /**
     * @return Input_SetCustom
     */
    private function _getSpamscoreCustomInput()
    {

        $values         = array(5, 7, 10);
        $display_values = array(
            '5 (##LG_AGGRESSIVE##)',
            '7 (##LG_LESSAGGRESSIVE##)',
            '10 (##LG_CONSERVATIVE##)'
        );

        $DS = new DataSource_Array($values, $display_values);

        $CustomInput = new Input_Number(self::INPUT_SPAMSCORE);
        $CustomInput->setRange(0, 999);
        $CustomInput->setLabel('(##LG_CUSTOM##)');


        $spamscore = $this->getDatastoreVar(self::SPAMSCORE);
        return new Input_SetCustom(
            self::INPUT_SPAMSCORE,
            $DS,
            $CustomInput,
            $spamscore
        );
    }

    /**
     * @return Input_SetCustom
     */
    private function _getRewriteSubjectCustomInput()
    {
        $CustomInput = new Input_String(self::INPUT_REWRITE_SUBJECT, '[SPAM]');
        $CustomInput->setLabel('(##LG_CUSTOM##)');

        $DS = new DataSource_Array(
            array('no'),
            array('##LG_DONOTREWRITESUBJECT##')
        );

        $rewrite_subject = $this->getDatastoreVar(self::REWRITE_SUBJECT);

        return new Input_SetCustom(
            self::INPUT_REWRITE_SUBJECT,
            $DS,
            $CustomInput,
            $rewrite_subject
        );
    }
    /**
     * Process the form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function processMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        $enableSpam = $Form->getValue(self::ENABLE_SPAM);
        $dropscore = $Form->getValue(self::INPUT_DROPSCORE);
        $spamscore = $Form->getValue(self::INPUT_SPAMSCORE);
        $rewriteSubject = $Form->getValue(self::INPUT_REWRITE_SUBJECT);
        $reportSafe = $Form->getValue(self::REPORT_SAFE);

        $this->setDatastoreVar(self::ENABLE_SPAM, $enableSpam);
        $this->setDatastoreVar(self::DROPSCORE, $dropscore);
        $this->setDatastoreVar(self::SPAMSCORE, $spamscore);
        $this->setDatastoreVar(self::REWRITE_SUBJECT, $rewriteSubject);
        $this->setDatastoreVar(self::REPORT_SAFE, $reportSafe);
    }
}
