<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Matt Parmelee <mparmelee@interworx.com>
 * @date    Jul 22, 2011
 */

/**
 * Tests for the Autoset Spamprefs plugin.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class AutosetSpamprefsTest extends IWorx_Framework_TestCase
{

    /**
     * SetUpBeforeClass function.
     */
    public static function setUpBeforeClass()
    {
        IW::PluginManager()->clearInstance();
        Ini::reload();
        if (IW::PluginManager()->getPlugin('autoset-spamprefs')->isEnabled()) {
            Ini::set(Ini::PLUGINS, 'autoset-spamprefs', 0);
            Ini::write();
        }
    }

    /**
     * Test enabling of the plugin.
     */
    public function testEnablePlugin()
    {
        Ini::set(Ini::PLUGINS, 'autoset-spamprefs', 1);
        Ini::write();
        IW::PluginManager()->clearInstance();
        Ini::reload();
        $this->assertTrue(IW::PluginManager()
                       ->getPlugin('autoset-spamprefs')->isEnabled());
    }

    /**
     * Test disabling of the plugin.
     */
    public function testDisablePlugin()
    {
        Ini::set(Ini::PLUGINS, 'autoset-spamprefs', 0);
        Ini::write();
        IW::PluginManager()->clearInstance();
        Ini::reload();
        $this->assertFalse(IW::PluginManager()
                        ->getPlugin('autoset-spamprefs')->isEnabled());
    }

    /**
     * Ensure default spam prefs are set.
     */
    public function testDefaultSpamPrefs()
    {
        self::deleteSWAccountIfNecessary();

        //baseline add sw account
        self::addSWAccount();

        //assert spam deafult
        $spm   = new SpamPrefManager();
        $prefs = $spm->getDomainPrefs(self::SW_DOMAIN);

        $this->assertFalse(isset($assoc['enable_spam']));
        $this->assertFalse(isset($assoc['dropscore']));
        $this->assertFalse(isset($assoc['spamscore']));
        $this->assertFalse(isset($assoc['rewrite_subject']));
        $this->assertFalse(isset($assoc['report_safe']));

        //delete sw account
        self::deleteSWAccountIfNecessary();
    }

    /**
     * Ensure custom spam prefs are set upon account creation.
     */
    public function testCustomSpamPrefs()
    {
        //enable plugin
        Ini::set(Ini::PLUGINS, 'autoset-spamprefs', 1);
        Ini::write();
        IW::PluginManager()->clearInstance();
        Ini::reload();
        $this->assertTrue(IW::PluginManager()
                       ->getPlugin('autoset-spamprefs')->isEnabled());

        //add values to plugin
        IW::PluginManager()->getPlugin('autoset-spamprefs')
        ->setDatastoreVar('enable_spam', 1);
        IW::PluginManager()->getPlugin('autoset-spamprefs')
        ->setDatastoreVar('dropscore', '16');
        IW::PluginManager()->getPlugin('autoset-spamprefs')
        ->setDatastoreVar('spamscore', 3.5);
        IW::PluginManager()->getPlugin('autoset-spamprefs')
        ->setDatastoreVar('rewrite_subject', '[LOL]');
        IW::PluginManager()->getPlugin('autoset-spamprefs')
        ->setDatastoreVar('report_safe', 2);

        //add sw account
        self::addSWAccount();

        //check values
        $spm = new SpamPrefManager();
        $prefs = $spm->getDomainPrefs(self::SW_DOMAIN);
        $values = [];
        foreach ($prefs as $pref) {
            $values[$pref->getPreference()] = $pref->getValue();
        }
        $sa = new SpamAssassin();
        $this->assertTrue($sa->isEnabledDomain(self::SW_DOMAIN));
        $this->assertTrue($sa->getDropScore(self::SW_DOMAIN) === '16');
        $this->assertTrue($values['required_score'] === '3.5');
        $this->assertTrue($values['rewrite_header subject'] === '[LOL]');
        $this->assertTrue($values['report_safe'] === '2');
    }
}
