<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Sep 29, 2009
 */

/**
 * Catchall account option plugin.
 *
 * Disables catchalls from being set by default, and adds catchall setting
 * access as a siteworx account option.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_CatchallAccountOption extends Plugin
{

    /**
     * Pre action.
     *
     * @param string        $ctrl_act
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function preAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        if ($ctrl_act === 'Ctrl_Siteworx_Email:editBounceCommit') {
            if ($this->userIsBlocked(IW::SW()->getUniqname())) {
                /**
                 * @var Form.
                 */
                $Form     = $params;
                $catchall = $Form->getValue('enablebounce');
                if ($catchall != '1' && $catchall != '0') {
                    $msg = 'Enabling a catchall is disabled for this account.';
                    throw new IWorx_Exception_Plugin($msg);
                }
            }
        }
    }

    /**
     * Get category.
     *
     * @return string
     */
    public function getCategory()
    {
        return Plugin_Category::EMAIL;
    }

    /**
     * Post action.
     *
     * @param string        $ctrl_act Controller_Name:action
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function postAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        switch ($ctrl_act) {
            case 'Ctrl_Nodeworx_Siteworx:editCommit';
            case 'Ctrl_Nodeworx_Siteworx:addCommit':
                assert($params instanceof Form);
                $Form = $params;
                if ($Form->getValue('catchall_access') === '1') {
                    $this->unblockUser($Form->getValue('uniqname'));
                } else {
                    $this->blockUser($Form->getValue('uniqname'));
                }

            default:
          return;
        }
    }

    /**
     * Initialize Forms we care about.
     *
     * @param Form $Form
     */
    public function initializeForm(Form $Form)
    {
        $form_class = get_class($Form);

        $ds_type = DataSource_Boolean::YES_NO;
        if ($form_class === 'Form_NW_Siteworx_Add') {
            $default = 0;
            $Form->addGroup('catchall', 'E-mail Catchall Settings')
            ->addInput(new Input_Select('catchall_access', $default))
            ->setDataSource(DataSource_Boolean::factory($ds_type))
            ->setLabel('Account can set an e-mail catchall address');
        }

        if ($form_class === 'Form_NW_Siteworx_Edit') {
            $uniq    = $Form->getInput('uniqname')->getCurrentValue();
            $current = ($this->userIsBlocked($uniq)) ? 0 : 1;
            $Form->addGroup('catchall', 'E-mail Catchall Settings')
            ->addInput(new Input_Select('catchall_access', $current))
            ->setDataSource(DataSource_Boolean::factory($ds_type))
            ->setLabel('Account can set an e-mail catchall address');
        }
    }

    /**
     * True if the provided unixuser is blocked from setting a catchall.
     *
     * @param string $unixuser
     * @return boolean
     */
    public function userIsBlocked($unixuser)
    {
        if (!in_array($unixuser, (array)$this->getDatastoreVar('allowed_users'))) {
            return true;
        }
        return false;
    }

    /**
     * Block a unixuser from using a catchall.
     *
     * @param string $unixuser
     */
    public function blockUser($unixuser)
    {
        $allowed_users = $this->getDatastoreVar('allowed_users');
        if (!is_array($allowed_users)) {
            $allowed_users = [];
        }
        $key = array_search($unixuser, $allowed_users);
        if ($key !== false) {
            unset($allowed_users[ $key ]);
            $this->setDatastoreVar('allowed_users', $allowed_users);
        }
    }

    /**
     * Unblock a unixuser from using a catchall.
     *
     * @param string $unixuser
     */
    public function unblockUser($unixuser)
    {
        $allowed_users = $this->getDatastoreVar('allowed_users');
        if (!is_array($allowed_users)) {
            $allowed_users = [];
        }
        $allowed_users[] = $unixuser;
        $allowed_users   = array_unique($allowed_users);
        $this->setDatastoreVar('allowed_users', $allowed_users);
    }
}
