<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Matt Parmelee <mparmelee@interworx.com>
 * @date    Dec 14, 2011
 */

/**
 * CloudFlare Controller.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Ctrl_Siteworx_Cloudflare extends Ctrl_Siteworx_Plugin
{
    /**
     * Default action.
     */
    public function indexAction()
    {
        $Plugin = $this->_getPlugin();
        $this->getView()->assign('title', 'CloudFlare Management');

        if (!$Plugin->isCloudflareUserSetup()) {
            $this->_showLoginForm();
        } else {
            $this->_showMainCloudFlareInterface();
        }
    }

    /**
     * Show CloudFlare Login Form.
     */
    private function _showLoginForm()
    {
        $this->getView()->assign(
            'iw_form',
            Form::getInstance(
                'Form_SW_Cloudflare_Login',
                'loginCommit',
                'index'
            )
        );
    }

    /**
     * Show CloudFlare main interface.
     */
    private function _showMainCloudFlareInterface()
    {
        $Plugin = $this->_getPlugin();
        $email = $Plugin->getCloudflareUserEmail();
        $this->getView()->assign('iw_user_key', $email);
        $this->getView()->assign(
            'iw_form',
            Form::getInstance(
                'Form_SW_Cloudflare_Logout',
                'logoutCommit',
                'index'
            )
        );

        $Domains = Payload_Factory_SW_Cloudflare::domains(IW::SW());
        $Domains->setRefreshAction('listDomains');
        $this->getView()->assign('iw_cloudflare_domains', $Domains);

        $Subdomains = Payload_Factory_SW_Cloudflare::subdomains(IW::SW(), 'none');
        $Subdomains->setRefreshAction('queryDomains');
        $this->getView()->assign('iw_subdomains', $Subdomains);

        $API = $Plugin->getHostAPI();
        list( $email, $user_key, $user_api_key ) = $this->_getPlugin()->getUserKeyData();
    }

    /**
     * List domains.
     *
     * @param Form_SW_Cloudflare_Domains $Form
     */
    public function queryDomainsCommitAction(Form_SW_Cloudflare_Domains $Form)
    {
        $SW = IW::SW();
        $D = $Form->getValue('domain');
        $this->_displayPayload(Payload_Factory_SW_Cloudflare::subdomains($SW, $D));
    }

    /**
     * List settings.
     */
    public function listDomainsAction()
    {
        $this->_displayPayload(Payload_Factory_SW_Cloudflare::domains(IW::SW()));
    }

    /**
     * Edit Cloudflare settings.
     *
     * @param Form_SW_Cloudflare_Settings $Form
     */
    public function settingsCommitAction(Form_SW_Cloudflare_Settings $Form)
    {
        $domain = $Form->getValue('domain');
        $Plugin = $this->_getPlugin();
        $errors = [];

        $CfApi = $Plugin->getHostAPI();
        list( $email, $user_key, $user_api_key ) = $Plugin->getUserKeyData();
        $API = new IWorx_CloudflareClientAPI($user_api_key);

        if ($Form->getInput('security_level')->isChanged()) {
            $sec_lvl = $Form->getValue('security_level');
            $ret = $API->setSecurityLevel($domain, $email, $sec_lvl);
            $errors[] = ( $ret['result'] === 'success' ) ? '' : $ret['msg'];
        }

        if ($Form->getInput('cache_level')->isChanged()) {
            $cache_lvl = $Form->getValue('cache_level');
            $ret = $API->setCacheLevel($domain, $email, $cache_lvl);
            $errors[] = ( $ret['result'] === 'success' ) ? '' : $ret['msg'];
        }

        if ($Form->getInput('dev_mode')->isChanged()) {
            $dev_mode = $Form->getValue('dev_mode');
            $ret = $API->toggleDevMode($domain, $email, $dev_mode);
            $errors[] = ( $ret['result'] === 'success' ) ? '' : $ret['msg'];
        }

        if ($Form->getInput('ipv6_enabled')->isChanged()) {
            $toggle_ipv6 = $Form->getValue('ipv6_enabled');
            $ret = $API->toggleIPv6($domain, $email, $toggle_ipv6);
            $errors[] = ( $ret['result'] === 'success' ) ? '' : $ret['msg'];
        }

        $errors = array_unique($errors);
        $errors = array_merge($Form->getErrors(), $errors);
        $errors = array_filter($errors);
        if (empty($errors)) {
            $this->_displaySuccess('index', '##LG_SETTINGSUPDATEDSUCCESSFULLY##');
        } else {
            $this->_displayFailure('index', $errors);
        }
    }

    /**
     * Enable Cloudflare for a subdomain.
     *
     * @param Form_SW_Cloudflare_Enable $Form
     */
    public function enableCommitAction(Form_SW_Cloudflare_Enable $Form)
    {
        $parent    = $Form->getValue('parent');
        $subdomain = $Form->getValue('subdomain');

        $errors = [];

        $this->_enableSubdomain($subdomain, $parent, $errors);

        if (empty($errors)) {
            $this->_getPlugin()->setDomainStatusData($subdomain, '1');
            $this->_displaySuccess('index', 'CloudFlare Enabled on ' . $subdomain);
        } else {
            $this->_displayFailure('index', $errors);
        }
    }

    /**
     * Disable Cloudflare for a subdomain.
     *
     * @param Form_SW_Cloudflare_Disable $Form
     */
    public function disableCommitAction(Form_SW_Cloudflare_Disable $Form)
    {
        $parent    = $Form->getValue('parent');
        $subdomain = $Form->getValue('subdomain');

        $errors = [];

        $this->_disableSubdomain($subdomain, $parent, $errors);

        if (empty($errors)) {
            $this->_getPlugin()->setDomainStatusData($subdomain, '0');
            $this->_displaySuccess('index', 'CloudFlare Disabled on ' . $subdomain);
        } else {
            $this->_displayFailure('index', $errors);
        }
    }

    /**
     * Disable cloud flare for 1 subdomain.
     *
     * @param string $subdomain
     * @param string $parent
     * @param array  $err
     */
    private function _disableSubdomain($subdomain, $parent, array &$err)
    {
        list( $Zone,
          $Record,
          $subdomains,
          $resolve_to,
          $where,
          $CfApi ) = $this->_initCloudFlareEditVariables(
              $subdomain,
              $parent,
              true
          );

        list( $email, $user_key, $user_api_key ) = $this->_getPlugin()->getUserKeyData();

        if (empty($subdomains)) {
            $ret = $CfApi->zoneDelete($user_key, $parent);
            if (!isset($ret['error'])) {
                $records = $Zone->getRecords($where);
                foreach ($records as $rec) {
                    $DelRec = new Dns_Record("record_id={$rec['record_id']}");
                    $DelRec->delete();
                }
            }
        } else {
            $ret = $CfApi->zoneSet($user_key, $parent, $resolve_to, $subdomains);
        }
        if (isset($ret['error'])) {
            $err[] = $ret['error'];
        } else {
            $Record->setAlias($parent);
            $Record->save();
        }
    }

    /**
     * Enable cloud flare for 1 subdomain.
     *
     * @param string $subdomain
     * @param string $parent
     * @param array  $err
     */
    private function _enableSubdomain($subdomain, $parent, array &$err)
    {
        list( $Zone,
          $Record,
          $subdomains,
          $resolve_to,
          $where,
          $CfApi ) = $this->_initCloudFlareEditVariables(
              $subdomain,
              $parent,
              false
          );

        list( $email, $user_key, $user_api_key ) = $this->_getPlugin()->getUserKeyData();

        if (empty($subdomains)) {
            $subdomains = str_replace(".{$parent}", '', $subdomain);
        }
        $ret = $CfApi->zoneSet($user_key, $parent, $resolve_to, $subdomains);
        if (!isset($ret['error'])) {
            $records = $Zone->getRecords($where);
            if (count($records) === 0) {
                $Zone->addResourceRecord(
                    $resolve_to,
                    Dns_Record::CNAME,
                    900,
                    $parent
                );
            }
            $Record->setAlias("{$subdomain}.cdn.cloudflare.net");
            $Record->save();
        } else {
            $err[] = $ret['error'];
        }
    }

    /**
     * Initialize and return variables used by enable/disable cloudflare on subdomain.
     *
     * @param string  $subdomain
     * @param string  $parent
     * @param boolean $exclude_sub
     * @return array
     */
    private function _initCloudFlareEditVariables(
        $subdomain,
        $parent,
        $exclude_sub
    ) {
        return
        array( $Zone   = new Dns_Zone('domain=' . q($parent)),
             $Record = new Dns_Record_CNAME("host='{$subdomain}'"),
             $subdomains = $this->_getCfSubdomains(
                 $parent,
                 $subdomain,
                 $exclude_sub
             ),
             $resolve_to = "cloudflare-resolve-to.{$parent}",
             $where = 'host=' . q($resolve_to) . ' AND zone_id = '.$Zone->getId(),
             $CfApi = $this->_getPlugin()->getHostAPI() );
    }

    /**
     * Get analytic information from CloudFlare.
     *
     * @param Form_SW_Cloudflare_Statistics $Form
     */
    public function showStatisticsCommitAction(Form_SW_Cloudflare_Statistics $Form)
    {
        $domain = $Form->getValue('domain');
        $Plugin = $this->_getPlugin();

        list( $email, $user_key, $user_api_key ) = $Plugin->getUserKeyData();
        $API = new IWorx_CloudflareClientAPI($user_api_key);
        $stats = $API->stats($domain, $email);

        $html = self::_getChartHtml($domain);
        $Printout = new Printout($html);
        $Printout->setTitle('Cloudflare Statistics');
        $Printout->setJs('/plugins/images/cloudflare/stats.js');
        $Printout->setJsData($stats);
        $this->_displayPrintout($Printout);
    }

    /**
     * Build the YUI Chart for CloudFlare analytic data.
     *
     * @param string $domain
     * @return string
     */
    private function _getChartHtml($domain)
    {
        $start = date('F') . ' ' . ( date('j') - 8 ) . ', ' . date('Y');
        $end   = date('F') . ' ' . ( date('j') - 1 ) . ', ' . date('Y');
        $html  = "<pre>{$domain}: {$start} - {$end}</pre>";
        $html .= "<div class='charts'>";
        $html .=   '<table>';
        $html .=     '<tr>';
        $html .=       "<td><div id='bandwidth_chart'></div></td>";
        $html .=       "<td><div id='requests_chart'></div></td>";
        $html .=     '</tr>';
        $html .=     '<tr>';
        $html .=       "<td><div id='views_chart'></div></td>";
        $html .=       "<td><div id='uniques_chart'></div></td>";
        $html .=     '</tr>';
        $html .=   '</table>';
        $html .= '</div>';

        return $html;
    }

    /**
     * Get Cloudflare subdomain data.
     *
     * @param string  $parent
     * @param string  $domain
     * @param boolean $exclude_sub
     * @return string
     */
    private function _getCfSubdomains($parent, $domain, $exclude_sub)
    {
        $q_parent = q($parent);
        $Zone = new Dns_Zone("domain={$q_parent}");
        $records = $Zone->getRecordObjects(Dns_Record::CNAME);
        $cloudflare_enabled = [];

        foreach ($records as $Record) {
            $pattern = '/\.cdn\.cloudflare\.net$/';
            if (preg_match($pattern, $Record->getAlias()) === 0) {
                if ($Record->getHost() !== $domain) {
                    continue;
                }
            }

            if (($Record->getHost() === $domain) && $exclude_sub) {
                continue;
            }

            $cloudflare_enabled[] = $Record->getHost();
        }
        if (count($cloudflare_enabled) > 0) {
            $result = implode(',', $cloudflare_enabled);
        } else {
            $result = '';
        }
        return $result;
    }

    /**
     * Cloudflare login.
     *
     * @param Form_SW_Cloudflare_Login $Form
     */
    public function loginCommitAction(Form_SW_Cloudflare_Login $Form)
    {
        $email = $Form->getValue('email');
        $password = $Form->getValue('password');

        $Plugin = $this->_getPlugin();
        $CfApi = $Plugin->getHostAPI();

        $errors = [];
        $verify = $CfApi->userLookup('cloudflare_email', $email);

        $user_created = false;
        $user_authed  = false;
        $create = $Form->getValue('create');

        if (!$verify['user_exists'] && $create[0] === '1') {
            $ret = $CfApi->userCreate($email, $password);
            $errors[] = ( $ret['result'] === 'success' ) ? '' : $ret['error'];
            $user_created = true;
            $user_authed  = true;
        } else {
            $ret = $CfApi->userAuth($email, $password);
            $errors[] = ( $ret['result'] === 'success' ) ? '' : $ret['error'];
            $user_authed  = true;
        }

        if (!isset($ret['error'])) {
            $sw_id = IW::SW()->getId();
            $user_key = $ret['user_key'];
            $user_api_key = $ret['user_api_key'];
            $Plugin->setUserKeyData(array( $email, $user_key, $user_api_key ));

            $this->_displaySuccess('index', '##LG_SUCCESS##', true);
        } else {
            $this->_displayFailure('index', $errors);
        }
    }

    /**
     * Cloudflare logout.
     *
     * @param Form_SW_Cloudflare_Logout $Form
     */
    public function logoutCommitAction(Form_SW_Cloudflare_Logout $Form)
    {
        $Plugin = $this->_getPlugin();
        $Plugin->clearUserKeyData();
        $this->_displaySuccess('index', '##LG_SUCCESS##', true);
    }
}
