<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Matt Parmelee <mparmelee@interworx.com>
 * @date    Dec 29, 2011
 */

/**
 * Formatter for enabled/disabled status columns.
 *
 * @package    InterWorx
 * @subpackage Formatter
 */
class IWorx_Formatter_CloudflareEnabledDisabled extends IWorx_Formatter_Complex
{
    /**
     * List of expected values and the modes to go with them.
     *
     * @var array
     */
    static private $_modes = array( '0' => 'cf-cloud-0.png',
                                  '1' => 'cf-cloud-1.png' );

    static private $_messages = array( '0' => '##LG_DISABLED##',
                                     '1' => '##LG_ENABLED##' );

    private $_domain_row;

    /**
     * Constructor.
     *
     * @param string $domain_row Row name for $row variable in format for domain.
     */
    public function __construct($domain_row)
    {
        $this->_domain_row = $domain_row;
    }

    /**
     * Formats value as an html message block.
     *
     * @param string $value
     * @param object $row
     * @param array  $data
     * @return string
     */
    public function format($value, $row, array $data)
    {
        $mode = self::$_modes[ $value ];
        $msg  = self::$_messages[ $value ];

        $img = '/plugins/images/cloudflare/' . $mode;
        $tag = array( 'src'   => $img,
                  'class' => 'iw-cf',
                  'title' => $msg,
                  "data-iw-{$this->_domain_row}" => $row->{$this->_domain_row},
                  'data-iw-status' => $value );

        if (isset($row->parent)) {
            $tag['data-iw-parent'] = $row->parent;
        }
        return IWorxHTML::buildHTMLTag('img', $tag);
    }
}
