<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Mar 3, 2010
 */

/**
 * Disable local delivery plugin.
 *
 * Disabled local delivery by default when SiteWorx accounts are created.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_DisableLocalDelivery extends Plugin
{

    /**
     * Get category.
     *
     * @return string
     */
    public function getCategory()
    {
        return Plugin_Category::EMAIL;
    }

    /**
     * Initialize the given form.
     *
     * @param Form $Form
     */
    public function initializeForm(Form $Form)
    {
        $form_class = get_class($Form);

        $ds_type = DataSource_Boolean::YES_NO;
        if ($form_class === 'Form_NW_Siteworx_Add') {
            $default = 0;

            $Gp = $Form->addGroup('delivery', 'E-mail Delivery Settings');
            $Gp->addInput(new Input_Select('local_delivery', $default))
            ->setDataSource(DataSource_Boolean::factory($ds_type))
            ->setLabel('Use local mail server');
        }//end if
    }

    /**
     * Post action.
     *
     * @param string        $ctrl_act
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function postAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        switch ($ctrl_act) {
            case 'Ctrl_Nodeworx_Siteworx:addCommit':
                assert($params instanceof Form);
                $Form   = $params;
                $domain = $Form->getValue('master_domain');

                $this->_disableLocalDelivery($domain);
                break;

            case 'Ctrl_Siteworx_DomainsSlave:addCommit':
                assert($params instanceof Form);
                $Form   = $params;
                $domain = $Form->getValue('domain');

                $this->_disableLocalDelivery($domain);
                break;

            default:
                return;
        }//end switch
    }

    /**
     * Disabled local delivery on the specified domain.
     *
     * @param string $domain
     */
    private function _disableLocalDelivery($domain)
    {
        Qmail::delVirtualDomain($domain);
        Qmail::delRcptHost($domain);
    }
}
