<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jon Daniel <jdaniel@interworx.com>
 * @date    Feb 22, 2011
 */

/**
 * Controller for displaying session histories.
 *
 * @package    InterWorx
 * @subpackage Ctrl
 */
class Ctrl_Nodeworx_HistoryChanges extends Ctrl_Nodeworx_Plugin
{

    /**
     * Index action.
     */
    public function indexAction()
    {
        $ChangesForm = Form::getInstance('Form_NW_History_Changes');

        $View       = $this->getView();
        $options    = [];
        $session_id = $ChangesForm->getValue('session');

        if (!empty($session_id)) {
            $options['session_id'] = $session_id;
        }

        $Payload = Payload_Factory_NW_History::changes($options);
        $Form    = Form::getInstance('Form_Search');

        $Payload->setRefreshAction('viewChanges');
        $Payload->setRefreshActionInput($ChangesForm);

        $HistorySession = Model_History_Session::find($session_id);
        $nickname       = $HistorySession->nickname;
        $email          = $HistorySession->email;
        $ipaddr         = $HistorySession->remote_ipaddr;

        $user_display = $email;
        if (!empty($nickname)) {
            $user_display = "{$nickname} ({$user_display})";
        }
        $user_display .= ' ' . $ipaddr;

        $View->assign('search_form', $Form);
        $View->assign('template', 'changes');
        $View->assign('title', "##LG_CHANGES## :: {$user_display}");
        $View->assign('iw_payload', $Payload);
    }

    /**
     * View the specified session's changes.
     *
     * @param Form_NW_History_Changes $Form
     * @return void
     */
    public function viewChangesCommitAction(Form_NW_History_Changes $Form)
    {
        $session_id = $Form->getValue('session');
        $options    = [];

        if (!empty($session_id)) {
            $options['session_id'] = $session_id;
        }

        $Payload = Payload_Factory_NW_History::changes($options);

        $Payload->setRefreshActionInput($Form);
        $this->_displayPayload($Payload);
    }
}
