<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Form
 * @author     Jon Daniel <jdaniel@interworx.com>
 * @date       Feb 22, 2011
 */

/**
 * Select which user's session history do you want to see.
 *
 * @package    InterWorx
 * @subpackage Form
 */
class Form_NW_History_Sessions extends Form
{

    /**
     * Initialize.
     */
    protected function _initialize()
    {
        parent::_initialize('history_sessions');

        $this->addInput(new Input_Select('user'))
        ->setDataSource(new DataSource_ArrayAssoc($this->_getUserData()))
        ->setRequired(false);

        $this->setSaveValue('##LG_SELECT##');
        $this->setCancelDisplay(false);
        $this->setUseHandler(false);
    }

    /**
     * Gets the user data.
     *
     * @return array
     */
    private function _getUserData()
    {
        $Search = new IWorx_Search_History_Sessions();

        $Search->groupBy('global_uid');
        $Search->addSelectableFields(array( 'global_uid', 'email', 'nickname' ));

        $data    = $Search->search();
        $ds_data = [];

        foreach ($data as $row) {
            $value = $row->email;
            if (!empty($row->nickname)) {
                $value = "{$row->nickname} ($value)";
            }

            $ds_data[$row->global_uid] = $value;
        }

        return $ds_data;
    }
}
