<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Formatter
 * @author     Jon Daniel <jdaniel@interworx.com>
 * @date       Feb 22, 2011
 */

/**
 * Groups by global uid.
 *
 * @package    InterWorx
 * @subpackage Formatter
 */
class IWorx_Formatter_Grouper_History_Action extends IWorx_Formatter_Grouper
{

    /**
     * Pass in the name of the global uid field.
     *
     * @param string $field
     */
    public function __construct($field)
    {
        parent::__construct($field, 'desc');
    }

    /**
     * Format the grouper.
     *
     * @param string $value
     * @return string
     */
    public function format($value)
    {
        $Action      = Model_History_Action::find($value);
        $ctrl_name   = $Action->ctrl_name;
        $action_name = $Action->action_name;

        $doc = IWorxReflect::ctrlActionToDoc($ctrl_name, $action_name);

        $query_string = IWorxSecUtil::html_entity_decode($Action->query_string);
        $query_string = (array)json_decode($Action->query_string);

        $q = [];

        foreach ($query_string as $key => $value) {
            if (is_array($value)) {
                $value = implode(',', $value);
            }
            $q[] = "{$key}={$value}";
        }

        if (!empty($q)) {
            $doc .= ' ' . implode(', ', $q);
        }

        $ctrl_short = substr($ctrl_name, strrpos($ctrl_name, '_') + 1);

        $result = $ctrl_short . ': ' . $doc;
        if (strlen($result) > 200) {
            $result = substr($result, 0, 200) . '...';
        }
        return $result;
    }
}
