<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Payload
 * @author     Jon Daniel <jdaniel@interworx.com>
 * @date       Feb 21, 2011
 */

/**
 * Formatter for the query string.
 *
 * @package    InterWorx
 * @subpackage Payload
 */
class IWorx_Formatter_History_QueryString extends IWorx_Formatter_NoneIfEmpty
{

    /**
     * Format.
     *
     * @param mixed  $value The value being rendered
     * @param object $row   The data for the entire row
     * @param array  $data  The entire dataset of the Payload
     *
     * @return string
     */
    public function format($value, $row, array $data)
    {
        $value = IWorxSecUtil::html_entity_decode($value);

        if ($this->_isJson($value) === true) {
            $value = (array)json_decode($value);
        }

        if (empty($value)) {
            return parent::format($value, $row, $data);
        }

        $output = [];
        foreach ($value as $key => $v) {
            $key = $this->_bold($key);
            $v   = $this->_bold($v);

            $output[] = "##LG_FIELD_EQUALS_FIELD|{$key}|{$v}##";
        }

        return implode('<br>', $output);
    }

    /**
     * Make the given text bold.
     *
     * @param string $value
     * @return string
     */
    private function _bold($value)
    {
        return "<b>{$value}</b>";
    }

    /**
     * Checks to see if the specified string is a JSON array.
     *
     * @param string $value
     * @return boolean
     */
    private function _isJson($value)
    {
        $tmp = json_decode($value);
        return json_last_error() == 0;
    }
}
