<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Search
 * @author     Jon Daniel <jdaniel@interworx.com>
 * @date       Feb 18, 2011
 */

/**
 * History of Session Actions search engine.
 *
 * @package    InterWorx
 * @subpackage Search
 */
class IWorx_Search_History_Changes extends IWorx_Search_Engine
{
    const TABLE_HISTORY_CHANGES  = 'hc';
    const TABLE_HISTORY_ACTIONS  = 'ha';
    const TABLE_HISTORY_SESSIONS = 'hs';

    /**
     * Construct.
     */
    public function __construct()
    {
        parent::__construct();

        $this->setBaseTable(self::TABLE_HISTORY_ACTIONS);
    }

    /**
     * Initialize the selectable field.
     */
    protected function _initSelectableFields()
    {
        $hs  = self::TABLE_HISTORY_SESSIONS;
        $ha  = self::TABLE_HISTORY_ACTIONS;
        $hc  = self::TABLE_HISTORY_CHANGES;

        $this->_addSelectableFieldFrom($hs, 'session_id');

        $this->_addAliasedSelectableFieldFrom(
            $hs,
            'global_uid',
            'session_global_uid'
        );

        $this->_addAliasedSelectableFieldFrom(
            $hs,
            'nickname',
            'session_nickname'
        );

        $this->_addAliasedSelectableFieldFrom(
            $hs,
            'email',
            'session_email'
        );

        $this->_addSelectableFieldFrom(
            $hs,
            'user_type',
            "IF( {$hs}.global_uid LIKE 'nw%',
                                         'nodeworx',
                                         'siteworx' )"
        );

        $this->_addSelectableFieldFrom($hc, 'input_name');
        $this->_addSelectableFieldFrom($hc, 'input_label');
        $this->_addSelectableFieldFrom($hc, 'initial_value');
        $this->_addSelectableFieldFrom($hc, 'current_value');

        $this->_addSelectableFieldFrom($ha, 'action_id');
        $this->_addSelectableFieldFrom($ha, 'query_string');
        $this->_addSelectableFieldFrom($ha, 'ctrl_name');
        $this->_addSelectableFieldFrom($ha, 'action_name');

        $this->_addSelectableDateFieldFrom($ha, 'timestamp');

        $this->_addSelectableFieldFrom($ha, 'working_user');
        $this->_addSelectableFieldFrom($ha, 'working_domain');
        $this->_addSelectableFieldFrom($ha, 'env');
    }

    /**
     * Init the table objects.
     */
    protected function _initTableObjects()
    {
        parent::_initTableObjects();

        $this->_makeAndAddTable(
            'history_sessions',
            self::TABLE_HISTORY_SESSIONS
        );

        $this->_makeAndAddTable(
            'history_actions',
            self::TABLE_HISTORY_ACTIONS
        );

        $this->_makeAndAddTable(
            'history_changes',
            self::TABLE_HISTORY_CHANGES
        );
    }

    /**
     * Init the table object links.
     */
    protected function _initTableObjectLinks()
    {
        parent::_initTableObjectLinks();

        $this->_linkTables(
            self::TABLE_HISTORY_SESSIONS,
            self::TABLE_HISTORY_ACTIONS,
            'session_id',
            'session_id',
            null,
            'INNER'
        );

        $this->_linkTables(
            self::TABLE_HISTORY_ACTIONS,
            self::TABLE_HISTORY_CHANGES,
            'action_id',
            'action_id',
            null,
            'LEFT'
        );
    }

    /**
     * Build the Where Fragment.
     *
     * @param string $index
     * @param string $values
     * @param array  $having
     * @return string
     */
    protected function _buildWhereFragment($index, $values, array &$having)
    {
        if ($values->$index === '') {
            return '';
        }

        switch ($index) {
            case 'search':
                $hs  = self::TABLE_HISTORY_SESSIONS;
                $ha  = self::TABLE_HISTORY_ACTIONS;
                $hc  = self::TABLE_HISTORY_CHANGES;

                $this->addJoinsTo($hs);
                $this->addJoinsTo($ha);
                $this->addJoinsTo($hc);

                $this->setPartialMatches(true);

                $search = "*{$values->$index}*";

                $q = [];

                $q[] = $this->expandTextRange($search, "{$ha}.ctrl_name");
                $q[] = $this->expandTextRange($search, "{$ha}.action_name");
                $q[] = $this->expandTextRange($search, "{$ha}.env");
                $q[] = $this->expandTextRange($search, "{$ha}.timestamp");

                $q[] = $this->expandTextRange($search, "{$hs}.email");
                $q[] = $this->expandTextRange($search, "{$hs}.nickname");
                $q[] = $this->expandTextRange($search, "{$hs}.remote_ipaddr");

                $q[] = $this->expandTextRange($search, "{$hc}.input_name");
                $q[] = $this->expandTextRange($search, "{$hc}.initial_value");
                $q[] = $this->expandTextRange($search, "{$hc}.current_value");

                return ' AND ' . implode(' OR ', $q);
            break;

            default:
                return parent::_buildWhereFragment($index, $values, $having);
            break;
        }//end switch
    }

    /**
     * Reset the search engine.
     */
    public function resetEngine()
    {
        $this->_addSearchParameter('search', '');

        parent::resetEngine();
    }
}
