<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Search
 * @author     Jon Daniel <jdaniel@interworx.com>
 * @date       Feb 21, 2011
 */

/**
 * History of Session Actions search engine.
 *
 * @package    InterWorx
 * @subpackage Search
 */
class IWorx_Search_History_Sessions extends IWorx_Search_Engine
{

    const TABLE_HISTORY_CHANGES  = 'hc';
    const TABLE_HISTORY_ACTIONS  = 'ha';
    const TABLE_HISTORY_SESSIONS = 'hs';

    /**
     * Constructor.
     */
    public function __construct()
    {
        parent::__construct();

        $this->setBaseTable(self::TABLE_HISTORY_SESSIONS);
    }

    /**
     * Initialize the selectable field.
     */
    protected function _initSelectableFields()
    {
        $hs = self::TABLE_HISTORY_SESSIONS;
        $ha = self::TABLE_HISTORY_ACTIONS;

        $this->_addSelectableFieldFrom($hs, 'session_id');
        $this->_addSelectableFieldFrom($hs, 'global_uid');
        $this->_addSelectableFieldFrom($hs, 'email');
        $this->_addSelectableFieldFrom($hs, 'nickname');
        $this->_addSelectableFieldFrom($hs, 'remote_ipaddr');
        $this->_addSelectableFieldFrom($hs, 'last_ctrl');
        $this->_addSelectableFieldFrom($hs, 'last_action');

        $format = "MAX( UNIX_TIMESTAMP( {$hs}.last_action_timestamp ) )";
        $this->_addSelectableFieldFrom($hs, 'last_action_timestamp', $format);

        $this->_addSelectableFieldFrom(
            $hs,
            'user_type',
            "IF( {$hs}.global_uid LIKE 'nw%',
                                         'nodeworx',
                                         'siteworx' )"
        );

        $this->_addSelectableFieldFrom(
            $hs,
            'session_start',
            "UNIX_TIMESTAMP( {$hs}.timestamp )"
        );

        // have to group by session id to use this.
        $this->_addSelectableFieldFrom(
            $hs,
            'session_count',
            "COUNT( DISTINCT {$hs}.session_id )"
        );

        $this->_addSelectableFieldFrom(
            $ha,
            'action_count',
            "COUNT( DISTINCT {$ha}.action_id )"
        );

        $this->_addSelectableFieldFrom($ha, 'env');
    }

    /**
     * Init the table objects.
     */
    protected function _initTableObjects()
    {
        parent::_initTableObjects();

        $this->_makeAndAddTable(
            'history_sessions',
            self::TABLE_HISTORY_SESSIONS
        );

        $this->_makeAndAddTable(
            'history_actions',
            self::TABLE_HISTORY_ACTIONS
        );
    }

    /**
     * Init the table object links.
     */
    protected function _initTableObjectLinks()
    {
        parent::_initTableObjectLinks();

        $this->_linkTables(
            self::TABLE_HISTORY_SESSIONS,
            self::TABLE_HISTORY_ACTIONS,
            'session_id',
            'session_id',
            null,
            'INNER'
        );
    }
}
