<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jon Daniel <jdaniel@interworx.com>
 * @date    Feb 18, 2011
 */

/**
 * Plugin for User History
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_History extends Plugin_NodeworxFeature
{

    const DAYS_TO_KEEP = 'days_to_keep';
    //const DISPLAY_ENVS = 'display_envs';

    protected $_reseller_allowed = false;

    protected $_nw_user_perm_name = 'userhistory';

    protected $_nw_user_perm_label = '##LG_VIEW_USER_HISTORY##';

    private $_History_Session = false;

    private $_days_to_keep = 30;

    private $_bad_controllers = array( 'Ctrl_Nodeworx_History',
                                     'Ctrl_Nodeworx_HistorySearch',
                                     'Ctrl_Nodeworx_HistoryChanges',
                                     'Ctrl_Nodeworx_HistorySessions',
                                     'Ctrl_Nodeworx_Notice',
                                     'Ctrl_Index',
                                     'Ctrl_Xhr' );

    private $_allowed_envs = array( 'WEB', 'CLI', 'API' );
    //private $_display_envs = array( 'WEB' );

    /**
     * Init the plugin.
     */
    protected function _init()
    {
        // this is first, if it isn't then you moved it and you suck.
        $this->_setupDatabaseTables();

        $days_to_keep = $this->getDatastoreVar(self::DAYS_TO_KEEP);
        //    $display_envs = $this->getDatastoreVar( self::DISPLAY_ENVS );

        if ($days_to_keep !== '') {
            $this->_days_to_keep = $days_to_keep;
        }

        //if( !empty( $display_envs ) && is_array( $display_envs ) ) {
        //  $this->_display_envs = $display_envs;
        //}

        parent::_init();
    }

    /**
     * Initialize this plugin's form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function initializeMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        $Group = $Form->addGroup('extra_options', '##LG_EXTRA_PLUGIN_SETTINGS##');
        $days_to_keep = $this->_days_to_keep;
        //$display_envs = $this->_display_envs;

        $Group->addInput(new Input_Integer(self::DAYS_TO_KEEP, $days_to_keep))
        ->setRange(1, Ini::get(Ini::CONSTANTS, 'unlimited'))
        ->setUnlimitedAllowed(true)
        ->setLabel('##LG_DAYS_TO_RETAIN_LOGS##')
        ->setDescription('##LG_DESC_DAYS_TO_RETAIN_LOGS##');

        //    $envs = array( 'WEB' => '##LG_WEB_ENV##',
        //                   'CLI' => '##LG_CLI_ENV##',
        //                   'API' => '##LG_API_ENV##' );
        //
        //    $Group->addInput( new Input_Set( self::DISPLAY_ENVS, $display_envs ) )
        //      ->setDataSource( new DataSource_ArrayAssoc( $envs ) )
        //      ->setMultipleAllowed( true )
        //      ->setLabel( '##LG_DISPLAY_ENVS##' );
    }

    /**
     * Process the form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function processMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        $days_to_keep = $Form->getValue(self::DAYS_TO_KEEP);
        //    $envs         = $Form->getValue( self::DISPLAY_ENVS );

        $this->setDatastoreVar(self::DAYS_TO_KEEP, $days_to_keep);
        //    $this->setDatastoreVar( self::DISPLAY_ENVS, $envs );
    }

    /**
     * Updates the NodeWorx Menu.
     *
     * @param IWorxMenuManager $MenuMan
     */
    public function updateNodeworxMenu(IWorxMenuManager $MenuMan)
    {
        if ($this->isCurrentNwUserAllowed() === true) {
            $new_data = array( 'text' => '##LG_SESSION_HISTORY##',
                         'url' => '/nodeworx/history',
                         'parent' => 'iw-menu-nw',
                         'class' => 'iw-i-log',
                         'disabled_for_reseller' => '1' );

            $MenuMan->addMenuItemAfter(
                'iw-menu-nw-subscriptions',
                'menu-session-history',
                $new_data
            );
        }
    }

    /**
     * Load the history session object.
     *
     * @param string $ctrl_act
     */
    private function _loadHistorySession($ctrl_act)
    {
        $has_session = $this->_hasActiveSession($ctrl_act);
        if ($has_session === true) {
            $History_Session = $this->_retreveHistorySession();

            if (is_null($History_Session)) {
                $History_Session = $this->_createHistorySession();
            }

            $this->_storeHistorySession($History_Session);
            $this->_History_Session = $History_Session;
        }
    }

    /**
     * Create the History Session object.
     *
     * @return Model_History_Session
     */
    private function _createHistorySession()
    {
        $History_Session = Model_History_Session::create();

        if (IWorxValidate::ip($_SERVER['REMOTE_ADDR']) === true) {
            $remote_ipaddr = $_SERVER['REMOTE_ADDR'];
        } else {
            $remote_ipaddr = '##LG_UNKNOWN##';
        }

        $InterWorx   = IW::Env()->getActiveSession()->getInterWorx();
        $WorkingUser = $InterWorx->getWorkingUser();

        $History_Session->remote_ipaddr = $remote_ipaddr;
        $History_Session->global_uid    = $WorkingUser->getGlobalId();
        $History_Session->nickname      = $WorkingUser->getNickname();
        $History_Session->email         = $WorkingUser->getEmail();
        $History_Session->timestamp     = IW::now();
        $History_Session->last_action_timestamp = IW::now();

        $History_Session->save();

        return $History_Session;
    }

    /**
     * Store the session history in the user's session.
     *
     * @param Model_History_Session $History_Session
     */
    private function _storeHistorySession(Model_History_Session $History_Session)
    {
        $_SESSION['interworx_history_session'] = $History_Session->getPrimaryId();
    }

    /**
     * Retreve the History Session object.
     *
     * We grab the object's id from the session and load it.  If something weird
     * happens and the data stored in the session is invalid, we catch the
     * exception and return null.
     *
     * @return Model_History_Session|null
     */
    private function _retreveHistorySession()
    {
        try {
            if (isset($_SESSION['interworx_history_session'])) {
                $session_id      = $_SESSION['interworx_history_session'];
                $History_Session = Model_History_Session::load($session_id);

                return $History_Session;
            }
        } catch (IWorx_Exception_Model_InvalidRecord $e) {
            //eat it.
        }
        return null;
    }

    /**
     * Post actions.
     *
     * @param string        $ctrl_act
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param string        $params
     */
    public function postAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        if ($this->_hasActiveSession($ctrl_act) !== true) {
            return;
        }

        // If something database related goes wrong it really shouldn't cause
        // any other issues like actions not completing.  So in the event that
        // something Model-related breaks during the logging, we just mark it in
        // the iworx.log and pretend like everything is fine.  Better this than
        // have the history plugin potentially causing all Commits to break.
        //   -Jon (jdaniel@interworx.com)
        try {
            if ($this->_shouldLogCommitAction($Ctrl, $action, $params) === true) {
                $this->_loadHistorySession($ctrl_act);
                $this->_logAction($Ctrl, $action, $params);
            }
        } catch (IWorx_Exception_Model $e) {
            $msg = "Failed logging {$ctrl_act}.  Cause: {$e->getMessage()}";
            IW::getLog()->error($msg);
        }
    }

    /**
     * Checks to see if the commit action should be logged.
     *
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param array|Form    $params
     * @return boolean
     */
    private function _shouldLogCommitAction(Ctrl_Abstract $Ctrl, $action, $params)
    {
        if (!$Ctrl->isViewSuccess()) {
            return false;
        }

        // if it's not a Commit action, we don't care about it.
        if (substr($action, -6) !== 'Commit') {
            return false;
        }

        // Only log if it's one of the ENVs we care about.
        $current_env = IW::Env()->getUserInterfaceTypeString();
        if (in_array($current_env, $this->_allowed_envs) !== true) {
            return false;
        }

        if (in_array(get_class($Ctrl), $this->_bad_controllers) === true) {
            return false;
        }

        if (( $params instanceof Form ) === false) {
            return false;
        }

        return true;
    }

    /**
     * Log the action.
     *
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param string        $params
     */
    private function _logAction(Ctrl_Abstract $Ctrl, $action, $params)
    {
        $History_Session = $this->_History_Session;
        $History_Action  = Model_History_Action::create();

        $InterWorx   = IW::Env()->getActiveSession()->getInterWorx();
        $WorkingUser = $InterWorx->getWorkingUser();

        $History_Action->session_id   = $History_Session->getPrimaryId();
        $History_Action->ctrl_name    = get_class($Ctrl);
        $History_Action->action_name  = $action;
        $History_Action->env          = IW::Env()->getUserInterfaceTypeString();
        $History_Action->timestamp    = IW::now();
        $History_Action->query_string = $this->_getQueryString($params);
        $History_Action->working_user = $WorkingUser->getEmail();

        if ($InterWorx instanceof SiteWorx) {
            $History_Action->working_domain = $InterWorx->getWorkingDomainName();
        }

        $History_Action->save();

        $History_Session->last_ctrl   = get_class($Ctrl);
        $History_Session->last_action = $action;
        $History_Session->last_action_timestamp = IW::now();

        $History_Session->save();

        if ($params instanceof Form) {
            $this->_logChanges($History_Action, $params);
        }
    }

    /**
     * Gets the query string from the Input_Form.
     *
     * @param mixed $Form
     * @return string
     */
    private function _getQueryString($Form)
    {
        if ($Form instanceof Form) {
            $InputForm    = $Form->getInputForm();
            $query_string = [];
            foreach ($InputForm as $Group) {
                foreach ($Group as $Input) {
                    if ($Input instanceof Input_Password) {
                        continue;
                    }

                    $query_string[$Input->getName()] = $Input->getCurrentValue();
                }
            }

            return json_encode($query_string);
        }
        return null;
    }

    /**
     * Log the changes.
     *
     * @param Model_History_Action $History_Action
     * @param Form                 $Form
     */
    private function _logChanges(Model_History_Action $History_Action, Form $Form)
    {
        if ($History_Action->action_name === 'loginCommit') {
            return;
        }
        foreach ($Form as $Group) {
            foreach ($Group as $Input) {
                if ($Input->isChanged() !== true) {
                    continue;
                }
                if ($Input instanceof Input_PasswordConfirm) {
                    continue;
                }
                if ($Input->getName() === 'security_token') {
                    continue;
                }
                $initial_value = $Input->getInitialValue();
                $current_value = $Input->getCurrentValue();

                // Hack because isChanged appears to mishandle when an initial value
                // is null and the current value is an empty string.  I really didn't
                // want to go change something like that in the form system without
                // a lot of discussion, so I added this duct tape fix.
                //
                //   - Jon (jdaniel@interworx.com)
                //
                if (is_null($initial_value) && $current_value == '') {
                    continue;
                }
                //end duct tape fix.

                if ($Input instanceof Input_Password) {
                    $current_value = 'xxxxxxxx';
                }

                if (is_array($initial_value)) {
                    $initial_value = json_encode($initial_value);
                }

                if (is_array($current_value)) {
                    $current_value = json_encode($current_value);
                }

                $History_Change = Model_History_Change::create();

                $History_Change->action_id     = $History_Action->getPrimaryId();
                $History_Change->input_name    = $Input->getName();
                $History_Change->input_label   = $Input->getLabel();
                $History_Change->initial_value = $initial_value;
                $History_Change->current_value = $current_value;

                $History_Change->save();
            }//end foreach
        }//end foreach
    }

    /**
     * Executes CRON jobs.
     *
     * @param string $runtime
     */
    public function executeCron($runtime)
    {
        if ($runtime == IWorxCron::DAILY) {
            $this->_rotateSessionHistory();
        }
    }

    /**
     * Delete all the sessions that are past the expiration point.
     */
    private function _rotateSessionHistory()
    {
        if ($this->_days_to_keep == Ini::get(Ini::CONSTANTS, 'unlimited')) {
            return;
        }

        $one_day = 60 * 60 * 24;
        $time    = time() - ( $one_day * $this->_days_to_keep ); // X days ago.
        $q_time  = q($time);

        // Doing a multi table delete, because it's easier than doing it through
        // the search engine and deleting each record individually.

        $sql = "DELETE hs,
                   ha,
                   hac
            FROM `history_sessions` AS hs
            LEFT JOIN `history_actions` AS ha
              ON ha.session_id = hs.session_id
            LEFT JOIN `history_changes` AS hac
              ON ha.action_id = hac.action_id
            WHERE UNIX_TIMESTAMP( ha.timestamp ) < {$q_time}";

        IWorxDbUtil::db_query(IW::DB(), $sql);
    }

    /**
     * Sets up the database tables.
     */
    private function _setupDatabaseTables()
    {
        $sql = 'CREATE TABLE IF NOT EXISTS `history_sessions` (
      `session_id` INT( 11 ) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY ,
      `remote_ipaddr` VARCHAR( 39 ) NOT NULL ,
      `global_uid` VARCHAR( 13 ) NOT NULL ,
      `nickname` VARCHAR( 128 ) NULL ,
      `email` VARCHAR( 128 ) NOT NULL ,
      `timestamp` DATETIME NOT NULL,
      `last_ctrl` VARCHAR( 60 ) NULL,
      `last_action` VARCHAR( 60 ) NULL,
      `last_action_timestamp` DATETIME NULL,
    INDEX (  `global_uid` )
    ) ENGINE = MYISAM';

        IWorxDbUtil::db_query(IW::DB(), $sql);

        $sql = 'CREATE TABLE IF NOT EXISTS `history_actions` (
      `action_id` INT( 11 ) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY ,
      `session_id` INT( 11 ) UNSIGNED NOT NULL ,
      `ctrl_name` VARCHAR( 60 ) NOT NULL ,
      `action_name` VARCHAR( 60 ) NOT NULL ,
      `query_string` TEXT NOT NULL ,
      `working_user` VARCHAR( 60 ) NOT NULL ,
      `working_domain` VARCHAR( 60 ) NULL ,
      `env` VARCHAR( 60 ) NOT NULL ,
      `timestamp` DATETIME NOT NULL,
    INDEX (  `session_id` )
    ) ENGINE = MYISAM';

        IWorxDbUtil::db_query(IW::DB(), $sql);

        $sql = 'CREATE TABLE IF NOT EXISTS `history_changes` (
      `change_id` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
      `action_id` INT(11) UNSIGNED NOT NULL ,
      `input_name` VARCHAR(60) NOT NULL ,
      `input_label` VARCHAR(60) NOT NULL ,
      `initial_value` TEXT NULL,
      `current_value` TEXT NOT NULL,
    PRIMARY KEY  (`change_id`),
    KEY `action_id` (`action_id` )
    ) ENGINE=MyISAM';

        IWorxDbUtil::db_query(IW::DB(), $sql);
    }
}
