<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Oct 7, 2009
 */

/**
 * Installatron Plugin.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_Installatron extends Plugin
{

    /**
     * Plugin init.
     */
    protected function _init()
    {
        if (!is_dir('/var/installatron')) {
            $this->_disable('Can not find /var/installatron. ' .
                       'Install installatron first.');
            return;
        }

        if (!is_dir('/usr/local/installatron')) {
            $this->_disable('Can not find /usr/local/installatron. ' .
                       'Install installatron first.');
            return;
        }

        if (!is_link('/usr/local/interworx/html/installatron')) {
            symlink(
                '/var/installatron/frontend',
                '/usr/local/interworx/html/installatron'
            );
        }
    }

    /**
     * Get category.
     *
     * @return string
     */
    public function getCategory()
    {
        return Plugin_Category::SCRIPT_INSTALLERS;
    }

    /**
     * Get priority.
     *
     * @return integer
     */
    public function getPriority()
    {
        return 50;
    }

    /**
     * Run installatron.
     */
    public function run()
    {
        session_write_close();
        include '/var/installatron/frontend/index.php';
    }

    /**
     * Get user data.
     *
     * @param string $user
     * @return array;
     */
    public function getUserData($user)
    {
        if ($user === 'admin') {
            // master nodeworx account
            $data = $this->_getAdminData();
        } elseif (strpos($user, '@') !== false) {
            // reseller
            $data = $this->_getResellerData($user);
        } else {
            // siteworx account
            $data = $this->_getSiteworxData($user);
        }

        return $data;
    }

    /**
     * Get admin data helper.
     *
     * @return array
     */
    private function _getAdminData()
    {
        $id         = NodeWorx::MASTER_ID;
        $path_home  = realpath('/usr/local/interworx');
        $path_home .= "/var/installatron/{$id}";
        $NW         = new NodeWorx(NodeWorx::MASTER_ID);

        $r = array( 'user'         => 'admin',
                'type'         => 'admin',
                'system_user'  => 'iworx',
                'system_group' => 'iworx',
                'path_home'    => $path_home,
                'path_data'    => $path_home . '/.installatron',
                'path_pref'    => $path_home . '/.installatron',
                'email'        => $NW->getMasterUser()->getEmail()
                );
        return $r;
    }

    /**
     * Get reseller data helper.
     *
     * @param string $user
     * @return array
     */
    private function _getResellerData($user)
    {
        $NW         = new NodeWorx($user);
        $id         = $NW->getId();
        $path_home  = realpath('/usr/local/interworx');
        $path_home .= "/var/installatron/{$id}";

        $r = array( 'user'         => $user,
                'type'         => 'reseller',
                'system_user'  => 'iworx',
                'system_group' => 'iworx',
                'parent'       => '!root',
                'path_home'    => $path_home,
                'path_data'    => $path_home . '/.installatron',
                'path_pref'    => $path_home . '/.installatron',
                'email'        => $NW->getMasterUser()->getEmail()
                );
        return $r;
    }

    /**
     * Get siteworx data helper.
     *
     * @param string $user
     * @return array
     */
    private function _getSiteworxData($user)
    {
        $NW = new NodeWorx(NodeWorx::MASTER_ID);
        $SW = $NW->getSiteWorx($user);
        if ($SW === false || $SW->getId() == -1) {
            return [];
        }
        $mysql_host = $SW->getMySQLDriver()->getHostname();
        if ('unix(/var/lib/mysql/mysql.sock)' === $mysql_host) {
            $mysql_host = '127.0.0.1';
        }

        $package_root = realpath($SW->getPackageRoot());

        if ($SW->getParentId() == NodeWorx::MASTER_ID) {
            $parent = 'admin';
        } else {
            $parent = $SW->getParentEmail();
        }
        $r =
        array( 'user'         => $user,
             'system_user'  => $SW->getUniqname(),
             'system_group' => $SW->getUniqname(),
             'type'         => 'user',
             'parent'       => $parent,
             'path_home' => $package_root,
             'path_data' => $package_root . '/.installatron',
             'path_pref' => $package_root . '/.installatron',
             'email' => $SW->getWorkingUser()->getEmail(),
             'mysql' => $SW->getPackage()->getOptionValue(Option::MYSQL_DBS),
             'mysql_host' => $mysql_host,
             'quota' => $SW->getPackage()->getOptionValue(Option::STORAGE)
             );

        return $r;
    }

    /**
     * Customizations to siteworx menu can be done here.
     *
     * @param IWorxMenuManager $MenuMan
     */
    public function updateSiteworxMenu(IWorxMenuManager $MenuMan)
    {
        $new_data = array( 'text'   => 'Installatron',
                       'url'    => '/siteworx/installatron?action=launch',
                       'parent' => 'iw-menu-features',
                       'class'  => 'iw-i-plugin' );
        $MenuMan->addMenuItemAfter(
            'iw-menu-features-ssl',
            'installatron',
            $new_data
        );
    }

    /**
     * Integrate into the nodeworx menu.
     *
     * @param IWorxMenuManager $MenuMan
     */
    public function updateNodeworxMenu(IWorxMenuManager $MenuMan)
    {
        $new_data = array( 'text' => 'Installatron',
                       'url' => '/nodeworx/installatron?action=launch',
                       'parent' => 'iw-menu-sw',
                       'class' => 'iw-i-plugin' );

        $MenuMan->addMenuItemAfter(
            'iw-menu-sw-import',
            'installatron',
            $new_data
        );
    }
}
