<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>x
 * @date    Oct 9, 2009
 */

/**
 * Installatron integration backend.php class.
 *
 * @package    InterWorx
 * @subpackage Installatron-Lib
 */
class i_custom extends i_panel
{
    /**
     * The control panel name.
     *
     * @return string Control panel name
     */
    public function getName()
    {
        return 'InterWorx';
    }

    /**
     * The control panel version.
     *
     * @return string Control panel version
     */
    public function getVersion()
    {
        return '4.0.0';
    }

    /**
     * Information about a user.
     *
     * @param string $u Username of user
     * @return  array   User information array
     */
    public function getUser($u)
    {
  #    echo "\n";
  #    echo $u;
  #    debug_print_backtrace();
  #    echo "\n";
        if (is_null($u)) {
            return [];
        }

        $Soap = new SoapClient('https://127.0.0.1:2443/nodeworx/soap.php?wsdl');
        $key  = array( 'sessionid' => getenv('IWORX_SESSION_ID') );

        $current = itron::$session['user'];

        if ($current === 'admin') {
          // master nodeworx account
            $result = $Soap->route(
                $key,
                'nodeworx/installatron',
                'listUserData',
                array( 'user' => $u )
            );
        } elseif (strpos($current, '@') !== false) {
          // reseller
            $result = $Soap->route(
                $key,
                'nodeworx/installatron',
                'listUserData',
                array( 'user' => $u )
            );
        } else {
          // siteworx account
            $result = $Soap->route(
                $key,
                'siteworx/installatron',
                'listUserData',
                array( 'user' => $u )
            );
        }

        $r = $result['payload'];
        return $r;
    }

    /**
     * Users owned by a reseller.
     *
     * @param mixed $u Reseller username
     * @return array List of users
     */
    public function getUsers($u)
    {
        $Soap = new SoapClient('https://127.0.0.1:2443/nodeworx/soap.php?wsdl');
        $key  = array( 'sessionid' => getenv('IWORX_SESSION_ID') );

        if ($u === 'admin') {
          // master nodeworx account
            $result = $Soap->route(
                $key,
                'nodeworx/siteworx',
                'listMasterDomains',
                array()
            );
        } elseif (strpos($u, '@') !== false) {
          // reseller
            $result = $Soap->route(
                $key,
                'nodeworx/siteworx',
                'listMasterDomains',
                array()
            );
        } else {
          // siteworx account
            $result = [];
        }

        return $result['payload'];
    }

    /**
     * Domains owned by a user.
     *
     * @return array List of "effective" user's domains
     */
    public function getDomains()
    {
        $Soap = new SoapClient('https://127.0.0.1:2443/nodeworx/soap.php?wsdl');

        $key = array( 'sessionid' => getenv('IWORX_SESSION_ID') );

        $u = itron::$session['effective_user'];

        if ($u === 'admin') {
          // master nodeworx account
            $result = $Soap->route(
                $key,
                'nodeworx/siteworx',
                'listMasterDomains',
                array()
            );
        } elseif (strpos($u, '@') !== false) {
          // reseller
            $result = $Soap->route(
                $key,
                'nodeworx/siteworx',
                'listMasterDomains',
                array()
            );
        } else {
          // siteworx account
            $result = $Soap->route(
                $key,
                'siteworx/domains/slave',
                'list',
                array()
            );
        }

        $r = $result['payload'];
        return $r;
    }

    /**
     * Install location as a server path.
     *
     * @param string $httphttps "http" | "https"
     * @param string $domain    Domain part of the install URL
     * @param string $path      Path part of the install URL
     * @param string $subdomain Sub-domain part of the install URL
     * @return string Server path to the install location
     */
    public function getInstallPath($httphttps, $domain, $path, $subdomain)
    {
        $Soap = new SoapClient('https://127.0.0.1:2443/nodeworx/soap.php?wsdl');

        $key = array( 'sessionid' => getenv('IWORX_SESSION_ID') );

        $u = itron::$session['effective_user'];

        $input = array( 'proto'  => $httphttps,
                    'domain' => $domain,
                    'path'   => $path,
                    'subdomain' => $subdomain );

        if ($u === 'admin') {
          // master nodeworx account
            $result = $Soap->route(
                $key,
                'nodeworx/installatron',
                'listInstallPath',
                $input
            );
        } elseif (strpos($u, '@') !== false) {
          // reseller
            $result = $Soap->route(
                $key,
                'nodeworx/installatron',
                'listInstallPath',
                $input
            );
        } else {
          // siteworx account
            $result = $Soap->route(
                $key,
                'siteworx/installatron',
                'listInstallPath',
                $input
            );
        }

        $r = $result['payload'];
        return $r;
    }

    /**
     * Databases owned by the effective user.
     *
     * @param string $type Database type
     * @return array  User's databases
     */
    public function getDBs($type)
    {
        assert($type === 'mysql');

        $Soap = new SoapClient('https://127.0.0.1:2443/nodeworx/soap.php?wsdl');

        $key = array( 'sessionid' => getenv('IWORX_SESSION_ID') );

        $result = $Soap->route(
            $key,
            'siteworx/mysql/db',
            'list',
            array( 'format' => 'display_values' )
        );

        return $result['payload'];
    }

    /**
     * Create a database.
     *
     * + If your panel requires each database name and username
     * start with the control panel user's username, note that the
     * incoming $name value does not have the username prepended.
     *
     * @param string $name New database name +
     * @param string $user New database username +
     * @param string $pass New database username's password
     * @param string $type Database type (must be "mysql")
     * @param string $host New database host (as originally returned by
     *                     getUser(effective_user).mysql_host).
     * @return boolean true if sucessful; false otherwise
     */
    public function createDB($name, $user, $pass, $type, $host)
    {
        assert($type === 'mysql');

        $prefix = itron::$session['system_user'] . '_';
        $name = substr($name, strlen($prefix));
        $user = substr($user, strlen($prefix));

        $Soap = new SoapClient('https://127.0.0.1:2443/nodeworx/soap.php?wsdl');

        $key       = array( 'sessionid' => getenv('IWORX_SESSION_ID') );
        $ctrl_name = 'siteworx/mysql/db';
        $action    = 'add';
        $input     = array( 'name' => $name );

        $result = $Soap->route($key, $ctrl_name, $action, $input);

        $ctrl_name = 'siteworx/mysql/user';
        $action    = 'add';
        $input     = array( 'name' => $user,
                        'password' => $pass,
                        'confirm_password' => $pass );

        $result = $Soap->route($key, $ctrl_name, $action, $input);

        assert($result[ 0 ] == 0);

        $ctrl_name = 'siteworx/mysql/perms';
        $action    = 'add';
        $input     = array( 'name' => $name,
                        'user' => $user );

        $result = $Soap->route($key, $ctrl_name, $action, $input);

        if ($result[ 'status' ] == 0) {
            $success = true;
        } else {
            $success = false;
        }

        return $success;
    }

    /**
     * Remove a database.
     *
     * If database usernamess are tied to database names,
     * the database username can also be removed.
     *
     * @param string $name Name of database to be removed
     * @param string $type Database type
     * @param string $host New database host (as originally returned by
     *                     getUser(effective_user).mysql_host).
     * @return boolean true if sucessful; false otherwise
     */
    public function removeDB($name, $type, $host)
    {
        assert($type === 'mysql');

        $Soap = new SoapClient('https://127.0.0.1:2443/nodeworx/soap.php?wsdl');

        $key       = array( 'sessionid' => getenv('IWORX_SESSION_ID') );
        $ctrl_name = 'siteworx/mysql/db';
        $action    = 'delete';
        $input     = array( 'name' => $name );

        $result = $Soap->route($key, $ctrl_name, $action, $input);

        if ($result[ 'status' ] == 0) {
            $success = true;
        } else {
            $success = false;
        }

        return $success;
    }

    /**
     * Init session.
     */
    public function initSession()
    {
        $cookies = $_SERVER['HTTP_COOKIE'];
        $cookies = explode(';', $cookies);
        foreach ($cookies as $cookie) {
            if (preg_match('/^interworx-cp=/', trim($cookie)) === 1) {
                list( $noo, $session_id ) = explode('=', $cookie);
            }
        }

        putenv('IWORX_SESSION_ID=' . $session_id);

        $Soap = new SoapClient('https://127.0.0.1:2443/nodeworx/soap.php?wsdl');

        $key = array( 'sessionid' => getenv('IWORX_SESSION_ID') );

        if (strstr($this->_getUri(), '/nodeworx/installatron') !== false) {
            $result = $Soap->route($key, 'nodeworx/users', 'isReseller', array());
            assert($result['status'] == 0);

            if ($result['payload'] === true) {
                $result = $Soap->route(
                    $key,
                    'nodeworx/users',
                    'listMasterUser',
                    array()
                );
                assert($result['status'] == 0);
                assert(isset($result['payload']['email']));
                $user = $result['payload']['email'];
            } else {
                $user = 'admin';
            }
        } else {
            $result = $Soap->route(
                $key,
                'siteworx/overview',
                'listMasterDomain',
                array()
            );
            assert($result['status'] == 0);
            $user = $result['payload'];
        }//end if

        itron::$session['user'] = trim($user);
        return true;
    }

    public function generateDB($type)
    {
        $existingDbs = $this->getDBs($type);
        do {
            $randstr = i_lib::randstr(2);
            $name    = itron::$session['system_user'].'_';
            $name   .= substr($randstr.itron::$input["installer"], 0, 7);
        } while (in_array($name, $existingDbs));
        return array($name, $name);
    }

    /**
     * Uri helper.
     */
    private function _getUri()
    {
        if (isset($_SERVER['SCRIPT_URL'])) {
            return $_SERVER['SCRIPT_URL'];
        }
        if (isset($_SERVER['REQUEST_URI'])) {
            return $_SERVER['REQUEST_URI'];
        }
        if (isset($_SERVER['REDIRECT_URL'])) {
            return $_SERVER['REDIRECT_URL'];
        }
    }
}
