<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Apr 7, 2010
 */

/**
 * Plugin class for the LiteSpeed webserver.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_Litespeed extends Plugin
{
    /**
     * Init the plugin, disabling if necessary.
     */
    protected function _init()
    {
        /*
         * You can conditionally disable the plugin
         */
        //$this->_disable( 'reason for disabling goes here, will go in iworx.log' );

        /*
         * You can set plugin variables as needed like this
         */
        //$this->_setVar( 'myvariable', 'somevalue' );

        /*
         * And retrieve them later like this
         */
        //$this->getVar( 'myvariable' );

        /*
         * You can set mixed datastore variables as needed like this
         */
        //$this->setDatastoreVar( 'storedvar', array( 1, 2, 3 ) );

        /*
         * And retrieve it later like this
         */
        //$this->getDatastoreVar( 'storedvar' );
    }

    /**
     * Process my edit form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function processMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        $enabled = $Form->getValue('status');
        if ($enabled === '1') {
            Ini::set(Ini::DRIVERS, 'http', 'LiteSpeed');
        } else {
            Ini::set(Ini::DRIVERS, 'http', 'Apache');
        }
        Ini::write();
    }

    /**
     * Plugin is enabled.
     *
     * @return array
     */
    public function canBeEnabled()
    {
        $result = [];

        if (!is_dir('/usr/local/lsws')) {
            $result[] = 'Cannot find /usr/local/lsws. ' .
                            'Please install LiteSpeed first!';
        }

        if (!IWorxFileSys::isFile('/usr/local/lsws/VERSION')) {
            $result[] = 'Cannot find /usr/local/lsws/VERSION file. ' .
                            'Please check your LiteSpeed installation.';
        }

        if (IWorxFileSys::isFile('/usr/local/lsws/VERSION')) {
            $version = trim(file_get_contents('/usr/local/lsws/VERSION'));
            if (version_compare($version, '4.0.14', '<')) {
                $result[] = 'LiteSpeed version 4.0.14 or higher is required! ' .
                            'You have version: ' . $version;
            }
        }

        return $result;
    }
}
