#!/home/interworx/bin/php -c/home/interworx/etc
<?php
/**
 * InterWorx Hosting Control Panel
 *
 * r1soft CDP integration script
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Sep 17, 2009
 * @Id      $Id$
 */

/**
 * Print auth ok and quit.
 *
 * @param integer $uid
 * @param integer $gid
 * @param string  $home
 */
function printAuthOk($uid, $gid, $home)
{
    echo 'auth_ok:1', "\n";
    echo "uid:{$uid}\n";
    echo "gid:{$gid}\n";
    echo "dir:home:{$home}\n";
    echo "end\n";
}

/**
 * Attempt to authenticate using the SOAP api and a session id.
 *
 * @param string $account
 * @param string $sessionid
 */
function tryApiSession($account, $sessionid)
{
    $contextOptions = array( 'ssl' =>
                           array( 'verify_peer' => false,
                                  'verify_peer_name' => false,
                                  'allow_self_signed' => true ) );
    $sslContext = stream_context_create($contextOptions);
    $client     = new SoapClient(
        'https://127.0.0.1:2443/nodeworx/soap?wsdl',
        array( 'stream_context' => $sslContext )
    );
    $key       = array( 'sessionid' => $sessionid );
    $ctrl_name = '/siteworx/r1soft';
    $action    = 'listAuthInfo';
    $input     = [];
    $result    = $client->route($key, $ctrl_name, $action, $input);

  // in CDP2 land, account could be an e-mail and it would work.
  // in CDP3 land, it has to be a the unix user.
    if (strpos($account, '@') !== false) {
        $mode = 'workinguser';
    } else {
        $mode = 'unixuser';
    }
    if ($result['status'] === 0 && $account === $result['payload'][$mode]) {
        $result['payload']['packageroot'] = realpath($result['payload']['packageroot']);
        printAuthOk(
            $result['payload']['uid'],
            $result['payload']['gid'],
            $result['payload']['packageroot']
        );
    } else {
        echo 'auth_ok:0', "\n";
    }
}

/**
 * Check login for cdp2.
 *
 * @param string $email_colon_domain
 * @param string $password
 * @param object $SW
 */
function checkLoginCdp2($email_colon_domain, $password, &$SW)
{
    $login_success = false;
    list( $email, $domain ) = explode(':', $email_colon_domain);
    $SW = new SiteWorx($domain);
    if ($SW->exists()) {
        if ($SW->login($email, $password)) {
            $login_success = true;
        }
    }
    return $login_success;
}

/**
 * Check login for cdp3.
 *
 * @param string $unixuser
 * @param string $password
 * @param object $SW
 */
function checkLoginCdp3($unixuser, $password, &$SW)
{
    $login_success = false;
    $SW = new SiteWorx();
    $SW->populateUniqname($unixuser);
    if ($SW->exists()) {
        $email = $SW->getMasterUser()->getEmail();
        if ($SW->login($email, $password)) {
            $login_success = true;
        }
    }
    return $login_success;
}

/**
 * Execution starts here.
 */
function main()
{
    ini_set('memory_limit', '24M');
    require_once('/usr/local/interworx/include/iworx.conf.php');

    $account  = getenv('BUAGENT_AUTH_ACCOUNT');
    $password = getenv('BUAGENT_AUTH_PASSWORD');

    $login_success = false;

    $SW = null;

  // in CDP2 land, account could be an e-mail and it would work.
  // in CDP3 land, it has to be a the unix user.
    if (strpos($account, '@') !== false) {
        $login_success = checkLoginCdp2($account, $password, $SW);
    } else {
        $login_success = checkLoginCdp3($account, $password, $SW);
    }

    if ($login_success) {
        printAuthOk(
            ShellAccount::getUID($SW->getUniqname()),
            ShellAccount::getGID($SW->getUniqname()),
            realpath($SW->getPackageRoot())
        );
    } else {
        tryApiSession($account, $password);
    }
}

main();
