#!/bin/bash
# Temp iwx backups script.  Add to root crontab at prefered time.  Packaged compressed, local backups only at this time.  Although you could use something like NFS to make the backups remote.

# Feature Backlog :
# incremental backup
# Custom pre and post backup scripts.
# FTP
# S3

# How many Daily copies to keep.
# Daily Backups are currently Needed for this Script
DC=$1
# How many Weekly copies to keep. 0 is disabled.
WC=$2
# How many Monthly copies to keep. 0 is disabled.
MC=$3

#Backup dir
BKDIR=$4
SPCE=$5
ADMIN_EMAIL="$(nodeworx -u -n --controller Users --action listMasterUser | grep email | awk '{print $2}')"

DIR="$(dirname "${BASH_SOURCE[0]}")"

source "$DIR"/iwx-backup.conf
BKSTATLOG="$DIR"/iwx-sshfs.log

date=`date +'%Y%m%d'`

[ $? -eq 1 ] && exit 1

function umountsshfs {
   #umount gracefully if possible
   umount $BKDIR  > /dev/null 2>&1

   #kill any frozen process on the mount
   fuser -k $BKDIR  > /dev/null 2>&1
   fusermount -u $BKDIR  > /dev/null 2>&1
   umount -l $BKDIR  > /dev/null 2>&1
   umount $BKDIR  > /dev/null 2>&1

   #kill last sshfs process and mount, in case it is hung
   [ -f $DIR/sshfs-pid ] && local sshfspid=$(cat $DIR/sshfs-pid)
   if [ -n ${sshfspid} ] && [ -d "/proc/${sshfspid}" ]; then
        kill -9 $sshfspid > /dev/null 2>&1
        umount "$BKDIR" > /dev/null 2>&1
   fi
   echo $(date) "Un-mounted remote SSH \ SCP volume on $BKDIR" >> $BKSTATLOG
}

function mountsshfs {
#!/bin/bash
local user=$(sed -n '/\[sshfs\]/,/\[.*\]/p' $DIR/iwx-sshfs.conf | awk -F= '/user/{print $2}' | tr -d \' | tr -d ' ')
local pass=$(sed -n '/\[sshfs\]/,/\[.*\]/p' $DIR/iwx-sshfs.conf | awk -F= '/pass/{print $2}' | tr -d \' | tr -d ' ')
local host=$(sed -n '/\[sshfs\]/,/\[.*\]/p' $DIR/iwx-sshfs.conf | awk -F= '/host/{print $2}' | tr -d \' | tr -d ' ')
local port=$(sed -n '/\[sshfs\]/,/\[.*\]/p' $DIR/iwx-sshfs.conf | awk -F= '/port/{print $2}' | tr -d \' | tr -d ' ')
local rdir=$(sed -n '/\[sshfs\]/,/\[.*\]/p' $DIR/iwx-sshfs.conf | awk -F= '/rdir/{print $2}' | tr -d \' | tr -d ' ')

local remotemounttestfile="$BKDIR/iw-sshfs-mounted-backup-folder"
[[ -d $BKDIR ]] || mkdir -p $BKDIR/
if mountpoint -q $BKDIR && [ -f $remotemounttestfile ]; then
    RC=$?
    echo $(date) "Remote SSH or SCP volume $rdir on $BKDIR Mounted and working" >> $BKSTATLOG
else
   echo "Unable to mount $rdir on $BKDIR, check connection settings" >> $BKSTATLOG
   [[ $7 == 1 ]] && umountsshfs
   #remount backup server
   sshfs -o StrictHostKeyChecking=no,reconnect,ServerAliveInterval=15,nonempty,password_stdin $user@$host:$rdir/ $BKDIR/ -p$port <<< "${pass}"
   pgrep -f "sshfs.*$rdir.*$BKDIR" > $DIR/sshfs-pid
   if mountpoint -q $BKDIR; then
        touch $remotemounttestfile
   fi
   if mountpoint -q $BKDIR && [ -f $remotemounttestfile ]; then
    echo $(date) "Mounted remote SSH or SCP volume $rdir on $BKDIR" >> $BKSTATLOG
    RC=$?
   else
   echo $(date) "Could not mount remote SSH or SCP volume $rdir on $BKDIR" >> $BKSTATLOG
   RC=1
   fi
fi
}

function prebackupprune {
    # Cleanup all old Daily backups over the set Daily Count.
    if [[ -d $BKDIR/daily/ ]] ; then
        if [[ $DC != 0 ]] ; then
            while [ $(\ls -p $BKDIR/daily/ | grep '^Daily_[0-9]\{8\}/$' | wc -l) -ge $DC ]
                do ionice -c3 rm -rf $BKDIR/daily/Daily$(\ls -p $BKDIR/daily/ | grep 'Daily_[0-9]\{8\}/$' | head -n1 | sed 's/.*_/_/')
            done
        fi
    fi
    # Cleanup all old Weekly backups over the set Weekly Count.
    if [[ -d $BKDIR/weekly/ ]] ; then
        if [[ $WC != 0 ]] ; then
            while [ $(\ls -p $BKDIR/weekly/ | grep '^Weekly_[0-9]\{8\}/$' | wc -l) -ge $WC ]
                do ionice -c3 rm -rf $BKDIR/weekly/Weekly$(\ls -p $BKDIR/weekly/ | grep 'Weekly_[0-9]\{8\}/$' | head -n1 | sed 's/.*_/_/')
            done
        fi
    fi
        if [[ -d $BKDIR/monthly/ ]] ; then
    # Cleanup all old Monthly backups over the set Monthly Count.
        if [[ $MC != 0 ]] ; then
            while [ $(\ls -p $BKDIR/monthly/ | grep '^Monthly_[0-9]\{8\}/$' | wc -l) -ge $MC ]
                do ionice -c3 rm -rf $BKDIR/monthly/Monthly$(\ls -p $BKDIR/monthly/ | grep 'Monthly_[0-9]\{8\}/$' | head -n1 | sed 's/.*_/_/')
            done
        fi
    fi
}

function daily {
    # Ensure Daily Folder Exists
    [[ -d $BKDIR/daily/ ]] || mkdir -p $BKDIR/daily/

    # Check if Daily Backups are Enabled
    if [ $DC -gt "0" ]; then
        # Create New Backup's Folder
        [[ -d $BKDIR/daily/Daily_$date ]] || mkdir $BKDIR/daily/Daily_$date
        # Create New Backup
        [[ -d /home/iwbktmp ]] || mkdir -p /home/iwbktmp
        /home/interworx/bin/backup.pex --backup-options all --domains all --tmp-dir /home/iwbktmp --output-dir $BKDIR/daily/Daily_$date/ --email $ADMIN_EMAIL > $BKDIR/daily/Daily_$date/backup.log
    else
        #Delete all previous daily backups if changed to 0.
        if [ $DC == "0" ]; then
            while [ $(\ls -p $BKDIR/daily/ | grep '^Daily_[0-9]\{8\}/$' | wc -l) -ge 1 ]
                do ionice -c3 rm -rf $BKDIR/daily/Daily$(\ls -p $BKDIR/daily/ | grep 'Daily_[0-9]\{8\}/$' | head -n1 | sed 's/.*_/_/')
            done
        fi
    fi
}

function weekly {
    # Ensure Weekly Folder Exists
    [[ -d $BKDIR/weekly/ ]] || mkdir -p $BKDIR/weekly/

    # Check if Weekly Backups are Enabled
    if [ $WC -gt "0" ]; then
        # Check if the current day is a Weekly Backup Day
        if [ $(date '+%u') -eq $DAY_WEEKLY ]; then
            # Copy Daily backup to the Weekly Folder
            if  [ $DC -gt "0" ]; then
                ionice -c3 rsync -aH --delete --link-dest=$BKDIR/daily/Daily_$date/ $BKDIR/daily/Daily_$date/ $BKDIR/weekly/Weekly_$date/
            else
                [[ -d $BKDIR/weekly/Weekly_$date ]] || mkdir $BKDIR/weekly/Weekly_$date
                /home/interworx/bin/backup.pex --backup-options all --domains all --tmp-dir /home/iwbktmp --output-dir $BKDIR/weekly/Weekly_$date/ --email $ADMIN_EMAIL > $BKDIR/weekly/Weekly_$date/backup.log
            fi
        fi
    else
        #Delete all previous weekly backups if changed to 0.
        if [ $WC == "0" ]; then
            while [ $(\ls -p $BKDIR/weekly/ | grep '^Weekly_[0-9]\{8\}/$' | wc -l) -ge 1 ]
                do ionice -c3 rm -rf $BKDIR/weekly/Weekly$(\ls -p $BKDIR/weekly/ | grep 'Weekly_[0-9]\{8\}/$' | head -n1 | sed 's/.*_/_/')
            done
        fi
    fi
}

function monthly {
    # Ensure Monthly Folder Exists
    [[ -d $BKDIR/monthly/ ]] || mkdir -p $BKDIR/monthly/

    # Check if Monthly disabled
    if [ $MC -gt "0" ]
    then
        # Check if the current day is a Monthly Backup Day
        if [ $(date '+%d') -eq $DAY_MONTHLY ]; then
            # Copy Daily backup to the Monthly Folder
            if [ $DC -gt "0" ] ; then 
                ionice -c3 rsync -aH --delete --link-dest=$BKDIR/daily/Daily_$date/ $BKDIR/daily/Daily_$date/ $BKDIR/monthly/Monthly_$date/
            else
                if [ $WC -gt "0" ] ; then
                    ionice -c3 rsync -aH --delete --link-dest=$BKDIR/weekly/Weekly_$date/ $BKDIR/weekly/Weekly_$date/ $BKDIR/monthly/Monthly_$date/
                else
                    [[ -d $BKDIR/monthly/Monthly_$date ]] || mkdir $BKDIR/monthly/Monthly_$date
                    /home/interworx/bin/backup.pex --backup-options all --domains all --tmp-dir /home/iwbktmp --output-dir $BKDIR/monthly/Monthly_$date/ --email $ADMIN_EMAIL > $BKDIR/monthly/Monthly_$date/backup.log
                fi
            fi
        fi
    else
        #Delete all previous monthly backups if changed to 0.
        if [ $MC == "0" ]; then
            while [ $(\ls -p $BKDIR/monthly/ | grep '^Monthly_[0-9]\{8\}/$' | wc -l) -ge 1 ]
                do ionice -c3 rm -rf $BKDIR/monthly/Monthly$(\ls -p $BKDIR/monthly/ | grep 'Monthly_[0-9]\{8\}/$' | head -n1 | sed 's/.*_/_/')
            done
        fi
    fi
}

if [[ -f $DIR/iw-backup-pid ]]; then
        iwbackuppid=$(cat $DIR/iw-backup-pid)
        if [ -n ${iwbackuppid} ] && [ -d "/proc/${iwbackuppid}" ]; then
                echo "Last backup still running, skipping" $(date) >> $BKSTATLOG
                exit 1
        fi
fi

echo "===================================================" >> $BKSTATLOG

if [[ $6 == 1 ]]; then
    mountsshfs
    if [[ $RC == 1 ]]; then
        exit $RC
	fi
    echo $$ > $DIR/iw-backup-pid
    echo $(date) "Starting Backups" >> $BKSTATLOG
    prebackupprune
    #check disk usage
    "$DIR"/iwx-backup-diskusage.sh $BKDIR $SPCE >> $BKSTATLOG
    if [[ $? == 1 ]]; then
        exit 1
	fi
    daily
    weekly
    monthly
    echo $(date) "Backups Completed" >> $BKSTATLOG
    [[ $7 == 1 ]] && umountsshfs
    \rm $DIR/iw-backup-pid
else
    echo $$ > $DIR/iw-backup-pid
    echo $(date) "Starting Backups" >> $BKSTATLOG
    prebackupprune
    #check disk usage
    "$DIR"/iwx-backup-diskusage.sh $BKDIR $SPCE >> $BKSTATLOG
    if [[ $? == 1 ]]; then
        exit 1
	fi
    daily
    weekly
    monthly
    echo $(date) "Backups Completed" >> $BKSTATLOG
    [[ $7 == 1 ]] && umountsshfs
    \rm $DIR/iw-backup-pid
fi
