<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2016 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 24, 2016
 */

namespace Interworx\CliCore;

/**
 * Class PathFilterTest
 *
 * @package    interworx-cli-core
 * @subpackage Tests
 */
class PathFilterTest extends \PHPUnit_Framework_TestCase
{
    /**
     * Test that an explicit value is not changed.
     */
    public function testFilterBasicValueWorks()
    {
        $Filter = new PathFilter();
        $this->assertEquals('/home', $Filter->filter('/home'));
    }

    /**
     * Test the use of a ~/ prefix.
     */
    public function testFilterExpandsMyUserDir()
    {
        $Filter = new PathFilter();

        $this->assertEquals(getenv('HOME'), $Filter->filter("~/"));
    }

    /**
     * Test the use of a ~$user/ prefix.
     */
    public function testFilterExpandsAnyUserDir()
    {
        $user   = getenv('USER');
        $Filter = new PathFilter();

        $this->assertEquals(getenv('HOME'), $Filter->filter("~{$user}/"));
    }

    /**
     * Test the use of ~$user.
     */
    public function testFilterExpandsAnyUserDirWithoutTrailingSlash()
    {
        $user   = getenv('USER');
        $Filter = new PathFilter();

        $this->assertEquals(getenv('HOME'), $Filter->filter("~{$user}"));
    }

    /**
     * Test the use of a ~$user/ prefix.
     */
    public function testFilterReturnsBullshitAsIs()
    {
        $Filter = new PathFilter();

        $this->assertEquals('/does/not/exist/', $Filter->filter("/does/not/exist/"));
    }
}
