/*jslint undef: true, widget: true, browser: true, onevar: true, eqeqeq: true */
/*global YAHOO, iw_js_data, IW_PAYLOAD_DATA, iw_xhr, iw_lib, iw_ctrl_handler, iw_form */

var iw_payload, iw_payload_widget;

iw_payload = {
    init: function () {
        var Dom = YAHOO.util.Dom,
        payloads = Dom.getElementsByClassName('iw-payload', 'div');

        this.initEventHandlers(payloads);
        this.initBrowserHistory(payloads);
        this.initGroupClickHandlers(payloads);
    },

    initBrowserHistory: function ( payloads ) {
        var i, payload_count = payloads.length, state, temp, payload, data;

        for (i=0; i<payload_count; ++i) {
            data  = IW_PAYLOAD_DATA[ payloads[i].id ];
            state = YAHOO.util.History.getBookmarkedState(payloads[i].id) || data.pgn8state;

            payload = iw_payload_widget(payloads[i]);

            this.setData(state, payload);

            state = payload.getState();

            if ( state !== YAHOO.lang.JSON.stringify(data.pgn8state) ) {
                payload.refresh();
                payload.restoreStateEvent.fire();
            }

            payload.cache[ state ] = payload.getElement().innerHTML;

            YAHOO.util.History.register(
                payloads[i].id,
                state,
                this.historyStateChangeHandler,
                payload
            );
        }//end for
    },

    parseState: function ( state ) {
        if ( YAHOO.lang.isString(state) ) {
            try {
                state = YAHOO.lang.JSON.parse(state);
            } catch ( e ) {
                state = { l: 20, o:0, or: null, d:'asc' };
            }
        }
        return state;
    },

    setData: function ( state, payload ) {
        var value;
        state = this.parseState(state);

        for (value in state) {
            if ( state.hasOwnProperty(value) ) {
                payload.setValue(value, state[ value ]);
            }
        }
    },

    historyStateChangeHandler: function ( newState, payload ) {
        if ( payload.ignoreCache() === true ) {
            payload.setIgnoreCache(false);
            return;
        }

        if ( payload.cache[ newState ] ) {
            payload.getElement().innerHTML = payload.cache[ newState ];
            payload.clearState();
            iw_payload.setData(newState, payload);
            payload.reconnectForm();
            payload.restoreStateEvent.fire();
            return;
        }

        if ( newState !== payload.getState() ) {
            iw_payload.setData(newState, payload);
            payload.refresh();
        }
    },

    initEventHandlers: function () {
        var E = YAHOO.util.Event;

        E.delegate(document.body, 'click', this.checkAll, 'input.iw-check-all');
        E.delegate(document.body, 'click', this.sort, '.iw-sort a');
        E.delegate(document.body, 'click', this.paginate, '.iw-pgn8-ctrl a');
        E.delegate(document.body, 'change', this.paginateJump, '.iw-pgn8 select');
        E.delegate(document.body, 'click', this.viewAll, 'a.iw-pgn8-all');
        E.delegate(document.body, 'click', this.titleClickSort, '.iw-payload-sortable');
        E.delegate(document.body, 'change', this.selectRow, '.iw-form-req');
        E.delegate(document.body, 'change', this.selectAllRow, '.iw-check-all');
        E.delegate(document.body, 'click', this.refresh, 'button.iw-payload-refresh');
    },

    initGroupClickHandlers: function () {
        var E = YAHOO.util.Event;

        E.delegate(document.body, 'dblclick', this.toggleGroupDisplay, 'tr.iw-payload-group-hd');
        E.delegate(document.body, 'click', this.toggleGroupDisplay, 'tr.iw-payload-group-hd img.iw-collapse');

    },

    selectRow: function ( e ) {
        var checkbox = YAHOO.util.Event.getTarget(e),
        row = YAHOO.util.Dom.getAncestorByTagName(checkbox, 'tr'),
        newclass = 'selected';

        if ( checkbox.checked ) {
            row.className += ' ' + newclass;
        } else {
            YAHOO.util.Dom.removeClass(row, newclass);
        }
    },

    selectAllRow: function ( e ) {
        var Dom = YAHOO.util.Dom,
        table = Dom.getAncestorByTagName(this, 'table'),
        checks = YAHOO.util.Selector.query('input[type=checkbox]', table),
        check_count = checks.length,
        checkbox = YAHOO.util.Event.getTarget(e),
        newclass = 'selected',
        row,
        i;

        for (i=0; i<check_count; ++i) {
            row = Dom.getAncestorByTagName(checks[i], 'tr');
            if ( checks[i].disabled ) {
                continue;
            }

            if ( checkbox.checked && checkbox !== checks[i] ) {
                row.className += ' ' + newclass;
            } else {
                YAHOO.util.Dom.removeClass(row, newclass);
            }
        }
    },

    viewAll: function ( e ) {
        var payload = iw_payload._getPayload(e);
        YAHOO.util.Event.stopEvent(e);

        payload.setLimit(0);
        payload.setOffset(0);
        payload.go();
    },

    paginate: function ( e ) {
        var payload = iw_payload._getPayload(e),
        img = this.getElementsByTagName('img')[0];

        YAHOO.util.Event.stopEvent(e);

        if ( img.className.indexOf('-off') !== -1 ) {
            return;
        }

        payload.setOffset(this.className);

        if ( payload.getLimit() === 0 ) {
            payload.setLimit(payload.initialLimit);
        }

        payload.go();
    },

    _getPayload: function ( e ) {
        var target = YAHOO.util.Event.getTarget(e),
        shell = YAHOO.util.Dom.getAncestorByClassName(target, 'iw-payload');

        return iw_payload_widget(shell);
    },

    paginateJump: function ( e ) {
        var payload = iw_payload._getPayload(e);
        YAHOO.util.Event.stopEvent(e);

        payload.setOffset(this.options[ this.selectedIndex ].value);
        payload.go();
    },

    toggleGroupDisplay: function ( e ) {
        var E = YAHOO.util.Event,
        D = YAHOO.util.Dom,
        S = YAHOO.util.Selector,
        target = D.getAncestorByTagName(E.getTarget(e), 'tr'),
        parent,
        group_num,
        children,
        child,
        img;

        parent = D.getAncestorByTagName(target, 'table');
        group_num = D.getAttribute(target, 'data-iw-group-num');
        children = S.query('tr.group-num-' + group_num, parent);
        img = S.query('img.iw-collapse', target)[0];

        if ( D.hasClass(img, 'iw-i-minus') ) {
            iw_lib.updateIWIcon(img, 'iw-i-plus');
        } else {
            iw_lib.updateIWIcon(img, 'iw-i-minus');
        }

        for (child in children) {
            if ( children.hasOwnProperty(child) ) {
                child = children[ child ];
                if ( D.hasClass(child, 'iw-hide') ) {
                    D.removeClass(child, 'iw-hide');
                } else {
                    D.addClass(child, 'iw-hide');
                }
            }//end if has own property
        }//end for
    },

    titleClickSort: function ( e ) {
        var payload = iw_payload._getPayload(e),
        target = YAHOO.util.Event.getTarget(e),
        Dom = YAHOO.util.Dom,
        direction = 'asc',
        img,
        td,
        a;

        if ( target.tagName.toLowerCase() === 'img' ) {
            return;
        }

        if ( Dom.hasClass(target, 'quickhelp') ) {
            return;
        }

        td = Dom.getElementsByClassName('sorted', 'td', this);
        a  = this.getElementsByTagName('a')[0];

        if ( YAHOO.lang.isUndefined(td[0]) === false ) {
            td  = td[0];
            img = td.getElementsByTagName('img')[0];

            if ( Dom.hasClass(img, 'up') ) {
                direction = 'desc';
            }
        }

        payload.setOrder(a.className, direction);
        payload.setOffset(0);
        payload.go();
    },

    refresh: function ( e ) {
        var payload = iw_payload._getPayload(e);
        YAHOO.util.Event.stopEvent(e);
        payload.refresh();
    },

    sort: function ( e ) {
        var payload = iw_payload._getPayload(e),
        img = this.getElementsByTagName('img')[0];
        YAHOO.util.Event.stopEvent(e);

        payload.setOrder(this.className, ( img.className === 'up' ) ? 'asc' : 'desc');
        payload.setOffset(0);
        payload.go();
    },

    checkAll: function ( e ) {
        var Dom = YAHOO.util.Dom,
        table = Dom.getAncestorByTagName(this, 'table'),
        checks = YAHOO.util.Selector.query('input[type=checkbox]', table),
        check_count = checks.length,
        i;

        for (i=0; i<check_count; ++i) {
            if ( checks[i].disabled ) {
                continue;
            }
            checks[i].checked = this.checked;
        }
    },

    markPayloadInProgress: function ( eventType, args ) {
        if ( args[1] === false ) {
            iw_lib.markRefreshInProgress(args[0]);
        }
    },

    clearPayloadInProgress: function ( eventType, args ) {
        iw_lib.clearRefreshInProgress(args[0]);
    },

    widgets: {}
};

iw_payload_widget = function ( id_or_el ) {
    var el, form, input, _callback, _refreshCompleteEvent, _refreshBeginEvent,
      _getForm, timeout, _isRefreshed, _stateInput = {}, _getState, _ignoreCache,
      _refresh, _refreshAfterPeriod, _startAutoRefresh, _cache, _reconnectForm,
      _refreshOn, _initRefreshOn, _reconnectEl, _isInPopup;

    el = iw_lib.getEl(id_or_el);

    _ignoreCache = false;

    if ( iw_payload.widgets[ el.id ] ) {
        // if cached, return the cached instance
        return iw_payload.widgets[ el.id ];
    }

    _getForm = function () {
        var my_form = YAHOO.util.Dom.getElementsByClassName('iw-form', 'form', el);
        return my_form[0];
    };

    form = _getForm();

    if ( !IW_PAYLOAD_DATA[ el.id ] ) {
        IW_PAYLOAD_DATA[ el.id ] = {};
    }

    input                   = IW_PAYLOAD_DATA[ el.id ];
    input.iw_refresh_action = form.iw_refresh_action.value;
    input.iw_refresh_ctrl   = form.iw_refresh_ctrl.value;

    if (form.iw_cache_key) {
        input.iw_cache_key = form.iw_cache_key.value;
    }

    input.security_token    = iw_js_data.iw_env.security_token;

    _refreshBeginEvent    = new YAHOO.util.CustomEvent('refreshBegin');
    _refreshCompleteEvent = new YAHOO.util.CustomEvent('refreshComplete');

    _refreshBeginEvent.subscribe(iw_payload.markPayloadInProgress);
    _refreshCompleteEvent.subscribe(iw_payload.clearPayloadInProgress);

    _cache       = {};

    _callback = function ( reply, success ) {
        var data = reply.getReplyData();
        YAHOO.util.Dom.removeClass('doc3', 'iw-thinking');
        if ( success === true ) {
            el.innerHTML = data.payload;
            _reconnectEl();
            _reconnectForm();
            _refreshCompleteEvent.fire(el);
            _cache[ _getState() ] = data.payload;
        }
    };

    _reconnectEl = function () {
        el = document.getElementById(el.id);
    };

    _reconnectForm = function () {
        // Reconnect the form element
        form = _getForm();
        iw_form.initFormEvents(form);
    };

    _isRefreshed = function () {
        var period = parseInt(form.iw_refresh_period.value, 10);
        return period > 0;
    };

    _refresh = function ( silent ) {
        silent = (silent === true);
        _refreshBeginEvent.fire(el, silent);
        input.pgn8state = _stateInput;
        iw_ctrl_handler.execute(null, _callback, 'index', 'livePayloadCommit', input);
        YAHOO.util.Dom.addClass('doc3', 'iw-thinking');
    };

    _refreshAfterPeriod = function () {
        var period = form.iw_refresh_period.value * 1000;
        timeout = window.setTimeout(_refresh, period);
    };

    _startAutoRefresh = function () {
        if ( _isRefreshed() ) {
            _refreshCompleteEvent.subscribe(_refreshAfterPeriod);
            _refreshAfterPeriod();
        }
    };

    _getState = function () {
        return YAHOO.lang.JSON.stringify(_stateInput);
    };

    _refreshOn = function ( action, ctrl ) {
        if ( ctrl === null ) {
            ctrl = form.iw_ctrl_name.value;
        }

        iw_xhr.subscribeTo(_refresh, action, ctrl);
    };

    _initRefreshOn = function () {
        var i, pair, refresh_now;

        for (i in input.refresh_on) {
            if ( input.refresh_on.hasOwnProperty(i) ) {
                pair = input.refresh_on[i];
                if (pair[0] === '_load_') {
                    refresh_now = true;
                    continue;
                }
                _refreshOn(pair[0], pair[1]);
            }
        }

        if (refresh_now) {
            _refresh();
        }
    };

    _isInPopup = function () {
        var panel = YAHOO.util.Dom.getAncestorByClassName(el, 'yui-panel', 'div');

        return (panel) ? true : false;
    };

    _initRefreshOn();
    _startAutoRefresh();

    iw_payload.widgets[ el.id ] = {
        getElement: function () {
            return el;
        },
        getForm: function () {
            return form;
        },
        getLimit: function () {
            return parseInt(_stateInput.l, 10);
        },
        setLimit: function ( integer ) {
            _stateInput.l = parseInt(integer, 10);
        },
        getOffset: function () {
            return parseInt(_stateInput.o, 10);
        },
        setOffset: function ( integer ) {
            _stateInput.o = parseInt(integer, 10);
        },
        getOrder: function () {
            return _stateInput.or;
        },
        setOrder: function ( order, direction ) {
            _stateInput.or = order;
            if ( direction ) {
                _stateInput.d = direction;
            }
        },

        callback: _callback,
        isInPopup: _isInPopup,

        clearState: function () {
            _stateInput = {};
        },
        getDirection: function () {
            return _stateInput.d || 'asc';
        },
        getState: _getState,
        setValue: function ( name, value ) {
            _stateInput[ name ] = value;
        },
        getValue: function ( name ) {
            if ( YAHOO.lang.isUndefined(_stateInput[ name ]) === false ) {
                return _stateInput[ name ];
            }
            return null;
        },
        reconnectForm: _reconnectForm,
        refreshOn: _refreshOn,
        refreshOnFailure: function ( action, ctrl ) {
            iw_xhr.subscribeTo(_refresh, action, ctrl, 'failure');
        },
        startAutoRefresh: _startAutoRefresh,
        stopAutoRefresh: function () {
            if ( timeout ) {
                window.clearTimeout(timeout);
                _refreshCompleteEvent.unsubscribe(_refreshAfterPeriod);
            }
        },
        /**
         * "go()" is used when the payload will be returning new results, ie pagination.
         *
         * Go adds history and records a navigable state conditionally - if the payload
         * in question is in the popup, then it won't, because that's a bad idea.
         */
        go: function () {
            _refresh();
            _ignoreCache = true;

            if ( _isInPopup() === false ) {
                YAHOO.util.History.navigate(el.id, _getState());
            }
        },
        /**
         * "refresh()" is used update the payload when content changes, ie "add" or "edit"
         *
         * Refresh DOES NOT add history
         */
        refresh: function ( silent ) {
            silent = (silent === true);
            _refresh(silent);
        },
        refreshCompleteEvent: _refreshCompleteEvent,
        refreshBeginEvent: _refreshBeginEvent,
        restoreStateEvent: new YAHOO.util.CustomEvent('restoreState'),
        id: el.id,
        cache: _cache,
        ignoreCache: function () {
            return _ignoreCache; },
        setIgnoreCache: function ( bool ) {
            _ignoreCache = bool; },
        initialLimit: _stateInput.l
    };

    return iw_payload.widgets[ el.id ];
};


iw_payload.init();
YAHOO.util.History.initialize('yui-history-field', 'yui-history-iframe');
