<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jon Daniel <jdaniel@interworx.com>
 * @date    Feb 21, 2011
 */

/**
 * Factory for session and action history.
 *
 * @package    InterWorx
 * @subpackage Payload
 */
class Payload_Factory_NW_History
{

    /**
     * Changes payload.
     *
     * @param array $options
     * @return Payload
     */
    public static function changes(array $options = null)
    {
        $Payload = new Payload();

        $Payload->setTitle('##LG_HISTORY_CHANGES##');
        $Payload->setName('history_changes');
        $Payload->setDefaultOrder('timestamp', 'desc');

        $Search = new IWorx_Search_History_Changes();

        if (!empty($options)) {
            $Search->setOptions($options);
        }

        $Search->addSelectableFields(array( 'action_id',
                                         'ctrl_name',
                                         'action_name',
                                         'query_string',
                                         'input_name',
                                         'input_label',
                                         'initial_value',
                                         'current_value',
                                         'working_user',
                                         'working_domain',
                                         'timestamp',
                                         'env' ));

        $Grouper = new IWorx_Formatter_Grouper_History_Action('action_id');
        $Payload->setGrouper($Grouper);

        Payload_Factory::buildSearchPayload($Search, $Payload);

        $Payload->setColumn(new Payload_Column('working_user'))
        ->setLabel('##LG_AS_USER##');

        $Payload->setColumn(new Payload_Column('working_domain'))
        ->addFormatter(new IWorx_Formatter_NoneIfEmpty());

        $Payload->setColumn(new Payload_Column('change'))
        ->addFormatter(new IWorx_Formatter_History_Change())
        ->setField('input_name')
        ->setLabel('##LG_CHANGES##');

        $Payload->setColumn(new Payload_Column('timestamp'))
        ->addFormatter(new IWorx_Formatter_Datetime());

        return $Payload;
    }

    /**
     * Sessions payload.
     *
     * @param array $options
     * @return Payload
     */
    public static function sessions(array $options = null)
    {
        $Payload = new Payload();

        $Payload->setTitle('##LG_HISTORY_SESSIONS##');
        $Payload->setName('history_sessions');
        $Payload->setDefaultOrder('last_action_timestamp', 'desc');

        $Search = new IWorx_Search_History_Sessions();

        if (!empty($options)) {
            $Search->setOptions($options);
        }

        $Search->groupBy('session_id');
        $Search->addSelectableFields(array( 'session_id',
                                         'remote_ipaddr',
                                         'action_count',
                                         'session_start',
                                         'last_action',
                                         'last_ctrl',
                                         'last_action_timestamp',
                                         'env' ));

        Payload_Factory::buildSearchPayload($Search, $Payload);

        $Payload->setColumn(new Payload_Column('action'))
        ->addFormatter(self::_getSessionsActionFormatter())
        ->setField('session_id');

        $Payload->setColumn(new Payload_Column('action_count'))
        ->setIsSortable(true)
        ->setLabel('##LG_ACTIONS##');

        $Payload->setColumn(new Payload_Column('remote_ipaddr'))
        ->setIsSortable(true);

        $Payload->setColumn(new Payload_Column('session_start'))
        ->setIsSortable(true)
        ->addFormatter(new IWorx_Formatter_FuzzyDatetime());

        $Payload->setColumn(new Payload_Column('last_ctrl'))
        ->setIsSortable(true)
        ->addFormatter(new IWorx_Formatter_CtrlActionDocumentation('last_action'))
        ->setLabel('##LG_LAST_ACTION##');

        $Payload->setColumn(new Payload_Column('last_action_timestamp'))
        ->setIsSortable(true)
        ->addFormatter(new IWorx_Formatter_FuzzyDatetime())
        ->setLabel('##LG_LAST_ACTION_TIME##');

        $Payload->setColumn(new Payload_Column('env'))
        ->setIsSortable(true)
        ->setLabel('##LG_ENVIRONMENT##');

        return $Payload;
    }

    /**
     * Users payload.
     *
     * @param array $options
     * @return Payload
     */
    public static function users(array $options = null)
    {
        $Payload = new Payload();

        $Payload->setTitle('##LG_HISTORY_USERS##');
        $Payload->setName('history_users');
        $Payload->setDefaultOrder('user_type, last_action_timestamp', 'desc');

        $Search = new IWorx_Search_History_Sessions();

        if (!empty($options)) {
            $Search->setOptions($options);
        }

        $Search->groupBy('global_uid');
        $Search->addSelectableFields(array( 'global_uid',
                                         'email',
                                         'nickname',
                                         'user_type',
                                         'session_count',
                                         'action_count',
                                         'last_action_timestamp' ));

        $Grouper = new IWorx_Formatter_Grouper_LoginType('user_type');
        $Payload->setGrouper($Grouper);

        Payload_Factory::buildSearchPayload($Search, $Payload);

        $Payload->setColumn(new Payload_Column('action'))
        ->addFormatter(self::_getUsersActionFormatter())
        ->setField('email');

        $Payload->setColumn(new Payload_Column('nickname'))
        ->setIsSortable(true)
        ->addFormatter(new IWorx_Formatter_NoneIfEmpty());

        $Payload->setColumn(new Payload_Column('email'))
        ->setIsSortable(true);

        $Payload->setColumn(new Payload_Column('session_count'))
        ->setIsSortable(true)
        ->setLabel('##LG_SESSIONS##');

        $Payload->setColumn(new Payload_Column('action_count'))
        ->setIsSortable(true)
        ->setLabel('##LG_ACTIONS##');

        $Payload->setColumn(new Payload_Column('last_action_timestamp'))
        ->setIsSortable(true)
        ->addFormatter(new IWorx_Formatter_FuzzyDatetime())
        ->setLabel('##LG_LAST_ACTION_TIME##');

        return $Payload;
    }

    /**
     * Search payload.
     *
     * @param array $options
     * @return Payload
     */
    public static function search(array $options = null)
    {
        $Payload = new Payload();

        $Payload->setTitle('##LG_HISTORY_SEARCH##');
        $Payload->setName('history_search');
        $Payload->setDefaultOrder('timestamp', 'desc');

        $Search = new IWorx_Search_History_Changes();

        if (!empty($options)) {
            $Search->setOptions($options);
        }

        $Search->addSelectableFields(array( 'session_global_uid',
                                         'ctrl_name',
                                         'session_id',
                                         'action_name',
                                         'query_string',
                                         'input_name',
                                         'input_label',
                                         'initial_value',
                                         'current_value',
                                         'working_user',
                                         'working_domain',
                                         'timestamp',
                                         'env' ));

        $Grouper = new IWorx_Formatter_Grouper_User('session_global_uid');
        $Payload->setGrouper($Grouper);

        Payload_Factory::buildSearchPayload($Search, $Payload);

        $Payload->setColumn(new Payload_Column('actions'))
        ->addFormatter(self::_getSearchActionFormatter())
        ->setField('session_id')
        ->setLabel('');

        $Payload->setColumn(new Payload_Column('action'))
        ->addFormatter(new IWorx_Formatter_CtrlActionDocumentation('action_name'))
        ->setField('ctrl_name');

        $Payload->setColumn(new Payload_Column('parameters'))
        ->setField('query_string')
        ->addFormatter(new IWorx_Formatter_History_QueryString());

        $Payload->setColumn(new Payload_Column('change'))
        ->addFormatter(new IWorx_Formatter_History_Change())
        ->setField('input_name')
        ->setLabel('##LG_CHANGES##');

        $Payload->setColumn(new Payload_Column('working_user'))
        ->setLabel('##LG_AS_USER##');

        $Payload->setColumn(new Payload_Column('working_domain'))
        ->addFormatter(new IWorx_Formatter_NoneIfEmpty())
        ->setIsSortable(true);

        $Payload->setColumn(new Payload_Column('timestamp'))
        ->addFormatter(new IWorx_Formatter_FuzzyDatetime())
        ->setIsSortable(true);

        $Payload->setColumn(new Payload_Column('env'))
        ->setIsSortable(true)
        ->setLabel('##LG_ENVIRONMENT##');

        return $Payload;
    }

    /**
     * Get the user payload's session formatter.
     *
     * @return IWorx_Formatter_Action
     */
    private static function _getSearchActionFormatter()
    {
        $action_data = array( array( 'ctrl'    => 'Ctrl_Nodeworx_HistoryChanges',
                                 'action'  => 'index',
                                 'query'   => 'session=%%session_id%%',
                                 'display' => '##LG_VIEW_DETAILS##',
                                 'handled' => false ) );

        return IWorx_Formatter_Action::factory($action_data);
    }

    /**
     * Get the user payload's session formatter.
     *
     * @return IWorx_Formatter_Action
     */
    private static function _getSessionsActionFormatter()
    {
        $action_data = array( array( 'ctrl'    => 'Ctrl_Nodeworx_HistoryChanges',
                                 'action'  => 'index',
                                 'query'   => 'session=%%session_id%%',
                                 'display' => '##LG_VIEW_DETAILS##',
                                 'handled' => false ) );

        return IWorx_Formatter_Action::factory($action_data);
    }

    /**
     * Get the user payload's action formatter.
     *
     * @return IWorx_Formatter_Action
     */
    private static function _getUsersActionFormatter()
    {
        $action_data = array( array( 'ctrl'    => 'Ctrl_Nodeworx_HistorySessions',
                                 'action'  => 'index',
                                 'query'   => 'user=%%global_uid%%',
                                 'display' => '##LG_VIEW_SESSIONS##',
                                 'handled' => false ) );

        return IWorx_Formatter_Action::factory($action_data);
    }
}
